@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-* Copyright (C) 1999-2001, International Business Machines Corporation and   *
+* Copyright (C) 1999-2004, International Business Machines Corporation and   *
 * others. All Rights Reserved.                                               *
 ******************************************************************************
 *   Date        Name        Description
@@ -15,6 +15,16 @@ U_NAMESPACE_BEGIN
 
 #define DEFUALT_CAPACITY 8
 
+/*
+ * Constants for hinting whether a key is an integer
+ * or a pointer.  If a hint bit is zero, then the associated
+ * token is assumed to be an integer. This is needed for iSeries
+ */
+#define HINT_KEY_POINTER   (1)
+#define HINT_KEY_INTEGER   (0)
+ 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(UVector)
+
 UVector::UVector(UErrorCode &status) :
     count(0),
     capacity(0),
@@ -35,7 +45,7 @@ UVector::UVector(int32_t initialCapacity, UErrorCode &status) :
     _init(initialCapacity, status);
 }
 
-UVector::UVector(UObjectDeleter d, UKeyComparator c, UErrorCode &status) :
+UVector::UVector(UObjectDeleter *d, UKeyComparator *c, UErrorCode &status) :
     count(0),
     capacity(0),
     elements(0),
@@ -45,7 +55,7 @@ UVector::UVector(UObjectDeleter d, UKeyComparator c, UErrorCode &status) :
     _init(DEFUALT_CAPACITY, status);
 }
 
-UVector::UVector(UObjectDeleter d, UKeyComparator c, int32_t initialCapacity, UErrorCode &status) :
+UVector::UVector(UObjectDeleter *d, UKeyComparator *c, int32_t initialCapacity, UErrorCode &status) :
     count(0),
     capacity(0),
     elements(0),
@@ -56,6 +66,9 @@ UVector::UVector(UObjectDeleter d, UKeyComparator c, int32_t initialCapacity, UE
 }
 
 void UVector::_init(int32_t initialCapacity, UErrorCode &status) {
+    if (U_FAILURE(status)) {
+        return;
+    }
     // Fix bogus initialCapacity values; avoid malloc(0)
     if (initialCapacity < 1) {
         initialCapacity = DEFUALT_CAPACITY;
@@ -74,6 +87,37 @@ UVector::~UVector() {
     elements = 0;
 }
 
+/**
+ * Assign this object to another (make this a copy of 'other').
+ * Use the 'assign' function to assign each element.
+ */
+void UVector::assign(const UVector& other, UTokenAssigner *assign, UErrorCode &ec) {
+    if (ensureCapacity(other.count, ec)) {
+        setSize(other.count);
+        for (int32_t i=0; i<other.count; ++i) {
+            if (elements[i].pointer != 0 && deleter != 0) {
+                (*deleter)(elements[i].pointer);
+            }
+            (*assign)(&elements[i], &other.elements[i]);
+        }
+    }
+}
+
+// This only does something sensible if this object has a non-null comparer
+UBool UVector::operator==(const UVector& other) {
+    int32_t i;
+    if (count != other.count) return FALSE;
+    if (comparer != NULL) {
+        // Compare using this object's comparer
+        for (i=0; i<count; ++i) {
+            if (!(*comparer)(elements[i], other.elements[i])) {
+                return FALSE;
+            }
+        }
+    }
+    return TRUE;
+}
+
 void UVector::addElement(void* obj, UErrorCode &status) {
     if (ensureCapacity(count + 1, status)) {
         elements[count++].pointer = obj;
@@ -82,7 +126,9 @@ void UVector::addElement(void* obj, UErrorCode &status) {
 
 void UVector::addElement(int32_t elem, UErrorCode &status) {
     if (ensureCapacity(count + 1, status)) {
-        elements[count++].integer = elem;
+        elements[count].pointer = NULL;     // Pointers may be bigger than ints.
+        elements[count].integer = elem;
+        count++;
     }
 }
 
@@ -99,8 +145,10 @@ void UVector::setElementAt(void* obj, int32_t index) {
 void UVector::setElementAt(int32_t elem, int32_t index) {
     if (0 <= index && index < count) {
         if (elements[index].pointer != 0 && deleter != 0) {
+            // TODO:  this should be an error.  mixing up ints and pointers.
             (*deleter)(elements[index].pointer);
         }
+        elements[index].pointer = NULL;
         elements[index].integer = elem;
     }
     /* else index out of range */
@@ -118,6 +166,19 @@ void UVector::insertElementAt(void* obj, int32_t index, UErrorCode &status) {
     /* else index out of range */
 }
 
+void UVector::insertElementAt(int32_t elem, int32_t index, UErrorCode &status) {
+    // must have 0 <= index <= count
+    if (0 <= index && index <= count && ensureCapacity(count + 1, status)) {
+        for (int32_t i=count; i>index; --i) {
+            elements[i] = elements[i-1];
+        }
+        elements[index].pointer = NULL;
+        elements[index].integer = elem;
+        ++count;
+    }
+    /* else index out of range */
+}
+
 void* UVector::elementAt(int32_t index) const {
     return (0 <= index && index < count) ? elements[index].pointer : 0;
 }
@@ -126,6 +187,48 @@ int32_t UVector::elementAti(int32_t index) const {
     return (0 <= index && index < count) ? elements[index].integer : 0;
 }
 
+UBool UVector::containsAll(const UVector& other) const {
+    for (int32_t i=0; i<other.size(); ++i) {
+        if (indexOf(other.elements[i]) < 0) {
+            return FALSE;
+        }
+    }
+    return TRUE;
+}
+
+UBool UVector::containsNone(const UVector& other) const {
+    for (int32_t i=0; i<other.size(); ++i) {
+        if (indexOf(other.elements[i]) >= 0) {
+            return FALSE;
+        }
+    }
+    return TRUE;
+}
+
+UBool UVector::removeAll(const UVector& other) {
+    UBool changed = FALSE;
+    for (int32_t i=0; i<other.size(); ++i) {
+        int32_t j = indexOf(other.elements[i]);
+        if (j >= 0) {
+            removeElementAt(j);
+            changed = TRUE;
+        }
+    }
+    return changed;
+}
+
+UBool UVector::retainAll(const UVector& other) {
+    UBool changed = FALSE;
+    for (int32_t j=size()-1; j>=0; --j) {
+        int32_t i = other.indexOf(elements[j]);
+        if (i < 0) {
+            removeElementAt(j);
+            changed = TRUE;
+        }
+    }
+    return changed;
+}
+
 void UVector::removeElementAt(int32_t index) {
     void* e = orphanElementAt(index);
     if (e != 0 && deleter != 0) {
@@ -153,26 +256,66 @@ void UVector::removeAllElements(void) {
     count = 0;
 }
 
-int32_t UVector::indexOf(void* obj, int32_t startIndex) const {
-    if (comparer != 0) {
+UBool   UVector::equals(const UVector &other) const {
+    int      i;
+
+    if (this->count != other.count) {
+        return FALSE;
+    }
+    if (comparer == 0) {
+        for (i=0; i<count; i++) {
+            if (elements[i].pointer != other.elements[i].pointer) {
+                return FALSE;
+            }
+        }
+    } else {
         UHashTok key;
-        key.pointer = obj;
-        for (int32_t i=startIndex; i<count; ++i) {
+        for (i=0; i<count; i++) {
+            key.pointer = &other.elements[i];
+            if (!(*comparer)(key, elements[i])) {
+                return FALSE;
+            }
+        }
+    }
+    return TRUE;
+}
+
+
+
+int32_t UVector::indexOf(void* obj, int32_t startIndex) const {
+    UHashTok key;
+    key.pointer = obj;
+    return indexOf(key, startIndex, HINT_KEY_POINTER);
+}
+
+int32_t UVector::indexOf(int32_t obj, int32_t startIndex) const {
+    UHashTok key;
+    key.integer = obj;
+    return indexOf(key, startIndex, HINT_KEY_INTEGER);
+}
+
+// This only works if this object has a non-null comparer
+int32_t UVector::indexOf(UHashTok key, int32_t startIndex, int8_t hint) const {
+    int32_t i;
+    if (comparer != 0) {
+        for (i=startIndex; i<count; ++i) {
             if ((*comparer)(key, elements[i])) {
                 return i;
             }
         }
-    }
-    return -1;
-}
-
-int32_t UVector::indexOf(int32_t obj, int32_t startIndex) const {
-    if (comparer != 0) {
-        UHashTok key;
-        key.integer = obj;
-        for (int32_t i=startIndex; i<count; ++i) {
-            if ((*comparer)(key, elements[i])) {
-                return i;
+    } else {
+        for (i=startIndex; i<count; ++i) {
+            /* Pointers are not always the same size as ints so to perform
+             * a valid comparision we need to know whether we are being
+             * provided an int or a pointer. */
+            if (hint & HINT_KEY_POINTER) {
+                if (key.pointer == elements[i].pointer) {
+                    return i;
+                }
+            } else {
+                if (key.integer == elements[i].integer) {
+                    return i;
+                }
             }
         }
     }
@@ -242,14 +385,14 @@ void** UVector::toArray(void** result) const {
     return result;
 }
 
-UObjectDeleter UVector::setDeleter(UObjectDeleter d) {
-    UObjectDeleter old = deleter;
+UObjectDeleter *UVector::setDeleter(UObjectDeleter *d) {
+    UObjectDeleter *old = deleter;
     deleter = d;
     return old;
 }
 
-UKeyComparator UVector::setComparer(UKeyComparator d) {
-    UKeyComparator old = comparer;
+UKeyComparator *UVector::setComparer(UKeyComparator *d) {
+    UKeyComparator *old = comparer;
     comparer = d;
     return old;
 }
@@ -276,49 +419,53 @@ void* UVector::orphanElementAt(int32_t index) {
     return e;
 }
 
-UStack::UStack(UErrorCode &status) :
-    UVector(status)
-{
+/**
+ * Insert the given object into this vector at its sorted position
+ * as defined by 'compare'.  The current elements are assumed to
+ * be sorted already.
+ */
+void UVector::sortedInsert(void* obj, USortComparator *compare, UErrorCode& ec) {
+    UHashTok tok;
+    tok.pointer = obj;
+    sortedInsert(tok, compare, ec);
 }
 
-UStack::UStack(int32_t initialCapacity, UErrorCode &status) :
-    UVector(initialCapacity, status)
-{
+/**
+ * Insert the given integer into this vector at its sorted position
+ * as defined by 'compare'.  The current elements are assumed to
+ * be sorted already.
+ */
+void UVector::sortedInsert(int32_t obj, USortComparator *compare, UErrorCode& ec) {
+    UHashTok tok;
+    tok.integer = obj;
+    sortedInsert(tok, compare, ec);
 }
 
-UStack::UStack(UObjectDeleter d, UKeyComparator c, UErrorCode &status) :
-    UVector(d, c, status)
-{
-}
-
-UStack::UStack(UObjectDeleter d, UKeyComparator c, int32_t initialCapacity, UErrorCode &status) :
-    UVector(d, c, initialCapacity, status)
-{
-}
-
-void* UStack::pop(void) {
-    int32_t n = size() - 1;
-    void* result = 0;
-    if (n >= 0) {
-        result = elementAt(n);
-        removeElementAt(n);
+// ASSUME elements[] IS CURRENTLY SORTED
+void UVector::sortedInsert(UHashTok tok, USortComparator *compare, UErrorCode& ec) {
+    // Perform a binary search for the location to insert tok at.  Tok
+    // will be inserted between two elements a and b such that a <=
+    // tok && tok < b, where there is a 'virtual' elements[-1] always
+    // less than tok and a 'virtual' elements[count] always greater
+    // than tok.
+    int32_t min = 0, max = count;
+    while (min != max) {
+        int32_t probe = (min + max) / 2;
+        int8_t c = (*compare)(elements[probe], tok);
+        if (c > 0) {
+            max = probe;
+        } else {
+            // assert(c <= 0);
+            min = probe + 1;
+        }
     }
-    return result;
-}
-
-int32_t UStack::popi(void) {
-    int32_t n = size() - 1;
-    int32_t result = 0;
-    if (n >= 0) {
-        result = elementAti(n);
-        removeElementAt(n);
+    if (ensureCapacity(count + 1, ec)) {
+        for (int32_t i=count; i>min; --i) {
+            elements[i] = elements[i-1];
+        }
+        elements[min] = tok;
+        ++count;
     }
-    return result;
-}
-
-int32_t UStack::search(void* obj) const {
-    int32_t i = indexOf(obj);
-    return (i >= 0) ? size() - i : i;
 }
 
 U_NAMESPACE_END
