@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (c) 2001-2002, International Business Machines
+*   Copyright (c) 2001-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -10,6 +10,10 @@
 #ifndef ESCTRN_H
 #define ESCTRN_H
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/translit.h"
 
 U_NAMESPACE_BEGIN
@@ -111,27 +115,28 @@ class U_I18N_API EscapeTransliterator : public Transliterator {
      */
     virtual Transliterator* clone() const;
 
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
+
  protected:
 
     /**
      * Implements {@link Transliterator#handleTransliterate}.
      */
-    void handleTransliterate(Replaceable& text, UTransPosition& offset,
+    virtual void handleTransliterate(Replaceable& text, UTransPosition& offset,
                              UBool isIncremental) const;
 
- private:
-
-    /**
-     * Factory methods
-     */
-    static Transliterator* _createUnicode(const UnicodeString& ID, Token context);
-    static Transliterator* _createJava(const UnicodeString& ID, Token context);
-    static Transliterator* _createC(const UnicodeString& ID, Token context);
-    static Transliterator* _createXML(const UnicodeString& ID, Token context);
-    static Transliterator* _createXML10(const UnicodeString& ID, Token context);
-    static Transliterator* _createPerl(const UnicodeString& ID, Token context);
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 #endif
