@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ******************************************************************************
 *   file name:  nfsubs.h
@@ -16,7 +16,10 @@
 #ifndef NFSUBS_H
 #define NFSUBS_H
 
+#include "unicode/utypes.h"
+#include "unicode/uobject.h"
 #include "nfrule.h"
+
 #if U_HAVE_RBNF
 
 #include "unicode/utypes.h"
@@ -26,7 +29,7 @@
 
 U_NAMESPACE_BEGIN
 
-class NFSubstitution {
+class NFSubstitution : public UObject {
     int32_t pos;
     const NFRuleSet* ruleSet;
     const DecimalFormat* numberFormat;
@@ -38,7 +41,16 @@ protected:
         const UnicodeString& description,
         UErrorCode& status);
     
+    /**
+     * Get the Ruleset of the object.
+     * @return the Ruleset of the object.
+     */
     const NFRuleSet* getRuleSet() const { return ruleSet; }
+
+    /**
+     * get the NumberFormat of this object.
+     * @return the numberformat of this object.
+     */
     const DecimalFormat* getNumberFormat() const { return numberFormat; }
     
 public:
@@ -50,9 +62,25 @@ public:
         const UnicodeString& description,
         UErrorCode& status);
     
+    /**
+     * Destructor.
+     */
     virtual ~NFSubstitution();
     
+    /**
+     * Return true if the given Format objects are semantically equal.
+     * Objects of different subclasses are considered unequal.
+     * @param rhs    the object to be compared with.
+     * @return       true if the given Format objects are semantically equal.
+     */
     virtual UBool operator==(const NFSubstitution& rhs) const;
+
+    /**
+     * Return true if the given Format objects are semantically unequal.
+     * Objects of different subclasses are considered unequal.
+     * @param rhs    the object to be compared with.
+     * @return       true if the given Format objects are semantically unequal.
+     */
     UBool operator!=(const NFSubstitution& rhs) const { return !operator==(rhs); }
     
     /**
@@ -62,10 +90,11 @@ public:
      * @param radix The radix of the divisor
      * @param exponent The exponent of the divisor
      */
-    virtual void setDivisor(int32_t radix, int32_t exponent);
+    virtual void setDivisor(int32_t radix, int32_t exponent, UErrorCode& status);
     
     /**
      * Replaces result with the string describing the substitution.
+     * @param result    Output param which will receive the string.
      */
     virtual void toString(UnicodeString& result) const;
     
@@ -84,6 +113,17 @@ public:
      * position to determine exactly where to insert the new text)
      */
     virtual void doSubstitution(int64_t number, UnicodeString& toInsertInto, int32_t pos) const;
+
+    /**
+     * Performs a mathematical operation on the number, formats it using
+     * either ruleSet or decimalFormat, and inserts the result into
+     * toInsertInto.
+     * @param number The number being formatted.
+     * @param toInsertInto The string we insert the result into
+     * @param pos The position in toInsertInto where the owning rule's
+     * rule text begins (this value is added to this substitution's
+     * position to determine exactly where to insert the new text)
+     */
     virtual void doSubstitution(double number, UnicodeString& toInsertInto, int32_t pos) const;
     
 protected:
@@ -97,6 +137,16 @@ protected:
      * @return The result of performing the opreration on the number
      */
     virtual int64_t transformNumber(int64_t number) const = 0;
+
+    /**
+     * Subclasses override this function to perform some kind of
+     * mathematical operation on the number.  The result of this operation
+     * is formatted using the rule set or DecimalFormat that this
+     * substitution refers to, and the result is inserted into the result
+     * string.
+     * @param The number being formatted
+     * @return The result of performing the opreration on the number
+     */
     virtual double transformNumber(double number) const = 0;
     
 public:
@@ -158,8 +208,10 @@ public:
      * Calculates an upper bound when searching for a rule that matches
      * this substitution.  Rules with base values greater than or equal
      * to upperBound are not considered.
-     * @param oldUpperBound The current upper-bound setting.  The new
-     * upper bound can't be any higher.
+     * @param oldUpperBound    The current upper-bound setting.  The new
+     *                         upper bound can't be any higher.
+     * @return                 the upper bound when searching for a rule that matches
+     *                         this substitution.
      */
     virtual double calcUpperBound(double oldUpperBound) const = 0;
     
@@ -197,8 +249,11 @@ public:
     virtual UBool isModulusSubstitution() const;
     
 private:
+    NFSubstitution(const NFSubstitution &other); // forbid copying of this class
+    NFSubstitution &operator=(const NFSubstitution &other); // forbid copying of this class
+
     static const char fgClassID;
-    
+
 public:
     static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
     virtual UClassID getDynamicClassID(void) const;
@@ -212,11 +267,11 @@ public:
         const UnicodeString& description,
         UErrorCode& status);
     
-    int64_t transformNumber(int64_t number) const { return number; }
-    double transformNumber(double number) const { return number; }
-    double composeRuleValue(double newRuleValue, double /*oldRuleValue*/) const { return newRuleValue; }
-    double calcUpperBound(double oldUpperBound) const { return oldUpperBound; }
-    UChar tokenChar() const { return (UChar)0x003d; } // '='
+    virtual int64_t transformNumber(int64_t number) const { return number; }
+    virtual double transformNumber(double number) const { return number; }
+    virtual double composeRuleValue(double newRuleValue, double /*oldRuleValue*/) const { return newRuleValue; }
+    virtual double calcUpperBound(double oldUpperBound) const { return oldUpperBound; }
+    virtual UChar tokenChar() const { return (UChar)0x003d; } // '='
 private:
     static const char fgClassID;
     
@@ -239,30 +294,37 @@ public:
         : NFSubstitution(_pos, _ruleSet, formatter, description, status), divisor(_divisor)
     {
         ldivisor = util64_fromDouble(divisor);
+        if (divisor == 0) {
+            status = U_PARSE_ERROR;
+        }
     }
     
-    void setDivisor(int32_t radix, int32_t exponent) { 
+    virtual void setDivisor(int32_t radix, int32_t exponent, UErrorCode& status) { 
         divisor = uprv_pow(radix, exponent);
         ldivisor = util64_fromDouble(divisor);
+
+        if(divisor == 0) {
+            status = U_PARSE_ERROR;
+        }
     }
     
-    UBool operator==(const NFSubstitution& rhs) const;
+    virtual UBool operator==(const NFSubstitution& rhs) const;
     
-    int64_t transformNumber(int64_t number) const {
+    virtual int64_t transformNumber(int64_t number) const {
         return number / ldivisor;
     }
     
-    double transformNumber(double number) const {
+    virtual double transformNumber(double number) const {
         return uprv_floor(number / divisor);
     }
     
-    double composeRuleValue(double newRuleValue, double /*oldRuleValue*/) const {
+    virtual double composeRuleValue(double newRuleValue, double /*oldRuleValue*/) const {
         return newRuleValue * divisor;
     }
     
-    double calcUpperBound(double /*oldUpperBound*/) const { return divisor; }
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return divisor; }
     
-    UChar tokenChar() const { return (UChar)0x003c; } // '<'
+    virtual UChar tokenChar() const { return (UChar)0x003c; } // '<'
 private:
     static const char fgClassID;
     
@@ -284,35 +346,39 @@ public:
         const UnicodeString& description,
         UErrorCode& status);
     
-    void setDivisor(int32_t radix, int32_t exponent) { 
+    virtual void setDivisor(int32_t radix, int32_t exponent, UErrorCode& status) { 
         divisor = uprv_pow(radix, exponent);
         ldivisor = util64_fromDouble(divisor);
+
+        if (divisor == 0) {
+            status = U_PARSE_ERROR;
+        }
     }
     
-    UBool operator==(const NFSubstitution& rhs) const;
+    virtual UBool operator==(const NFSubstitution& rhs) const;
     
-    void doSubstitution(int64_t number, UnicodeString& toInsertInto, int32_t pos) const;
-    void doSubstitution(double number, UnicodeString& toInsertInto, int32_t pos) const;
+    virtual void doSubstitution(int64_t number, UnicodeString& toInsertInto, int32_t pos) const;
+    virtual void doSubstitution(double number, UnicodeString& toInsertInto, int32_t pos) const;
     
-    int64_t transformNumber(int64_t number) const { return number % ldivisor; }
-    double transformNumber(double number) const { return uprv_fmod(number, divisor); }
+    virtual int64_t transformNumber(int64_t number) const { return number % ldivisor; }
+    virtual double transformNumber(double number) const { return uprv_fmod(number, divisor); }
     
-    UBool doParse(const UnicodeString& text, 
+    virtual UBool doParse(const UnicodeString& text, 
         ParsePosition& parsePosition,
         double baseValue,
         double upperBound,
         UBool lenientParse,
         Formattable& result) const;
     
-    double composeRuleValue(double newRuleValue, double oldRuleValue) const {
+    virtual double composeRuleValue(double newRuleValue, double oldRuleValue) const {
         return oldRuleValue - uprv_fmod(oldRuleValue, divisor) + newRuleValue;
     }
     
-    double calcUpperBound(double /*oldUpperBound*/) const { return divisor; }
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return divisor; }
     
-    UBool isModulusSubstitution() const { return TRUE; }
+    virtual UBool isModulusSubstitution() const { return TRUE; }
     
-    UChar tokenChar() const { return (UChar)0x003e; } // '>'
+    virtual UChar tokenChar() const { return (UChar)0x003e; } // '>'
 private:
     static const char fgClassID;
     
@@ -330,11 +396,11 @@ public:
         UErrorCode& status)
         : NFSubstitution(_pos, _ruleSet, formatter, description, status) {}
     
-    int64_t transformNumber(int64_t number) const { return number; }
-    double transformNumber(double number) const { return uprv_floor(number); }
-    double composeRuleValue(double newRuleValue, double oldRuleValue) const { return newRuleValue + oldRuleValue; }
-    double calcUpperBound(double /*oldUpperBound*/) const { return DBL_MAX; }
-    UChar tokenChar() const { return (UChar)0x003c; } // '<'
+    virtual int64_t transformNumber(int64_t number) const { return number; }
+    virtual double transformNumber(double number) const { return uprv_floor(number); }
+    virtual double composeRuleValue(double newRuleValue, double oldRuleValue) const { return newRuleValue + oldRuleValue; }
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return DBL_MAX; }
+    virtual UChar tokenChar() const { return (UChar)0x003c; } // '<'
 private:
     static const char fgClassID;
     
@@ -354,23 +420,23 @@ public:
         const UnicodeString& description,
         UErrorCode& status);
     
-    UBool operator==(const NFSubstitution& rhs) const;
+    virtual UBool operator==(const NFSubstitution& rhs) const;
     
-    void doSubstitution(double number, UnicodeString& toInsertInto, int32_t pos) const;
-    void doSubstitution(int64_t /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
-    int64_t transformNumber(int64_t /*number*/) const { return 0; }
-    double transformNumber(double number) const { return number - uprv_floor(number); }
+    virtual void doSubstitution(double number, UnicodeString& toInsertInto, int32_t pos) const;
+    virtual void doSubstitution(int64_t /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
+    virtual int64_t transformNumber(int64_t /*number*/) const { return 0; }
+    virtual double transformNumber(double number) const { return number - uprv_floor(number); }
     
-    UBool doParse(const UnicodeString& text,
+    virtual UBool doParse(const UnicodeString& text,
         ParsePosition& parsePosition,
         double baseValue,
         double upperBound,
         UBool lenientParse,
         Formattable& result) const;
     
-    double composeRuleValue(double newRuleValue, double oldRuleValue) const { return newRuleValue + oldRuleValue; }
-    double calcUpperBound(double /*oldUpperBound*/) const { return 0.0; }
-    UChar tokenChar() const { return (UChar)0x003e; } // '>'
+    virtual double composeRuleValue(double newRuleValue, double oldRuleValue) const { return newRuleValue + oldRuleValue; }
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return 0.0; }
+    virtual UChar tokenChar() const { return (UChar)0x003e; } // '>'
 private:
     static const char fgClassID;
     
@@ -388,11 +454,11 @@ public:
         UErrorCode& status)
         : NFSubstitution(_pos, _ruleSet, formatter, description, status) {}
     
-    int64_t transformNumber(int64_t number) const { return number >= 0 ? number : -number; }
-    double transformNumber(double number) const { return uprv_fabs(number); }
-    double composeRuleValue(double newRuleValue, double /*oldRuleValue*/) const { return -newRuleValue; }
-    double calcUpperBound(double /*oldUpperBound*/) const { return DBL_MAX; }
-    UChar tokenChar() const { return (UChar)0x003e; } // '>'
+    virtual int64_t transformNumber(int64_t number) const { return number >= 0 ? number : -number; }
+    virtual double transformNumber(double number) const { return uprv_fabs(number); }
+    virtual double composeRuleValue(double newRuleValue, double /*oldRuleValue*/) const { return -newRuleValue; }
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return DBL_MAX; }
+    virtual UChar tokenChar() const { return (UChar)0x003e; } // '>'
 private:
     static const char fgClassID;
     
@@ -404,41 +470,48 @@ public:
 class NumeratorSubstitution : public NFSubstitution {
     double denominator;
     int64_t ldenominator;
+    UBool withZeros;
 public:
+    static inline UnicodeString fixdesc(const UnicodeString& desc) {
+      if (desc.endsWith(LTLT, 2)) {
+        UnicodeString result(desc, 0, desc.length()-1);
+        return result;
+      }
+      return desc;
+    }
     NumeratorSubstitution(int32_t _pos,
         double _denominator,
         const NFRuleSet* _ruleSet,
         const RuleBasedNumberFormat* formatter,
         const UnicodeString& description,
         UErrorCode& status)
-        : NFSubstitution(_pos, _ruleSet, formatter, description, status), denominator(_denominator) 
+        : NFSubstitution(_pos, _ruleSet, formatter, fixdesc(description), status), denominator(_denominator) 
     {
         ldenominator = util64_fromDouble(denominator);
+        withZeros = description.endsWith(LTLT, 2);
     }
     
-    UBool operator==(const NFSubstitution& rhs) const;
+    virtual UBool operator==(const NFSubstitution& rhs) const;
     
-    int64_t transformNumber(int64_t number) const { return number * ldenominator; }
-    double transformNumber(double number) const { return uprv_round(number * denominator); }
+    virtual int64_t transformNumber(int64_t number) const { return number * ldenominator; }
+    virtual double transformNumber(double number) const { return uprv_round(number * denominator); }
     
-    UBool doParse(const UnicodeString& text, 
+    virtual void doSubstitution(int64_t /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
+    virtual void doSubstitution(double number, UnicodeString& toInsertInto, int32_t pos) const;
+    virtual UBool doParse(const UnicodeString& text, 
         ParsePosition& parsePosition,
         double baseValue,
         double upperBound,
         UBool /*lenientParse*/,
-        Formattable& result) const 
-    {
-        // we don't have to do anything special to do the parsing here,
-        // but we have to turn lenient parsing off-- if we leave it on,
-        // it SERIOUSLY messes up the algorithm
-        return NFSubstitution::doParse(text, parsePosition, baseValue, upperBound, FALSE, result);
-    }
-    double composeRuleValue(double newRuleValue, double oldRuleValue) const { return newRuleValue / oldRuleValue; }
-    double calcUpperBound(double /*oldUpperBound*/) const { return denominator; }
-    UChar tokenChar() const { return (UChar)0x003c; } // '<'
+        Formattable& result) const;
+
+    virtual double composeRuleValue(double newRuleValue, double oldRuleValue) const { return newRuleValue / oldRuleValue; }
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return denominator; }
+    virtual UChar tokenChar() const { return (UChar)0x003c; } // '<'
 private:
     static const char fgClassID;
-    
+    static const UChar LTLT[2];
+
 public:
     static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
     virtual UClassID getDynamicClassID(void) const;
@@ -453,22 +526,22 @@ public:
         UErrorCode& status)
         : NFSubstitution(_pos, _ruleSet, formatter, description, status) {}
     
-    void toString(UnicodeString& /*result*/) const {}
-    void doSubstitution(double /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
-    void doSubstitution(int64_t /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
-    int64_t transformNumber(int64_t /*number*/) const { return 0; }
-    double transformNumber(double /*number*/) const { return 0; }
-    UBool doParse(const UnicodeString& /*text*/,
+    virtual void toString(UnicodeString& /*result*/) const {}
+    virtual void doSubstitution(double /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
+    virtual void doSubstitution(int64_t /*number*/, UnicodeString& /*toInsertInto*/, int32_t /*_pos*/) const {}
+    virtual int64_t transformNumber(int64_t /*number*/) const { return 0; }
+    virtual double transformNumber(double /*number*/) const { return 0; }
+    virtual UBool doParse(const UnicodeString& /*text*/,
                 ParsePosition& /*parsePosition*/, 
                 double baseValue,
                 double /*upperBound*/,
                 UBool /*lenientParse*/,
                 Formattable& result) const
             { result.setDouble(baseValue); return TRUE; }
-    double composeRuleValue(double /*newRuleValue*/, double /*oldRuleValue*/) const { return 0.0; } // never called
-    double calcUpperBound(double /*oldUpperBound*/) const { return 0; } // never called
-    UBool isNullSubstitution() const { return TRUE; }
-    UChar tokenChar() const { return (UChar)0x0020; } // ' ' never called
+    virtual double composeRuleValue(double /*newRuleValue*/, double /*oldRuleValue*/) const { return 0.0; } // never called
+    virtual double calcUpperBound(double /*oldUpperBound*/) const { return 0; } // never called
+    virtual UBool isNullSubstitution() const { return TRUE; }
+    virtual UChar tokenChar() const { return (UChar)0x0020; } // ' ' never called
 private:
     static const char fgClassID;
     
