@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001-2002, International Business Machines
+*   Copyright (C) 2001-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -11,6 +11,9 @@
 #define NORTRANS_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/translit.h"
 #include "unicode/normlzr.h"
 
@@ -32,11 +35,6 @@ class U_I18N_API NormalizationTransliterator : public Transliterator {
      */
     int32_t options;
 
-    /**
-     * Alias to skippables set.  NOT OWNED.
-     */
-    UnicodeSet* skippable;
-
  public:
 
     /**
@@ -56,15 +54,32 @@ class U_I18N_API NormalizationTransliterator : public Transliterator {
 
     /**
      * Transliterator API.
+     * @return    A copy of the object.
      */
-    Transliterator* clone(void) const;
+    virtual Transliterator* clone(void) const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
 
  protected:
 
     /**
      * Implements {@link Transliterator#handleTransliterate}.
+     * @param text          the buffer holding transliterated and
+     *                      untransliterated text
+     * @param offset        the start and limit of the text, the position
+     *                      of the cursor, and the start and limit of transliteration.
+     * @param incremental   if true, assume more text may be coming after
+     *                      pos.contextLimit. Otherwise, assume the text is complete.
      */
-    void handleTransliterate(Replaceable& text, UTransPosition& offset,
+    virtual void handleTransliterate(Replaceable& text, UTransPosition& offset,
                              UBool isIncremental) const;
  public:
 
@@ -73,11 +88,6 @@ class U_I18N_API NormalizationTransliterator : public Transliterator {
      */
     static void registerIDs();
 
-    /**
-     * Static memory cleanup function.
-     */
-    static void cleanup();
-
  private:
 
     // Transliterator::Factory methods
@@ -90,10 +100,10 @@ class U_I18N_API NormalizationTransliterator : public Transliterator {
      */
     NormalizationTransliterator(const UnicodeString& id,
                                 UNormalizationMode mode, int32_t opt);
-
-    static void initStatics();
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 #endif
