@@ -1,17 +1,23 @@
 /*
-* Copyright (C) 2001, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 2001-2003, International Business Machines Corporation and others. All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   07/26/01    aliu        Creation.
 **********************************************************************
 */
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "quant.h"
 #include "unicode/unistr.h"
 #include "util.h"
 
 U_NAMESPACE_BEGIN
 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(Quantifier)
+
 Quantifier::Quantifier(UnicodeFunctor *adoptedMatcher,
                        uint32_t _minCount, uint32_t _maxCount) {
     // assert(adopted != 0);
@@ -23,6 +29,7 @@ Quantifier::Quantifier(UnicodeFunctor *adoptedMatcher,
 
 Quantifier::Quantifier(const Quantifier& o) :
     UnicodeFunctor(o),
+    UnicodeMatcher(o),
     matcher(o.matcher->clone()),
     minCount(o.minCount),
     maxCount(o.maxCount)
@@ -114,6 +121,15 @@ UBool Quantifier::matchesIndexValue(uint8_t v) const {
     return (minCount == 0) || matcher->toMatcher()->matchesIndexValue(v);
 }
 
+/**
+ * Implement UnicodeMatcher
+ */
+void Quantifier::addMatchSetTo(UnicodeSet& toUnionTo) const {
+    if (maxCount > 0) {
+    matcher->toMatcher()->addMatchSetTo(toUnionTo);
+    }
+}
+
 /**
  * Implement UnicodeFunctor
  */
@@ -123,4 +139,6 @@ void Quantifier::setData(const TransliterationRuleData* d) {
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 //eof
