@@ -1,55 +1,62 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999, International Business Machines
+*   Copyright (C) 1999-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   11/17/99    aliu        Creation.
 **********************************************************************
 */
-#include "rbt_data.h"
-#include "hash.h"
+
+#include "unicode/utypes.h"
+#include "umutex.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/unistr.h"
 #include "unicode/uniset.h"
+#include "rbt_data.h"
+#include "hash.h"
+#include "cmemory.h"
 
 U_NAMESPACE_BEGIN
 
 TransliterationRuleData::TransliterationRuleData(UErrorCode& status)
- : ruleSet(status),
-    variableNames(0), variables(0)
+ : UMemory(), ruleSet(status), variableNames(status),
+    variables(0), variablesAreOwned(TRUE)
 {
     if (U_FAILURE(status)) {
         return;
     }
-    variableNames = new Hashtable(status);
-    if (U_SUCCESS(status)) {
-        variableNames->setValueDeleter(uhash_deleteUnicodeString);
-    }
+    variableNames.setValueDeleter(uhash_deleteUnicodeString);
     variables = 0;
     variablesLength = 0;
 }
 
 TransliterationRuleData::TransliterationRuleData(const TransliterationRuleData& other) :
-    ruleSet(other.ruleSet),
+    UMemory(other), ruleSet(other.ruleSet),
+    variablesAreOwned(TRUE),
     variablesBase(other.variablesBase),
     variablesLength(other.variablesLength)
 {
     UErrorCode status = U_ZERO_ERROR;
-    variableNames = new Hashtable(status);
-    if (U_SUCCESS(status)) {
-        variableNames->setValueDeleter(uhash_deleteUnicodeString);
-        int32_t pos = -1;
-        const UHashElement *e;
-        while ((e = other.variableNames->nextElement(pos)) != 0) {
-            UnicodeString* value =
-                new UnicodeString(*(const UnicodeString*)e->value.pointer);
-            variableNames->put(*(UnicodeString*)e->key.pointer, value, status);
-        }
+    variableNames.setValueDeleter(uhash_deleteUnicodeString);
+    int32_t pos = -1;
+    const UHashElement *e;
+    while ((e = other.variableNames.nextElement(pos)) != 0) {
+        UnicodeString* value =
+            new UnicodeString(*(const UnicodeString*)e->value.pointer);
+        variableNames.put(*(UnicodeString*)e->key.pointer, value, status);
     }
 
     variables = 0;
     if (other.variables != 0) {
-        variables = new UnicodeFunctor*[variablesLength];
+        variables = (UnicodeFunctor **)uprv_malloc(variablesLength * sizeof(UnicodeFunctor *));
+        /* test for NULL */
+        if (variables == 0) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return;
+        }
         for (int32_t i=0; i<variablesLength; ++i) {
             variables[i] = other.variables[i]->clone();
         }
@@ -60,13 +67,12 @@ TransliterationRuleData::TransliterationRuleData(const TransliterationRuleData&
 }
 
 TransliterationRuleData::~TransliterationRuleData() {
-    delete variableNames;
-    if (variables != 0) {
+    if (variablesAreOwned && variables != 0) {
         for (int32_t i=0; i<variablesLength; ++i) {
             delete variables[i];
         }
-        delete[] variables;
     }
+    uprv_free(variables);
 }
 
 UnicodeFunctor*
@@ -87,4 +93,7 @@ TransliterationRuleData::lookupReplacer(UChar32 standIn) const {
     return (f != 0) ? f->toReplacer() : 0;
 }
 
+
 U_NAMESPACE_END
+
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
