@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-1999, International Business Machines Corporation and    *
+* Copyright (C) 1997-2005, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 *
@@ -21,19 +21,23 @@
 ********************************************************************************
 */
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/simpletz.h"
 #include "unicode/gregocal.h"
-#include "tzdat.h"
+#include "unicode/smpdtfmt.h"
 
 U_NAMESPACE_BEGIN
 
-const char SimpleTimeZone::fgClassID = 0; // Value is irrelevant
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(SimpleTimeZone)
 
 // WARNING: assumes that no rule is measured from the end of February,
 // since we don't handle leap years. Could handle assuming always
 // Gregorian, since we know they didn't have daylight time when
 // Gregorian calendar started.
-const int8_t SimpleTimeZone::staticMonthLength[] = {31,28,31,30,31,30,31,31,30,31,30,31};
+const int8_t SimpleTimeZone::STATICMONTHLENGTH[] = {31,29,31,30,31,30,31,31,30,31,30,31};
 
 // *****************************************************************************
 // class SimpleTimeZone
@@ -117,40 +121,6 @@ SimpleTimeZone::SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID,
               savingsDST, status);
 }
 
-/**
- * Construct from memory-mapped data.  For private use by TimeZone.
- */
-SimpleTimeZone::SimpleTimeZone(const StandardZone& stdZone,
-                               const UnicodeString& ID)
-:   TimeZone(ID)
-{
-    UErrorCode status = U_ZERO_ERROR;
-    construct(stdZone.gmtOffset,
-              0, 0, 0, 0, WALL_TIME,
-              0, 0, 0, 0, WALL_TIME,
-              0, status);
-}
-
-/**
- * Construct from memory-mapped data.  For private use by TimeZone.
- */
-SimpleTimeZone::SimpleTimeZone(const DSTZone& dstZone,
-                               const UnicodeString& ID)
-:   TimeZone(ID)
-{
-    UErrorCode status = U_ZERO_ERROR;
-    construct(dstZone.gmtOffset,
-              dstZone.onsetRule.month, dstZone.onsetRule.dowim,
-              dstZone.onsetRule.dow,
-              dstZone.onsetRule.time * (int32_t)60000,
-              (TimeMode)dstZone.onsetRule.mode,
-              dstZone.ceaseRule.month, dstZone.ceaseRule.dowim,
-              dstZone.ceaseRule.dow,
-              dstZone.ceaseRule.time * (int32_t)60000,
-              (TimeMode)dstZone.ceaseRule.mode,
-              dstZone.dstSavings * (int32_t)60000, status);
-}
-
 /**
  * Internal construction method.
  */
@@ -395,54 +365,44 @@ SimpleTimeZone::setEndRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
 
 // -------------------------------------
 
-// deprecated version
-int32_t
-SimpleTimeZone::getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
-                          uint8_t dayOfWeek, int32_t millis) const
-{
-    UErrorCode status = U_ZERO_ERROR;
-    return getOffset(era, year, month, day, dayOfWeek, millis, status);
-}
-
-
 int32_t
 SimpleTimeZone::getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                           uint8_t dayOfWeek, int32_t millis, UErrorCode& status) const
 {
-    // Check the month before indexing into staticMonthLength. This
+    // Check the month before indexing into STATICMONTHLENGTH. This
     // duplicates the test that occurs in the 7-argument getOffset(),
     // however, this is unavoidable. We don't mind because this method, in
     // fact, should not be called; internal code should always call the
     // 7-argument getOffset(), and outside code should use Calendar.get(int
     // field) with fields ZONE_OFFSET and DST_OFFSET. We can't get rid of
     // this method because it's public API. - liu 8/10/98
-    if(month < Calendar::JANUARY || month > Calendar::DECEMBER) {
+    if(month < UCAL_JANUARY || month > UCAL_DECEMBER) {
         status = U_ILLEGAL_ARGUMENT_ERROR;
         return 0;
     }
 
-    return getOffset(era, year, month, day, dayOfWeek, millis, staticMonthLength[month], status);
+    return getOffset(era, year, month, day, dayOfWeek, millis, STATICMONTHLENGTH[month], status);
 }
 
 int32_t 
 SimpleTimeZone::getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                           uint8_t dayOfWeek, int32_t millis, 
                           int32_t monthLength, UErrorCode& status) const {
-    // Check the month before indexing into staticMonthLength. This
+    // Check the month before indexing into STATICMONTHLENGTH. This
     // duplicates a test that occurs in the 9-argument getOffset(),
     // however, this is unavoidable. We don't mind because this method, in
     // fact, should not be called; internal code should always call the
     // 9-argument getOffset(), and outside code should use Calendar.get(int
     // field) with fields ZONE_OFFSET and DST_OFFSET. We can't get rid of
     // this method because it's public API. - liu 8/10/98
-    if (month < Calendar::JANUARY
-        || month > Calendar::DECEMBER) {
+    if (month < UCAL_JANUARY
+        || month > UCAL_DECEMBER) {
         status = U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
 
     // TODO FIX We don't handle leap years yet!
-    int32_t prevMonthLength = (month >= 1) ? staticMonthLength[month - 1] : 31;
+    int32_t prevMonthLength = (month >= 1) ? STATICMONTHLENGTH[month - 1] : 31;
 
     return getOffset(era, year, month, day, dayOfWeek, millis,
                      monthLength, prevMonthLength, status);
@@ -457,16 +417,18 @@ SimpleTimeZone::getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
     if(U_FAILURE(status)) return 0;
 
     if ((era != GregorianCalendar::AD && era != GregorianCalendar::BC)
-        || month < Calendar::JANUARY
-        || month > Calendar::DECEMBER
+        || month < UCAL_JANUARY
+        || month > UCAL_DECEMBER
         || day < 1
         || day > monthLength
-        || dayOfWeek < Calendar::SUNDAY
-        || dayOfWeek > Calendar::SATURDAY
+        || dayOfWeek < UCAL_SUNDAY
+        || dayOfWeek > UCAL_SATURDAY
         || millis < 0
         || millis >= U_MILLIS_PER_DAY
         || monthLength < 28
-        || monthLength > 31) {
+        || monthLength > 31
+        || prevMonthLength < 28
+        || prevMonthLength > 31) {
         status = U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
@@ -585,7 +547,7 @@ SimpleTimeZone::compareToRule(int8_t month, int8_t monthLen, int8_t prevMonthLen
         
         // if ruleDay is negative (we assume it's not zero here), we have to do
         // the same calculation figuring backward from the last day of the month.
-        // (staticMonthLength gives us that last day.  We don't take leap years
+        // (STATICMONTHLENGTH gives us that last day.  We don't take leap years
         // into account, so this may not work right for February.)
         else
         {
@@ -636,19 +598,6 @@ SimpleTimeZone::setRawOffset(int32_t offsetMillis)
     rawOffset = offsetMillis;
 }
 
-#ifdef ICU_SIMPLETIMEZONE_USE_DEPRECATES
-// -------------------------------------
-
-// deprecated
-void 
-SimpleTimeZone::setDSTSavings(int32_t millisSavedDuringDST) 
-{
-    UErrorCode status = U_ZERO_ERROR;
-    setDSTSavings(millisSavedDuringDST, status);
-}
-
-#endif
-
 // -------------------------------------
 
 void 
@@ -691,6 +640,11 @@ UBool SimpleTimeZone::inDaylightTime(UDate date, UErrorCode& status) const
     // and provided only for Java compatibility as of 8/6/97 [LIU].
     if (U_FAILURE(status)) return FALSE;
     GregorianCalendar *gc = new GregorianCalendar(*this, status);
+    /* test for NULL */
+    if (gc == 0) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        return FALSE;
+    }
     gc->setTime(date, status);
     UBool result = gc->inDaylightTime(status);
     delete gc;
@@ -836,7 +790,7 @@ SimpleTimeZone::decodeStartRule(UErrorCode& status)
         dstSavings = U_MILLIS_PER_HOUR;
     }
     if (startDay != 0) {
-        if (startMonth < Calendar::JANUARY || startMonth > Calendar::DECEMBER) {
+        if (startMonth < UCAL_JANUARY || startMonth > UCAL_DECEMBER) {
             status = U_ILLEGAL_ARGUMENT_ERROR;
             return;
         }
@@ -859,7 +813,7 @@ SimpleTimeZone::decodeStartRule(UErrorCode& status)
                     startMode = DOW_LE_DOM_MODE;
                 }
             }
-            if (startDayOfWeek > Calendar::SATURDAY) {
+            if (startDayOfWeek > UCAL_SATURDAY) {
                 status = U_ILLEGAL_ARGUMENT_ERROR;
                 return;
             }
@@ -869,7 +823,7 @@ SimpleTimeZone::decodeStartRule(UErrorCode& status)
                 status = U_ILLEGAL_ARGUMENT_ERROR;
                 return;
             }
-        } else if (startDay > staticMonthLength[startMonth]) {
+        } else if (startDay<1 || startDay > STATICMONTHLENGTH[startMonth]) {
             status = U_ILLEGAL_ARGUMENT_ERROR;
             return;
         }
@@ -891,7 +845,7 @@ SimpleTimeZone::decodeEndRule(UErrorCode& status)
         dstSavings = U_MILLIS_PER_HOUR;
     }
     if (endDay != 0) {
-        if (endMonth < Calendar::JANUARY || endMonth > Calendar::DECEMBER) {
+        if (endMonth < UCAL_JANUARY || endMonth > UCAL_DECEMBER) {
             status = U_ILLEGAL_ARGUMENT_ERROR;
             return;
         }
@@ -914,7 +868,7 @@ SimpleTimeZone::decodeEndRule(UErrorCode& status)
                     endMode = DOW_LE_DOM_MODE;
                 }
             }
-            if (endDayOfWeek > Calendar::SATURDAY) {
+            if (endDayOfWeek > UCAL_SATURDAY) {
                 status = U_ILLEGAL_ARGUMENT_ERROR;
                 return;
             }
@@ -924,7 +878,7 @@ SimpleTimeZone::decodeEndRule(UErrorCode& status)
                 status = U_ILLEGAL_ARGUMENT_ERROR;
                 return;
             }
-        } else if (endDay > staticMonthLength[endMonth]) {
+        } else if (endDay<1 || endDay > STATICMONTHLENGTH[endMonth]) {
             status = U_ILLEGAL_ARGUMENT_ERROR;
             return;
         }
@@ -933,4 +887,6 @@ SimpleTimeZone::decodeEndRule(UErrorCode& status)
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 //eof
