@@ -1,19 +1,110 @@
 /*
 *******************************************************************************
-*   Copyright (C) 1996-2001, International Business Machines
+*   Copyright (C) 1996-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 */
 
-#include "unicode/ucal.h"
+#include "unicode/utypes.h"
 
+#if !UCONFIG_NO_FORMATTING
+
+#include "unicode/ucal.h"
 #include "unicode/uloc.h"
 #include "unicode/calendar.h"
 #include "unicode/timezone.h"
+#include "unicode/gregocal.h"
+#include "unicode/simpletz.h"
 #include "unicode/ustring.h"
+#include "unicode/strenum.h"
+#include "cmemory.h"
+#include "ustrenum.h"
 
 U_NAMESPACE_USE
 
+static TimeZone*
+_createTimeZone(const UChar* zoneID, int32_t len, UErrorCode* ec) {
+    TimeZone* zone = NULL;
+    if (ec!=NULL && U_SUCCESS(*ec)) {
+        // Note that if zoneID is invalid, we get back GMT. This odd
+        // behavior is by design and goes back to the JDK. The only
+        // failure we will see is a memory allocation failure.
+        int32_t l = (len<0 ? u_strlen(zoneID) : len);
+        UnicodeString zoneStrID;
+        zoneStrID.setTo((UBool)(len < 0), zoneID, l); /* temporary read-only alias */
+        zone = TimeZone::createTimeZone(zoneStrID);
+        if (zone == NULL) {
+            *ec = U_MEMORY_ALLOCATION_ERROR;
+        }
+    }
+    return zone;
+}
+
+U_CAPI UEnumeration* U_EXPORT2
+ucal_openTimeZones(UErrorCode* ec) {
+    return uenum_openStringEnumeration(TimeZone::createEnumeration(), ec);
+}
+
+U_CAPI UEnumeration* U_EXPORT2
+ucal_openCountryTimeZones(const char* country, UErrorCode* ec) {
+    return uenum_openStringEnumeration(TimeZone::createEnumeration(country), ec);
+}
+
+U_CAPI int32_t U_EXPORT2
+ucal_getDefaultTimeZone(UChar* result, int32_t resultCapacity, UErrorCode* ec) {
+    int32_t len = 0;
+    if (ec!=NULL && U_SUCCESS(*ec)) {
+        TimeZone* zone = TimeZone::createDefault();
+        if (zone == NULL) {
+            *ec = U_MEMORY_ALLOCATION_ERROR;
+        } else {
+            UnicodeString id;
+            zone->getID(id);
+            delete zone;
+            len = id.extract(result, resultCapacity, *ec);
+        }
+    }
+    return len;
+}
+
+U_CAPI void U_EXPORT2
+ucal_setDefaultTimeZone(const UChar* zoneID, UErrorCode* ec) {
+    TimeZone* zone = _createTimeZone(zoneID, -1, ec);
+    if (zone != NULL) {
+        TimeZone::adoptDefault(zone);
+    }
+}
+
+U_CAPI int32_t U_EXPORT2
+ucal_getDSTSavings(const UChar* zoneID, UErrorCode* ec) {
+    int32_t result = 0;
+    TimeZone* zone = _createTimeZone(zoneID, -1, ec);
+    if (U_SUCCESS(*ec)) {
+        if (zone->getDynamicClassID() == SimpleTimeZone::getStaticClassID()) {
+            result = ((SimpleTimeZone*) zone)->getDSTSavings();
+        } else {
+            // Since there is no getDSTSavings on TimeZone, we use a
+            // heuristic: Starting with the current time, march
+            // forwards for one year, looking for DST savings.
+            // Stepping by weeks is sufficient.
+            UDate d = Calendar::getNow();
+            for (int32_t i=0; i<53; ++i, d+=U_MILLIS_PER_DAY*7.0) {
+                int32_t raw, dst;
+                zone->getOffset(d, FALSE, raw, dst, *ec);
+                if (U_FAILURE(*ec)) {
+                    break;
+                } else if (dst != 0) {
+                    result = dst;
+                    break;
+                }
+            }
+        }
+    }
+    delete zone;
+    return result;
+}
+
+#ifdef U_USE_UCAL_OBSOLETE_2_8
 U_CAPI const UChar* U_EXPORT2
 ucal_getAvailableTZIDs(        int32_t         rawOffset,
                 int32_t         index,
@@ -38,7 +129,7 @@ ucal_getAvailableTZIDs(        int32_t         rawOffset,
   else
     *status = U_INDEX_OUTOFBOUNDS_ERROR;
   
-  delete [] tzs;
+  uprv_free(tzs);
   return retVal;
 }
 
@@ -56,9 +147,10 @@ ucal_countAvailableTZIDs(int32_t rawOffset)
     return 0;
   }
 
-  delete [] tzs;
+  uprv_free(tzs);
   return count;
 }
+#endif
 
 U_CAPI UDate  U_EXPORT2
 ucal_getNow()
@@ -78,18 +170,11 @@ ucal_open(    const    UChar*          zoneID,
 
   if(U_FAILURE(*status)) return 0;
   
-  TimeZone *zone = 0;
-  if(zoneID == 0) {
-    zone = TimeZone::createDefault();
-  }
-  else {
-    int32_t length = (len == -1 ? u_strlen(zoneID) : len);
+  TimeZone* zone = (zoneID==NULL) ? TimeZone::createDefault()
+      : _createTimeZone(zoneID, len, status);
 
-    zone = TimeZone::createTimeZone(UnicodeString(zoneID, length));
-  }
-  if(zone == 0) {
-    *status = U_MEMORY_ALLOCATION_ERROR;
-    return 0;
+  if (U_FAILURE(*status)) {
+      return NULL;
   }
   
   return (UCalendar*)Calendar::createInstance(zone, Locale(locale), *status);
@@ -112,21 +197,12 @@ ucal_setTimeZone(    UCalendar*      cal,
   if(U_FAILURE(*status))
     return;
 
-  TimeZone *zone;
-  if(zoneID == NULL) {
-    zone = TimeZone::createDefault();
-  }
-  else {
-    int32_t length = (len == -1 ? u_strlen(zoneID) : len);
-    zone = TimeZone::createTimeZone(UnicodeString((UChar*)zoneID, 
-                          length, length));
-  }
-  if(zone == 0) {
-    *status = U_MEMORY_ALLOCATION_ERROR;
-    return;
-  }
+  TimeZone* zone = (zoneID==NULL) ? TimeZone::createDefault()
+      : _createTimeZone(zoneID, len, status);
 
-  ((Calendar*)cal)->adoptTimeZone(zone);
+  if (zone != NULL) {
+      ((Calendar*)cal)->adoptTimeZone(zone);
+  }
 }
 
 U_CAPI int32_t U_EXPORT2
@@ -141,7 +217,12 @@ ucal_getTimeZoneDisplayName(const     UCalendar*                 cal,
   if(U_FAILURE(*status)) return -1;
 
   const TimeZone& tz = ((Calendar*)cal)->getTimeZone();
-  UnicodeString id(result, 0, resultLength);
+  UnicodeString id;
+  if(!(result==NULL && resultLength==0)) {
+    // NULL destination for pure preflighting: empty dummy string
+    // otherwise, alias the destination buffer
+    id.setTo(result, 0, resultLength);
+  }
 
   switch(type) {
   case UCAL_STANDARD:
@@ -173,6 +254,32 @@ ucal_inDaylightTime(    const    UCalendar*      cal,
   return ((Calendar*)cal)->inDaylightTime(*status);
 }
 
+U_DRAFT void U_EXPORT2
+ucal_setGregorianChange(UCalendar *cal, UDate date, UErrorCode *pErrorCode) {
+  if(U_FAILURE(*pErrorCode)) {
+    return;
+  }
+  Calendar *cpp_cal = (Calendar *)cal;
+  if(cpp_cal->getDynamicClassID() != GregorianCalendar::getStaticClassID()) {
+    *pErrorCode = U_UNSUPPORTED_ERROR;
+    return;
+  }
+  ((GregorianCalendar *)cpp_cal)->setGregorianChange(date, *pErrorCode);
+}
+
+U_DRAFT UDate U_EXPORT2
+ucal_getGregorianChange(const UCalendar *cal, UErrorCode *pErrorCode) {
+  if(U_FAILURE(*pErrorCode)) {
+    return (UDate)0;
+  }
+  Calendar *cpp_cal = (Calendar *)cal;
+  if(cpp_cal->getDynamicClassID() != GregorianCalendar::getStaticClassID()) {
+    *pErrorCode = U_UNSUPPORTED_ERROR;
+    return (UDate)0;
+  }
+  return ((GregorianCalendar *)cpp_cal)->getGregorianChange();
+}
+
 U_CAPI int32_t U_EXPORT2
 ucal_getAttribute(    const    UCalendar*              cal,
             UCalendarAttribute      attr)
@@ -206,7 +313,7 @@ ucal_setAttribute(      UCalendar*              cal,
     break;
     
   case UCAL_FIRST_DAY_OF_WEEK:
-    ((Calendar*)cal)->setFirstDayOfWeek((Calendar::EDaysOfWeek)newValue);
+    ((Calendar*)cal)->setFirstDayOfWeek((UCalendarDaysOfWeek)newValue);
     break;
       
   case UCAL_MINIMAL_DAYS_IN_FIRST_WEEK:
@@ -284,7 +391,7 @@ ucal_equivalentTo(    const UCalendar*      cal1,
             const UCalendar*      cal2)
 {
 
-  return ((Calendar*)cal1)->equivalentTo(*((Calendar*)cal2));
+  return ((Calendar*)cal1)->isEquivalentTo(*((Calendar*)cal2));
 }
 
 U_CAPI void  U_EXPORT2
@@ -296,7 +403,7 @@ ucal_add(    UCalendar*                cal,
 
   if(U_FAILURE(*status)) return;
 
-  ((Calendar*)cal)->add((Calendar::EDateFields)field, amount, *status);
+  ((Calendar*)cal)->add(field, amount, *status);
 }
 
 U_CAPI void  U_EXPORT2
@@ -308,7 +415,7 @@ ucal_roll(        UCalendar*            cal,
 
   if(U_FAILURE(*status)) return;
 
-  ((Calendar*)cal)->roll((Calendar::EDateFields)field, amount, *status);
+  ((Calendar*)cal)->roll(field, amount, *status);
 }
 
 U_CAPI int32_t  U_EXPORT2
@@ -319,7 +426,7 @@ ucal_get(    const    UCalendar*                cal,
 
   if(U_FAILURE(*status)) return -1;
 
-  return ((Calendar*)cal)->get((Calendar::EDateFields)field, *status);
+  return ((Calendar*)cal)->get(field, *status);
 }
 
 U_CAPI void  U_EXPORT2
@@ -328,7 +435,7 @@ ucal_set(    UCalendar*                cal,
         int32_t                    value)
 {
 
-  ((Calendar*)cal)->set((Calendar::EDateFields)field, value);
+  ((Calendar*)cal)->set(field, value);
 }
 
 U_CAPI UBool  U_EXPORT2
@@ -336,7 +443,7 @@ ucal_isSet(    const    UCalendar*                cal,
         UCalendarDateFields        field)
 {
 
-  return ((Calendar*)cal)->isSet((Calendar::EDateFields)field);
+  return ((Calendar*)cal)->isSet(field);
 }
 
 U_CAPI void  U_EXPORT2
@@ -344,7 +451,7 @@ ucal_clearField(    UCalendar*            cal,
             UCalendarDateFields field)
 {
 
-  ((Calendar*)cal)->clear((Calendar::EDateFields)field);
+  ((Calendar*)cal)->clear(field);
 }
 
 U_CAPI void  U_EXPORT2
@@ -367,23 +474,23 @@ ucal_getLimit(    const    UCalendar*              cal,
   
   switch(type) {
   case UCAL_MINIMUM:
-    return ((Calendar*)cal)->getMinimum((Calendar::EDateFields)field);
+    return ((Calendar*)cal)->getMinimum(field);
 
   case UCAL_MAXIMUM:
-    return ((Calendar*)cal)->getMaximum((Calendar::EDateFields)field);
+    return ((Calendar*)cal)->getMaximum(field);
 
   case UCAL_GREATEST_MINIMUM:
-    return ((Calendar*)cal)->getGreatestMinimum((Calendar::EDateFields)field);
+    return ((Calendar*)cal)->getGreatestMinimum(field);
 
   case UCAL_LEAST_MAXIMUM:
-    return ((Calendar*)cal)->getLeastMaximum((Calendar::EDateFields)field);
+    return ((Calendar*)cal)->getLeastMaximum(field);
 
   case UCAL_ACTUAL_MINIMUM:
-    return ((Calendar*)cal)->getActualMinimum((Calendar::EDateFields)field,
+    return ((Calendar*)cal)->getActualMinimum(field,
                           *status);
 
   case UCAL_ACTUAL_MAXIMUM:
-    return ((Calendar*)cal)->getActualMaximum((Calendar::EDateFields)field,
+    return ((Calendar*)cal)->getActualMaximum(field,
                           *status);
 
   default:
@@ -391,3 +498,17 @@ ucal_getLimit(    const    UCalendar*              cal,
   }
   return -1;
 }
+
+U_CAPI const char * U_EXPORT2
+ucal_getLocaleByType(const UCalendar *cal, ULocDataLocaleType type, UErrorCode* status) 
+{
+    if (cal == NULL) {
+        if (U_SUCCESS(*status)) {
+            *status = U_ILLEGAL_ARGUMENT_ERROR;
+        }
+        return NULL;
+    }
+    return ((Calendar*)cal)->getLocaleID(type, *status);
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
