@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1998-2001, International Business Machines
+*   Copyright (C) 1998-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -26,43 +26,172 @@
 #ifndef UCOL_IMP_H
 #define UCOL_IMP_H
 
+#include "unicode/utypes.h"
+
+#define UCA_DATA_TYPE "icu"
+#define UCA_DATA_NAME "ucadata"
+#define INVC_DATA_TYPE "icu"
+#define INVC_DATA_NAME "invuca"
+
+/**
+ * Convenience string denoting the Collation data tree
+ * @internal ICU 3.0
+ */
+#define U_ICUDATA_COLL U_ICUDATA_NAME U_TREE_SEPARATOR_STRING "coll"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
-/*#include "ucmpe32.h"*/
 #include "utrie.h"
-#include "unicode/ures.h"
+#include "uresimp.h"
 #include "unicode/udata.h"
+#include "unicode/uiter.h"
+
+/* This is the internal header file which contains important declarations for 
+ * the collation framework. 
+ * Ready to use collators are stored as binary images. Both UCA and tailorings 
+ * share the same binary format. Individual files (currently only UCA) have a 
+ * udata header in front of the image and should be opened using udata_open.
+ * Tailoring images are currently stored inside resource bundles and are intialized
+ * through ucol_open API.
+ *
+ * The following describes the formats for collation binaries
+ * (UCA & tailorings) and for the inverse UCA table.
+ * Substructures are described in the collation design document at
+ * http://dev.icu-project.org/cgi-bin/viewcvs.cgi/~checkout~/icuhtml/design/collation/ICU_collation_design.htm
+ *
+ * -------------------------------------------------------------
+ *
+ * Here is the format of binary collation image.
+ *
+ * Physical order of structures:
+ * - header (UCATableHeader)
+ * - options (UColOptionSet)
+ * - expansions (CE[])
+ * - contractions (UChar[contractionSize] + CE[contractionSize])
+ * - serialized UTrie with mappings of code points to CEs
+ * - max expansion tables (CE[endExpansionCECount] + uint8_t[endExpansionCECount])
+ * - two bit sets for backward processing in strcoll (identical prefixes)
+ *   and for backward CE iteration (each set is uint8_t[UCOL_UNSAFECP_TABLE_SIZE])
+ * - UCA constants (UCAConstants)
+ * - UCA contractions (UChar[contractionUCACombosSize][contractionUCACombosWidth])
+ *
+ * UCATableHeader fields:
+ *
+ * int32_t size; - image size in bytes
+ *
+ * Offsets to interesting data. All offsets are in bytes.
+ * to get the address add to the header address and cast properly.
+ * Some offsets are zero if the corresponding structures are empty.
+ *
+ * Tailoring binaries that only set options and contain no mappings etc.
+ * will have all offsets 0 except for the options and expansion offsets,
+ * which give the position and length of the options array.
+ *
+ * uint32_t options; - offset to default collator options (UColOptionSet *),
+ *                     a set of 32-bit values. See declaration of UColOptionSet for more details
+ *
+ * uint32_t UCAConsts; - only used (!=0) in UCA image - structure which holds values for indirect positioning and implicit ranges
+ *                       See declaration of UCAConstants structure. This is a set of unsigned 32-bit values used to store 
+ *                       important constant values that are defined in the UCA and used for building and runtime.
+ *
+ * uint32_t contractionUCACombos; - only used (!=0) in UCA image - list of UCA contractions. This is a zero terminated array of UChar[contractionUCACombosWidth],
+ *                                  containing contractions from the UCA. These are needed in the build process to copy UCA contractions
+ *                                  in case the base contraction symbol is tailored.
+ *
+ * uint32_t magic; - must contain UCOL_HEADER_MAGIC (formatVersion 2.3)
+ *
+ * uint32_t mappingPosition;  - offset to UTrie (const uint8_t *mappingPosition). This is a serialized UTrie and should be treated as such. 
+ *                              Used as a primary lookup table for collation elements.
+ *
+ * uint32_t expansion;  - offset to expansion table (uint32_t *expansion). This is an array of expansion CEs. Never 0.
+ *
+ * uint32_t contractionIndex; - offset to contraction table (UChar *contractionIndex). Used to look up contraction sequences. Contents
+ *                              are aligned with the contents of contractionCEs table. 0 if no contractions.
+ *
+ * uint32_t contractionCEs;  - offset to resulting contraction CEs (uint32_t *contractionCEs). When a contraction is resolved in the
+ *                             in the contractionIndex table, the resulting index is used to look up corresponding CE in this table. 
+ *                             0 if no contractions.
+ * uint32_t contractionSize; - size of contraction table in elements (both Index and CEs). 
+ *
+ * Tables described below are used for Boyer-Moore searching algorithm - they define the size of longest expansion
+ * and last CEs in expansions.
+ * uint32_t endExpansionCE; - offset to array of last collation element in expansion (uint32_t *).
+ *                            Never 0.
+ * uint32_t expansionCESize; - array of maximum expansion sizes (uint8_t *)
+ * int32_t  endExpansionCECount; - size of endExpansionCE. See UCOL_GETMAXEXPANSION
+ *                                 for the usage model
+ *
+ * These two offsets point to byte tables that are used in the backup heuristics.
+ * uint32_t unsafeCP; - hash table of unsafe code points (uint8_t *). See ucol_unsafeCP function.
+ * uint32_t contrEndCP; - hash table of final code points in contractions (uint8_t *). See ucol_contractionEndCP.              
+ *
+ * int32_t contractionUCACombosSize; - number of UChar[contractionUCACombosWidth] in contractionUCACombos
+ *                                     (formatVersion 2.3)
+ * UBool jamoSpecial; - Jamo special indicator (uint8_t). If TRUE, Jamos are special, so we cannot use simple Hangul decomposition.
+ * UBool isBigEndian; - endianness of this collation binary (formatVersion 2.3)
+ * uint8_t charSetFamily; - charset family of this collation binary (formatVersion 2.3)
+ * uint8_t contractionUCACombosWidth; - number of UChars per UCA contraction in contractionUCACombos (formatVersion 2.3)
+ *
+ * Various version fields
+ * UVersionInfo version; - version 4 uint8_t
+ * UVersionInfo UCAVersion;  - version 4 uint8_t
+ * UVersionInfo UCDVersion;  - version 4 uint8_t
+ * UVersionInfo formatVersion; - version of the format of the collation binary
+ *                               same formatVersion as in ucadata.icu's UDataInfo header
+ *                               (formatVersion 2.3)
+ *
+ * uint8_t reserved[84];  - currently unused
+ *
+ * -------------------------------------------------------------
+ *
+ * Inverse UCA is used for constructing collators from rules. It is always an individual file
+ * and always has a UDataInfo header. 
+ * here is the structure:
+ * 
+ * uint32_t byteSize; - size of inverse UCA image in bytes
+ * uint32_t tableSize; - length of inverse table (number of uint32_t[3] rows)
+ * uint32_t contsSize; - size of continuation table (number of UChars in table)
+ *
+ * uint32_t table; - offset to inverse table (uint32_t *)
+ *                   Inverse table contains of rows of 3 uint32_t values. First two values are CE and a possible continuation
+ *                   the third value is either a code unit (if there is only one code unit for element) or an index to continuation 
+ *                   (number of code units combined with an index).
+ *                   table. If more than one codepoint have the same CE, continuation table contains code units separated by FFFF and final
+ *                   code unit sequence for a CE is terminated by FFFE.
+ * uint32_t conts; - offset to continuation table (uint16_t *). Contains code units that transform to a same CE.
+ *
+ * UVersionInfo UCAVersion; -  version of the UCA, read from file 4 uint8_t
+ * uint8_t padding[8]; - padding 8 uint8_t
+ * Header is followed by the table and continuation table.
+*/
+
+/* let us know whether reserved fields are reset to zero or junked */
+#define UCOL_HEADER_MAGIC 0x20030618
 
 /* UDataInfo for UCA mapping table */
-static const UDataInfo ucaDataInfo={
-    sizeof(UDataInfo),
-    0,
+/* dataFormat="UCol"            */
+#define UCA_DATA_FORMAT_0 ((uint8_t)0x55)
+#define UCA_DATA_FORMAT_1 ((uint8_t)0x43)
+#define UCA_DATA_FORMAT_2 ((uint8_t)0x6f)
+#define UCA_DATA_FORMAT_3 ((uint8_t)0x6c)
 
-    U_IS_BIG_ENDIAN,
-    U_CHARSET_FAMILY,
-    sizeof(UChar),
-    0,
-
-    {0x55, 0x43, 0x6f, 0x6c},     /* dataFormat="UCol"            */
-    /* 03/26/2002 bumped up version since format has changed */
-    {2, 0, 0, 0},                 /* formatVersion                */
-    {3, 0, 0, 0}                  /* dataVersion = Unicode Version*/
-};
+#define UCA_FORMAT_VERSION_0 ((uint8_t)2)
+#define UCA_FORMAT_VERSION_1 ((uint8_t)3)
+#define UCA_FORMAT_VERSION_2 ((uint8_t)0)
+#define UCA_FORMAT_VERSION_3 ((uint8_t)0)
 
 /* UDataInfo for inverse UCA table */
-static const UDataInfo invUcaDataInfo={
-    sizeof(UDataInfo),
-    0,
+/* dataFormat="InvC"            */
+#define INVUCA_DATA_FORMAT_0 ((uint8_t)0x49)
+#define INVUCA_DATA_FORMAT_1 ((uint8_t)0x6E)
+#define INVUCA_DATA_FORMAT_2 ((uint8_t)0x76)
+#define INVUCA_DATA_FORMAT_3 ((uint8_t)0x43)
 
-    U_IS_BIG_ENDIAN,
-    U_CHARSET_FAMILY,
-    sizeof(UChar),
-    0,
-
-    {0x49, 0x6E, 0x76, 0x43},     /* dataFormat="InvC"            */
-    /* 03/26/2002 bumped up version since format has changed */
-    {2, 0, 0, 0},                 /* formatVersion                */
-    {3, 0, 0, 0}                  /* dataVersion = Unicode Version*/
-};
+#define INVUCA_FORMAT_VERSION_0 ((uint8_t)2)
+#define INVUCA_FORMAT_VERSION_1 ((uint8_t)1)
+#define INVUCA_FORMAT_VERSION_2 ((uint8_t)0)
+#define INVUCA_FORMAT_VERSION_3 ((uint8_t)0)
 
 /* This is the size of the stack allocated buffer for sortkey generation and similar operations */
 /* if it is too small, heap allocation will occur.*/
@@ -72,7 +201,7 @@ static const UDataInfo invUcaDataInfo={
 #define UCOL_SECONDARY_MAX_BUFFER UCOL_MAX_BUFFER
 #define UCOL_TERTIARY_MAX_BUFFER UCOL_MAX_BUFFER
 #define UCOL_CASE_MAX_BUFFER UCOL_MAX_BUFFER/4
-#define UCOL_QUAD_MAX_BUFFER UCOL_MAX_BUFFER
+#define UCOL_QUAD_MAX_BUFFER 2*UCOL_MAX_BUFFER
 
 #define UCOL_NORMALIZATION_GROWTH 2
 #define UCOL_NORMALIZATION_MAX_BUFFER UCOL_MAX_BUFFER*UCOL_NORMALIZATION_GROWTH
@@ -125,10 +254,14 @@ minimum number for special Jamo
                               /* UCOL_WAS_HIRAGANA - set to TRUE if there was a Hiragana */
                               /* otherwise set to false                                  */
 #define UCOL_WAS_HIRAGANA   32 
+                              /* UCOL_USE_ITERATOR - set this if collIterate uses a */
+                              /* character iterator instead of simply accessing string */
+                              /* by index */
+#define UCOL_USE_ITERATOR   64
 
 #define NFC_ZERO_CC_BLOCK_LIMIT_  0x300
 
-struct collIterate {
+typedef struct collIterate {
   UChar *string; /* Original string */
   /* UChar *start;  Pointer to the start of the source string. Either points to string
                     or to writableBuffer */
@@ -145,7 +278,9 @@ struct collIterate {
   uint8_t   origFlags;
   uint32_t CEs[UCOL_EXPAND_CE_BUFFER_SIZE]; /* This is where we store CEs */
   UChar stackWritableBuffer[UCOL_WRITABLE_BUFFER_SIZE]; /* A writable buffer. */
-};
+  UCharIterator *iterator;
+  /*int32_t iteratorIndex;*/
+} collIterate;
 
 /* 
 struct used internally in getSpecial*CE.
@@ -158,10 +293,12 @@ struct collIterateState {
     uint32_t  buffersize;
     uint8_t   flags;
     uint8_t   origFlags;
+    uint32_t   iteratorIndex;
+    int32_t    iteratorMove;
 };
 
 U_CAPI void U_EXPORT2 
-init_collIterate(const UCollator *collator, const UChar *sourceString, int32_t sourceLen, collIterate *s);
+uprv_init_collIterate(const UCollator *collator, const UChar *sourceString, int32_t sourceLen, collIterate *s);
 
 
 struct UCollationElements
@@ -304,7 +441,7 @@ void *ucol_getABuffer(const UCollator *coll, uint32_t size);
 
 /* worker function for generating sortkeys */
 U_CFUNC
-int32_t
+int32_t U_CALLCONV
 ucol_calcSortKey(const    UCollator    *coll,
         const    UChar        *source,
         int32_t        sourceLength,
@@ -314,7 +451,7 @@ ucol_calcSortKey(const    UCollator    *coll,
         UErrorCode *status);
 
 U_CFUNC
-int32_t
+int32_t U_CALLCONV
 ucol_calcSortKeySimpleTertiary(const    UCollator    *coll,
         const    UChar        *source,
         int32_t        sourceLength,
@@ -340,6 +477,13 @@ ucol_getSortKeySize(const UCollator *coll, collIterate *s,
 U_CAPI uint8_t* U_EXPORT2 
 ucol_cloneRuleData(const UCollator *coll, int32_t *length, UErrorCode *status);
 
+/**
+ * Used to set requested and valid locales on a collator returned by the collator
+ * service.
+ */
+U_CAPI void U_EXPORT2
+ucol_setReqValidLocales(UCollator *coll, char *requestedLocaleToAdopt, char *validLocaleToAdopt);
+
 #define UCOL_SPECIAL_FLAG 0xF0000000
 #define UCOL_TAG_SHIFT 24
 #define UCOL_TAG_MASK 0x0F000000
@@ -402,11 +546,6 @@ ucol_cloneRuleData(const UCollator *coll, int32_t *length, UErrorCode *status);
 #define getExpansionSuffix(coleiter) ((coleiter)->iteratordata_.CEpos - (coleiter)->iteratordata_.toReturn)
 #define setExpansionSuffix(coleiter, offset) ((coleiter)->iteratordata_.toReturn = (coleiter)->iteratordata_.CEpos - leftoverces)
 
-#define UCA_DATA_TYPE "dat"
-#define UCA_DATA_NAME "ucadata"
-#define INVC_DATA_TYPE "dat"
-#define INVC_DATA_NAME "invuca"
-
 /* This is an enum that lists magic special byte values from the fractional UCA */
 /* TODO: all the #defines that refer to special byte values from the UCA should be changed to point here */
 
@@ -419,23 +558,25 @@ enum {
     UCOL_BYTE_FIRST_TAILORED = 0x04,
     UCOL_BYTE_COMMON = 0x05,
     UCOL_BYTE_FIRST_UCA = UCOL_BYTE_COMMON,
+    UCOL_CODAN_PLACEHOLDER = 0x27,
     UCOL_BYTE_LAST_LATIN_PRIMARY = 0x4C,
     UCOL_BYTE_FIRST_NON_LATIN_PRIMARY = 0x4D,
     UCOL_BYTE_UNSHIFTED_MAX = 0xFF
 }; 
 
+#if 0
 #define UCOL_RESET_TOP_VALUE                0x9F000303
 #define UCOL_FIRST_PRIMARY_IGNORABLE        0x00008705
 #define UCOL_LAST_PRIMARY_IGNORABLE         0x0000DD05
-#define UCOL_LAST_PRIMARY_IGNORABLE_CONT    0x0000C1C0
+#define UCOL_LAST_PRIMARY_IGNORABLE_CONT    0x000051C0
 #define UCOL_FIRST_SECONDARY_IGNORABLE      0x00000000
 #define UCOL_LAST_SECONDARY_IGNORABLE       0x00000500
 #define UCOL_FIRST_TERTIARY_IGNORABLE       0x00000000
 #define UCOL_LAST_TERTIARY_IGNORABLE        0x00000000
 #define UCOL_FIRST_VARIABLE                 0x05070505
-#define UCOL_LAST_VARIABLE                  0x13CF0505
-#define UCOL_FIRST_NON_VARIABLE             0x16200505
-#define UCOL_LAST_NON_VARIABLE              0x767C0505
+#define UCOL_LAST_VARIABLE                  0x179B0505
+#define UCOL_FIRST_NON_VARIABLE             0x1A200505
+#define UCOL_LAST_NON_VARIABLE              0x7B41058F
 
 #define UCOL_NEXT_TOP_VALUE                 0xE8960303
 #define UCOL_NEXT_FIRST_PRIMARY_IGNORABLE   0x00008905
@@ -445,10 +586,11 @@ enum {
 #define UCOL_NEXT_FIRST_TERTIARY_IGNORABLE  0x00000000
 #define UCOL_NEXT_LAST_TERTIARY_IGNORABLE   0x00000000
 #define UCOL_NEXT_FIRST_VARIABLE            0x05090505
-#define UCOL_NEXT_LAST_VARIABLE             0x16200505
+#define UCOL_NEXT_LAST_VARIABLE             0x1A200505
 
 #define PRIMARY_IMPLICIT_MIN 0xE8000000
 #define PRIMARY_IMPLICIT_MAX 0xF0000000
+#endif
 
 /* These constants can be changed - sortkey size is affected by them */
 #define UCOL_PROPORTION2 0.5
@@ -493,6 +635,11 @@ enum {
 
 #define UCOL_TERT_CASE_MASK   0xFF
 
+#define UCOL_ENDOFLATINONERANGE 0xFF
+#define UCOL_LATINONETABLELEN   (UCOL_ENDOFLATINONERANGE+50)
+#define UCOL_BAIL_OUT_CE      0xFF000000
+
+
 typedef enum {
     NOT_FOUND_TAG = 0,
     EXPANSION_TAG = 1,       /* This code point results in an expansion */
@@ -510,6 +657,9 @@ typedef enum {
     LONG_PRIMARY_TAG = 12,   /* This is a three byte primary with starting secondaries and tertiaries */
                              /* It fits in a single 32 bit CE and is used instead of expansion to save */
                              /* space without affecting the performance (hopefully) */
+                             
+    DIGIT_TAG = 13,          /* COllate Digits As Numbers (CODAN) implementation */
+    
     CE_TAGS_COUNT
 } UColCETags;
 
@@ -522,22 +672,63 @@ typedef enum {
 
 typedef struct {
       uint32_t variableTopValue;
-      UColAttributeValue frenchCollation;
-      UColAttributeValue alternateHandling; /* attribute for handling variable elements*/
-      UColAttributeValue caseFirst;         /* who goes first, lower case or uppercase */
-      UColAttributeValue caseLevel;         /* do we have an extra case level */
-      UColAttributeValue normalizationMode; /* attribute for normalization */
-      UColAttributeValue strength;          /* attribute for strength */
-      UColAttributeValue hiraganaQ;         /* attribuge for special Hiragana */
+      /*UColAttributeValue*/ int32_t frenchCollation;
+      /*UColAttributeValue*/ int32_t alternateHandling; /* attribute for handling variable elements*/
+      /*UColAttributeValue*/ int32_t caseFirst;         /* who goes first, lower case or uppercase */
+      /*UColAttributeValue*/ int32_t caseLevel;         /* do we have an extra case level */
+      /*UColAttributeValue*/ int32_t normalizationMode; /* attribute for normalization */
+      /*UColAttributeValue*/ int32_t strength;          /* attribute for strength */
+      /*UColAttributeValue*/ int32_t hiraganaQ;         /* attribute for special Hiragana */
+      /*UColAttributeValue*/ int32_t numericCollation;  /* attribute for numeric collation */
+      uint32_t reserved[15];                 /* for future use */
 } UColOptionSet;
 
+typedef struct {
+  uint32_t UCA_FIRST_TERTIARY_IGNORABLE[2];       /*0x00000000*/
+  uint32_t UCA_LAST_TERTIARY_IGNORABLE[2];        /*0x00000000*/
+  uint32_t UCA_FIRST_PRIMARY_IGNORABLE[2];        /*0x00008705*/
+  uint32_t UCA_FIRST_SECONDARY_IGNORABLE[2];      /*0x00000000*/
+  uint32_t UCA_LAST_SECONDARY_IGNORABLE[2];       /*0x00000500*/
+  uint32_t UCA_LAST_PRIMARY_IGNORABLE[2];         /*0x0000DD05*/
+  uint32_t UCA_FIRST_VARIABLE[2];                 /*0x05070505*/
+  uint32_t UCA_LAST_VARIABLE[2];                  /*0x13CF0505*/
+  uint32_t UCA_FIRST_NON_VARIABLE[2];             /*0x16200505*/
+  uint32_t UCA_LAST_NON_VARIABLE[2];              /*0x767C0505*/
+  uint32_t UCA_RESET_TOP_VALUE[2];                /*0x9F000303*/
+  uint32_t UCA_FIRST_IMPLICIT[2];
+  uint32_t UCA_LAST_IMPLICIT[2]; 
+  uint32_t UCA_FIRST_TRAILING[2];
+  uint32_t UCA_LAST_TRAILING[2]; 
+
+#if 0
+  uint32_t UCA_NEXT_TOP_VALUE[2];                 /*0xE8960303*/
+  uint32_t UCA_NEXT_FIRST_PRIMARY_IGNORABLE;   /*0x00008905*/
+  uint32_t UCA_NEXT_LAST_PRIMARY_IGNORABLE;    /*0x03000303*/
+  uint32_t UCA_NEXT_FIRST_SECONDARY_IGNORABLE; /*0x00008705*/
+  uint32_t UCA_NEXT_LAST_SECONDARY_IGNORABLE;  /*0x00000500*/
+  uint32_t UCA_NEXT_FIRST_TERTIARY_IGNORABLE;  /*0x00000000*/
+  uint32_t UCA_NEXT_LAST_TERTIARY_IGNORABLE;   /*0x00000000*/
+  uint32_t UCA_NEXT_FIRST_VARIABLE;            /*0x05090505*/
+  uint32_t UCA_NEXT_LAST_VARIABLE;             /*0x16200505*/
+#endif
+
+  uint32_t UCA_PRIMARY_TOP_MIN;
+  uint32_t UCA_PRIMARY_IMPLICIT_MIN; /*0xE8000000*/
+  uint32_t UCA_PRIMARY_IMPLICIT_MAX; /*0xF0000000*/
+  uint32_t UCA_PRIMARY_TRAILING_MIN; /*0xE8000000*/
+  uint32_t UCA_PRIMARY_TRAILING_MAX; /*0xF0000000*/
+  uint32_t UCA_PRIMARY_SPECIAL_MIN; /*0xE8000000*/
+  uint32_t UCA_PRIMARY_SPECIAL_MAX; /*0xF0000000*/
+} UCAConstants;
+
 typedef struct {
       int32_t size;
       /* all the offsets are in bytes */
       /* to get the address add to the header address and cast properly */
       uint32_t options; /* these are the default options for the collator */
+      uint32_t UCAConsts; /* structure which holds values for indirect positioning and implicit ranges */
       uint32_t contractionUCACombos;        /* this one is needed only for UCA, to copy the appropriate contractions */
-      uint32_t unusedReserved1;         /* reserved for future use */
+      uint32_t magic;            /* magic number - lets us know whether reserved data is reset or junked */
       uint32_t mappingPosition;  /* const uint8_t *mappingPosition; */
       uint32_t expansion;        /* uint32_t *expansion;            */
       uint32_t contractionIndex; /* UChar *contractionIndex;        */
@@ -556,12 +747,17 @@ typedef struct {
       uint32_t contrEndCP;          /* hash table of final code points  */
                                     /*   in contractions.               */
 
-      int32_t CEcount;
+      int32_t contractionUCACombosSize;     /* number of UCA contraction items. */
+                                            /*Length is contractionUCACombosSize*contractionUCACombosWidth*sizeof(UChar) */
       UBool jamoSpecial;                    /* is jamoSpecial */
-      uint8_t padding[3];                   /* for guaranteed alignment */
+      UBool isBigEndian;                    /* is this data big endian? from the UDataInfo header*/
+      uint8_t charSetFamily;                /* what is the charset family of this data from the UDataInfo header*/
+      uint8_t contractionUCACombosWidth;    /* width of UCA combos field */
       UVersionInfo version;
-      char charsetName[32];                 /* for charset CEs */
-      uint8_t reserved[64];                 /* for future use */
+      UVersionInfo UCAVersion;              /* version of the UCA, read from file */
+      UVersionInfo UCDVersion;              /* UCD version, obtained by u_getUnicodeVersion */
+      UVersionInfo formatVersion;           /* format version from the UDataInfo header */
+      uint8_t reserved[84];                 /* for future use */
 } UCATableHeader;
 
 #define U_UNKNOWN_STATE 0
@@ -639,7 +835,9 @@ typedef struct {
   uint32_t contsSize;
   uint32_t table;
   uint32_t conts;
-} InverseTableHeader;
+  UVersionInfo UCAVersion;              /* version of the UCA, read from file */
+  uint8_t padding[8];
+} InverseUCATableHeader;
 
 typedef int32_t U_CALLCONV
 SortKeyGenerator(const    UCollator    *coll,
@@ -650,56 +848,28 @@ SortKeyGenerator(const    UCollator    *coll,
         UBool allocatePrimary,
         UErrorCode *status);
 
+typedef void U_CALLCONV
+ResourceCleaner(UCollator *coll);
+
+
 struct UCollator {
-    UBool freeOptionsOnClose;
-    UColOptionSet *options;
+    UColOptionSet  *options;
     SortKeyGenerator *sortKeyGen;
-    UBool freeOnClose;
+    uint32_t *latinOneCEs;
+    char* validLocale;
     char* requestedLocale;
+    const UChar *rules;
+    const UCollator *UCA;
+    ResourceCleaner *resCleaner;
     UResourceBundle *rb;
-    UResourceBundle *binary;
+    UResourceBundle *elements;
     const UCATableHeader *image;
-    /*CompactEIntArray *mapping;*/
-    UTrie *mapping;
+    UTrie mapping;
     const uint32_t *latinOneMapping;
     const uint32_t *expansion;
-    const UChar *contractionIndex;
+    const UChar    *contractionIndex;
     const uint32_t *contractionCEs;
-    const uint8_t *scriptOrder;
-    uint8_t caseSwitch;
-    uint8_t tertiaryCommon;
-    uint8_t tertiaryMask;
-    int32_t tertiaryAddition; /* when switching case, we need to add or subtract different values */
-    uint8_t tertiaryTop; /* Upper range when compressing */
-    uint8_t tertiaryBottom; /* Upper range when compressing */
-    uint8_t tertiaryTopCount;
-    uint8_t tertiaryBottomCount;
-
-    uint32_t variableTopValue;
-    UColAttributeValue frenchCollation;
-    UColAttributeValue alternateHandling; /* attribute for handling variable elements*/
-    UColAttributeValue caseFirst;         /* who goes first, lower case or uppercase */
-    UColAttributeValue caseLevel;         /* do we have an extra case level */
-    UColAttributeValue normalizationMode; /* attribute for normalization */
-    UColAttributeValue strength;          /* attribute for strength */
-    UColAttributeValue hiraganaQ;         /* attribute for Hiragana */
-    UBool variableTopValueisDefault;
-    UBool frenchCollationisDefault;
-    UBool alternateHandlingisDefault; /* attribute for handling variable elements*/
-    UBool caseFirstisDefault;         /* who goes first, lower case or uppercase */
-    UBool caseLevelisDefault;         /* do we have an extra case level */
-    UBool normalizationModeisDefault; /* attribute for normalization */
-    UBool strengthisDefault;          /* attribute for strength */
-    UBool hiraganaQisDefault;         /* attribute for Hiragana */
-    UBool hasRealData;                /* some collators have only options, like French, no rules */
-                                      /* to speed up things, we use the UCA image, but we don't want it */
-                                      /* to run around */
-    const UChar *rules;
-    int32_t rulesLength;
-    UBool freeRulesOnClose;
-
-    UDataInfo dataInfo;               /* Data info of UCA table */
-    UErrorCode errorCode;             /* internal error code */
+    /*const uint8_t  *scriptOrder;*/
 
     const uint32_t *endExpansionCE;    /* array of last ces in an expansion ce.
                                           corresponds to expansionCESize */
@@ -712,40 +882,102 @@ struct UCollator {
     const uint8_t *contrEndCP;         /* Contraction ending chars hash table */
     UChar          minUnsafeCP;        /* Smallest unsafe Code Point. */
     UChar          minContrEndCP;      /* Smallest code point at end of a contraction */
+
+    int32_t rulesLength;
+    int32_t latinOneTableLen;
+
+    uint32_t variableTopValue;
+    UColAttributeValue frenchCollation;
+    UColAttributeValue alternateHandling; /* attribute for handling variable elements*/
+    UColAttributeValue caseFirst;         /* who goes first, lower case or uppercase */
+    UColAttributeValue caseLevel;         /* do we have an extra case level */
+    UColAttributeValue normalizationMode; /* attribute for normalization */
+    UColAttributeValue strength;          /* attribute for strength */
+    UColAttributeValue hiraganaQ;         /* attribute for Hiragana */
+    UColAttributeValue numericCollation;
+    UBool variableTopValueisDefault;
+    UBool frenchCollationisDefault;
+    UBool alternateHandlingisDefault; /* attribute for handling variable elements*/
+    UBool caseFirstisDefault;         /* who goes first, lower case or uppercase */
+    UBool caseLevelisDefault;         /* do we have an extra case level */
+    UBool normalizationModeisDefault; /* attribute for normalization */
+    UBool strengthisDefault;          /* attribute for strength */
+    UBool hiraganaQisDefault;         /* attribute for Hiragana */
+    UBool numericCollationisDefault;
+    UBool hasRealData;                /* some collators have only options, like French, no rules */
+                                      /* to speed up things, we use the UCA image, but we don't want it */
+                                      /* to run around */
+
+    UBool freeOnClose;
+    UBool freeOptionsOnClose;
+    UBool freeRulesOnClose;
+    UBool freeImageOnClose;
+
+    UBool latinOneUse;
+    UBool latinOneRegenTable;
+    UBool latinOneFailed;
+
+    int32_t tertiaryAddition; /* when switching case, we need to add or subtract different values */
+    uint8_t caseSwitch;
+    uint8_t tertiaryCommon;
+    uint8_t tertiaryMask;
+    uint8_t tertiaryTop; /* Upper range when compressing */
+    uint8_t tertiaryBottom; /* Upper range when compressing */
+    uint8_t tertiaryTopCount;
+    uint8_t tertiaryBottomCount;
+
+    UVersionInfo dataVersion;               /* Data info of UCA table */
 };
 
 U_CDECL_END
 
 /* various internal functions */
 
+/* do not close UCA returned by ucol_initUCA! */
 U_CFUNC
-void ucol_initUCA(UErrorCode *status);
+UCollator* ucol_initUCA(UErrorCode *status);
 
 U_CFUNC
-UCollator* ucol_initCollator(const UCATableHeader *image, UCollator *fillIn, UErrorCode *status);
+UCollator* ucol_initCollator(const UCATableHeader *image, UCollator *fillIn, const UCollator *UCA, UErrorCode *status);
 
 U_CFUNC
 void ucol_setOptionsFromHeader(UCollator* result, UColOptionSet * opts, UErrorCode *status);
 
+U_CFUNC
+UCollator* ucol_open_internal(const char* loc, UErrorCode* status);
+
 #if 0
 U_CFUNC
 void ucol_putOptionsToHeader(UCollator* result, UColOptionSet * opts, UErrorCode *status);
 #endif
 
 U_CFUNC
-void ucol_updateInternalState(UCollator *coll);
+void ucol_updateInternalState(UCollator *coll, UErrorCode *status);
 
 U_CAPI uint32_t U_EXPORT2 ucol_getFirstCE(const UCollator *coll, UChar u, UErrorCode *status);
-U_CAPI char* U_EXPORT2 ucol_sortKeyToString(const UCollator *coll, const uint8_t *sortkey, char *buffer, uint32_t *len);
-U_CAPI UBool U_EXPORT2 isTailored(const UCollator *coll, const UChar u, UErrorCode *status);
+U_CAPI UBool U_EXPORT2 ucol_isTailored(const UCollator *coll, const UChar u, UErrorCode *status);
+
+U_CAPI const InverseUCATableHeader* U_EXPORT2 ucol_initInverseUCA(UErrorCode *status);
+
+U_CAPI void U_EXPORT2 
+uprv_uca_initImplicitConstants(int32_t minPrimary, int32_t maxPrimary, UErrorCode *status);
+
+U_CAPI uint32_t U_EXPORT2
+uprv_uca_getImplicitFromRaw(UChar32 cp);
+
+U_CAPI uint32_t U_EXPORT2
+uprv_uca_getImplicitPrimary(UChar32 cp);
+
+U_CAPI UChar32 U_EXPORT2
+uprv_uca_getRawFromImplicit(uint32_t implicit);
+
+U_CAPI UChar32 U_EXPORT2
+uprv_uca_getRawFromCodePoint(UChar32 i);
+
+U_CAPI UChar32 U_EXPORT2
+uprv_uca_getCodePointFromRaw(UChar32 i);
+
 
-U_CAPI const InverseTableHeader* U_EXPORT2 ucol_initInverseUCA(UErrorCode *status);
-U_CAPI int32_t U_EXPORT2 ucol_inv_getNextCE(uint32_t CE, uint32_t contCE,
-                                            uint32_t *nextCE, uint32_t *nextContCE,
-                                            uint32_t strength);
-U_CAPI int32_t U_EXPORT2 ucol_inv_getPrevCE(uint32_t CE, uint32_t contCE,
-                                            uint32_t *prevCE, uint32_t *prevContCE,
-                                            uint32_t strength);
 
 #ifdef XP_CPLUSPLUS
 /*
@@ -766,7 +998,7 @@ static inline UBool ucol_unsafeCP(UChar c, const UCollator *coll) {
 
     hash = c;
     if (hash >= UCOL_UNSAFECP_TABLE_SIZE*8) {
-      if(UTF_IS_TRAIL(c)) {
+      if(UTF_IS_LEAD(c) || UTF_IS_TRAIL(c)) {
             /*  Trail surrogate                     */
             /*  These are always considered unsafe. */
             return TRUE;
@@ -779,5 +1011,6 @@ static inline UBool ucol_unsafeCP(UChar c, const UCollator *coll) {
 #endif /* XP_CPLUSPLUS */
 
 
-#endif
+#endif /* #if !UCONFIG_NO_COLLATION */
 
+#endif
