@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-*   Copyright (C) 2001, International Business Machines
+*   Copyright (C) 2001-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ******************************************************************************
 *
@@ -13,10 +13,14 @@
 *                         instead of calling the equivalent c++ api (coleitr.h)
 ******************************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucoleitr.h"
 #include "unicode/ustring.h"
-#include "ucol_imp.h"
 #include "unicode/sortkey.h"
+#include "ucol_imp.h"
 #include "cmemory.h"
 
 U_NAMESPACE_USE
@@ -27,19 +31,6 @@ typedef struct collIterate collIterator;
 
 /* public methods ---------------------------------------------------- */
 
-/**
-* Since this is going to be deprecated, I'll leave it as it is
-*/
-U_CAPI int32_t U_EXPORT2
-ucol_keyHashCode(const uint8_t *key, 
-                       int32_t  length)
-{
-
-    CollationKey newKey(key, length);
-    return newKey.hashCode();
-}
-
-
 U_CAPI UCollationElements* U_EXPORT2
 ucol_openElements(const UCollator  *coll,
                   const UChar      *text,
@@ -53,6 +44,11 @@ ucol_openElements(const UCollator  *coll,
   }
 
   result = (UCollationElements *)uprv_malloc(sizeof(UCollationElements));
+  /* test for NULL */
+  if (result == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      return NULL;
+  }
 
   result->reset_   = TRUE;
   result->isWritable = FALSE;
@@ -60,7 +56,7 @@ ucol_openElements(const UCollator  *coll,
   if (text == NULL) {
       textLength = 0;
   }
-  init_collIterate(coll, text, textLength, &result->iteratordata_);
+  uprv_init_collIterate(coll, text, textLength, &result->iteratordata_);
 
   return result;
 }
@@ -106,15 +102,16 @@ U_CAPI int32_t U_EXPORT2
 ucol_next(UCollationElements *elems, 
           UErrorCode         *status)
 {
-  uint32_t result;
+  int32_t result;
   if (U_FAILURE(*status)) {
     return UCOL_NULLORDER;
   }
 
   elems->reset_ = FALSE;
 
-  result = ucol_getNextCE(elems->iteratordata_.coll, &elems->iteratordata_, 
-                          status);
+  result = (int32_t)ucol_getNextCE(elems->iteratordata_.coll,
+                                   &elems->iteratordata_, 
+                                   status);
   
   if (result == UCOL_NO_MORE_CES) {
     result = UCOL_NULLORDER;
@@ -131,7 +128,7 @@ ucol_previous(UCollationElements *elems,
   }
   else
   {
-    uint32_t result;
+    int32_t result;
 
     if (elems->reset_ && 
         (elems->iteratordata_.pos == elems->iteratordata_.string)) {
@@ -146,8 +143,9 @@ ucol_previous(UCollationElements *elems,
 
     elems->reset_ = FALSE;
 
-    result = ucol_getPrevCE(elems->iteratordata_.coll, &(elems->iteratordata_), 
-                            status);
+    result = (int32_t)ucol_getPrevCE(elems->iteratordata_.coll,
+                                     &(elems->iteratordata_), 
+                                     status);
 
     if (result == UCOL_NO_MORE_CES) {
       result = UCOL_NULLORDER;
@@ -186,7 +184,7 @@ ucol_setText(      UCollationElements *elems,
   }
 
   elems->isWritable = FALSE;
-  init_collIterate(elems->iteratordata_.coll, text, textLength, 
+  uprv_init_collIterate(elems->iteratordata_.coll, text, textLength, 
                    &elems->iteratordata_);
 
   elems->reset_   = TRUE;
@@ -205,10 +203,10 @@ ucol_getOffset(const UCollationElements *elems)
       if (ci->fcdPosition == NULL) {
         return 0;
       }
-      return ci->fcdPosition - ci->string;
+      return (int32_t)(ci->fcdPosition - ci->string);
   }
   else {
-      return ci->pos - ci->string;
+      return (int32_t)(ci->pos - ci->string);
   }
 }
 
@@ -234,9 +232,27 @@ ucol_setOffset(UCollationElements    *elems,
       ci->flags |= UCOL_ITER_HASLEN;
   }
   ci->fcdPosition = NULL;
+  elems->reset_ = FALSE;
 }
 
+U_CAPI int32_t U_EXPORT2
+ucol_primaryOrder (int32_t order) 
+{
+  order &= UCOL_PRIMARYMASK;
+  return (order >> UCOL_PRIMARYORDERSHIFT);
+}
 
+U_CAPI int32_t U_EXPORT2
+ucol_secondaryOrder (int32_t order) 
+{
+  order &= UCOL_SECONDARYMASK;
+  return (order >> UCOL_SECONDARYORDERSHIFT);
+}
 
+U_CAPI int32_t U_EXPORT2
+ucol_tertiaryOrder (int32_t order) 
+{
+  return (order & UCOL_TERTIARYMASK);
+}
 
-
+#endif /* #if !UCONFIG_NO_COLLATION */
