@@ -1,5 +1,5 @@
 /*
-* Copyright (C) {1997-1999}, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) {1997-2005}, International Business Machines Corporation and others. All Rights Reserved.
 *                                                                              *
 ********************************************************************************
 *
@@ -23,15 +23,19 @@
 #ifndef SIMPLETZ_H
 #define SIMPLETZ_H
 
+#include "unicode/utypes.h"
+
+/**
+ * \file 
+ * \brief C++ API: SimpleTimeZone is a concrete subclass of TimeZone.
+ */
+ 
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/timezone.h"
 
-struct StandardZone;
-struct DSTZone;
-
 U_NAMESPACE_BEGIN
 
-class TimeZone;
-
 /**
  * <code>SimpleTimeZone</code> is a concrete subclass of <code>TimeZone</code>
  * that represents a time zone for use with a Gregorian calendar. This
@@ -46,7 +50,7 @@ class TimeZone;
  * @see      Calendar
  * @see      GregorianCalendar
  * @see      TimeZone
- * @author   David Goldsmith, Mark Davis, Chen-Lieh Huang, Alan Liu
+ * @author   D. Goldsmith, Mark Davis, Chen-Lieh Huang, Alan Liu
  */
 class U_I18N_API SimpleTimeZone: public TimeZone {
 public:
@@ -60,6 +64,7 @@ public:
      * UTC time.  Although it might seem that all times could be
      * converted to wall time, thus eliminating the need for this
      * parameter, this is not the case.
+     * @stable ICU 2.0
      */
     enum TimeMode {
         WALL_TIME = 0,
@@ -69,19 +74,21 @@ public:
 
     /**
      * Copy constructor
-     * @stable
+     * @param source the object to be copied.
+     * @stable ICU 2.0
      */
     SimpleTimeZone(const SimpleTimeZone& source);
 
     /**
      * Default assignment operator
-     * @stable
+     * @param right    the object to be copied.
+     * @stable ICU 2.0
      */
     SimpleTimeZone& operator=(const SimpleTimeZone& right);
 
     /**
      * Destructor
-     * @stable
+     * @stable ICU 2.0
      */
     virtual ~SimpleTimeZone();
 
@@ -92,7 +99,7 @@ public:
      * @param that  The SimpleTimeZone object to be compared with.
      * @return      True if the given time zone is equal to this time zone; false
      *              otherwise.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const TimeZone& that) const;
 
@@ -105,7 +112,7 @@ public:
      * @param rawOffsetGMT  The given base time zone offset to GMT.
      * @param ID         The timezone ID which is obtained from
      *                   TimeZone.getAvailableIDs.
-     * @stable
+     * @stable ICU 2.0
      */
     SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID);
 
@@ -126,7 +133,92 @@ public:
      * @param ID                The new SimpleTimeZone's time zone ID.
      * @param savingsStartMonth The daylight savings starting month. Month is
      *                          0-based. eg, 0 for January.
-     * @param savingsStartDay   The daylight savings starting
+     * @param savingsStartDayOfWeekInMonth   The daylight savings starting
+     *                          day-of-week-in-month. See setStartRule() for a
+     *                          complete explanation.
+     * @param savingsStartDayOfWeek The daylight savings starting day-of-week.
+     *                          See setStartRule() for a complete explanation.
+     * @param savingsStartTime  The daylight savings starting time, expressed as the
+     *                          number of milliseconds after midnight.
+     * @param savingsEndMonth   The daylight savings ending month. Month is
+     *                          0-based. eg, 0 for January.
+     * @param savingsEndDayOfWeekInMonth     The daylight savings ending day-of-week-in-month.
+     *                          See setStartRule() for a complete explanation.
+     * @param savingsEndDayOfWeek The daylight savings ending day-of-week.
+     *                          See setStartRule() for a complete explanation.
+     * @param savingsEndTime    The daylight savings ending time, expressed as the
+     *                          number of milliseconds after midnight.
+     * @param status            An UErrorCode to receive the status.
+     * @stable ICU 2.0
+     */
+    SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID,
+        int8_t savingsStartMonth, int8_t savingsStartDayOfWeekInMonth,
+        int8_t savingsStartDayOfWeek, int32_t savingsStartTime,
+        int8_t savingsEndMonth, int8_t savingsEndDayOfWeekInMonth,
+        int8_t savingsEndDayOfWeek, int32_t savingsEndTime,
+        UErrorCode& status);
+    /**
+     * Construct a SimpleTimeZone with the given raw GMT offset, time zone ID,
+     * and times to start and end daylight savings time. To create a TimeZone that
+     * doesn't observe daylight savings time, don't use this constructor; use
+     * SimpleTimeZone(rawOffset, ID) instead. Normally, you should use
+     * TimeZone.createInstance() to create a TimeZone instead of creating a
+     * SimpleTimeZone directly with this constructor.
+     * <P>
+     * Various types of daylight-savings time rules can be specfied by using different
+     * values for startDay and startDayOfWeek and endDay and endDayOfWeek.  For a
+     * complete explanation of how these parameters work, see the documentation for
+     * setStartRule().
+     *
+     * @param rawOffsetGMT      The new SimpleTimeZone's raw GMT offset
+     * @param ID                The new SimpleTimeZone's time zone ID.
+     * @param savingsStartMonth The daylight savings starting month. Month is
+     *                          0-based. eg, 0 for January.
+     * @param savingsStartDayOfWeekInMonth   The daylight savings starting
+     *                          day-of-week-in-month. See setStartRule() for a
+     *                          complete explanation.
+     * @param savingsStartDayOfWeek The daylight savings starting day-of-week.
+     *                          See setStartRule() for a complete explanation.
+     * @param savingsStartTime  The daylight savings starting time, expressed as the
+     *                          number of milliseconds after midnight.
+     * @param savingsEndMonth   The daylight savings ending month. Month is
+     *                          0-based. eg, 0 for January.
+     * @param savingsEndDayOfWeekInMonth     The daylight savings ending day-of-week-in-month.
+     *                          See setStartRule() for a complete explanation.
+     * @param savingsEndDayOfWeek The daylight savings ending day-of-week.
+     *                          See setStartRule() for a complete explanation.
+     * @param savingsEndTime    The daylight savings ending time, expressed as the
+     *                          number of milliseconds after midnight.
+     * @param savingsDST        The number of milliseconds added to standard time
+     *                          to get DST time. Default is one hour.
+     * @param status            An UErrorCode to receive the status.
+     * @stable ICU 2.0
+     */
+    SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID,
+        int8_t savingsStartMonth, int8_t savingsStartDayOfWeekInMonth,
+        int8_t savingsStartDayOfWeek, int32_t savingsStartTime,
+        int8_t savingsEndMonth, int8_t savingsEndDayOfWeekInMonth,
+        int8_t savingsEndDayOfWeek, int32_t savingsEndTime,
+        int32_t savingsDST, UErrorCode& status);
+
+    /**
+     * Construct a SimpleTimeZone with the given raw GMT offset, time zone ID,
+     * and times to start and end daylight savings time. To create a TimeZone that
+     * doesn't observe daylight savings time, don't use this constructor; use
+     * SimpleTimeZone(rawOffset, ID) instead. Normally, you should use
+     * TimeZone.createInstance() to create a TimeZone instead of creating a
+     * SimpleTimeZone directly with this constructor.
+     * <P>
+     * Various types of daylight-savings time rules can be specfied by using different
+     * values for startDay and startDayOfWeek and endDay and endDayOfWeek.  For a
+     * complete explanation of how these parameters work, see the documentation for
+     * setStartRule().
+     *
+     * @param rawOffsetGMT      The new SimpleTimeZone's raw GMT offset
+     * @param ID                The new SimpleTimeZone's time zone ID.
+     * @param savingsStartMonth The daylight savings starting month. Month is
+     *                          0-based. eg, 0 for January.
+     * @param savingsStartDayOfWeekInMonth   The daylight savings starting
      *                          day-of-week-in-month. See setStartRule() for a
      *                          complete explanation.
      * @param savingsStartDayOfWeek The daylight savings starting day-of-week.
@@ -137,7 +229,7 @@ public:
      *                          standard time, or UTC time. Default is local wall time.
      * @param savingsEndMonth   The daylight savings ending month. Month is
      *                          0-based. eg, 0 for January.
-     * @param savingsEndDay     The daylight savings ending day-of-week-in-month.
+     * @param savingsEndDayOfWeekInMonth     The daylight savings ending day-of-week-in-month.
      *                          See setStartRule() for a complete explanation.
      * @param savingsEndDayOfWeek The daylight savings ending day-of-week.
      *                          See setStartRule() for a complete explanation.
@@ -148,22 +240,8 @@ public:
      * @param savingsDST        The number of milliseconds added to standard time
      *                          to get DST time. Default is one hour.
      * @param status            An UErrorCode to receive the status.
-     * @stable
+     * @stable ICU 2.0
      */
-    SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID,
-        int8_t savingsStartMonth, int8_t savingsStartDayOfWeekInMonth,
-        int8_t savingsStartDayOfWeek, int32_t savingsStartTime,
-        int8_t savingsEndMonth, int8_t savingsEndDayOfWeekInMonth,
-        int8_t savingsEndDayOfWeek, int32_t savingsEndTime,
-        UErrorCode& status);
-
-    SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID,
-        int8_t savingsStartMonth, int8_t savingsStartDayOfWeekInMonth,
-        int8_t savingsStartDayOfWeek, int32_t savingsStartTime,
-        int8_t savingsEndMonth, int8_t savingsEndDayOfWeekInMonth,
-        int8_t savingsEndDayOfWeek, int32_t savingsEndTime,
-        int32_t savingsDST, UErrorCode& status);
-
     SimpleTimeZone(int32_t rawOffsetGMT, const UnicodeString& ID,
         int8_t savingsStartMonth, int8_t savingsStartDayOfWeekInMonth,
         int8_t savingsStartDayOfWeek, int32_t savingsStartTime,
@@ -178,7 +256,7 @@ public:
      * not to observe daylight savings time prior to that year; SimpleTimeZone doesn't
      * support historical daylight-savings-time rules.
      * @param year the daylight savings starting year.
-     * @stable
+     * @stable ICU 2.0
      */
     void setStartYear(int32_t year);
 
@@ -189,28 +267,74 @@ public:
      * setStartRule(TimeFields.APRIL, 1, TimeFields.SUNDAY, 2*60*60*1000);
      * The dayOfWeekInMonth and dayOfWeek parameters together specify how to calculate
      * the exact starting date.  Their exact meaning depend on their respective signs,
-     * allowing various types of rules to be constructed, as follows:<ul>
+     * allowing various types of rules to be constructed, as follows:
+     * <ul>
      *   <li>If both dayOfWeekInMonth and dayOfWeek are positive, they specify the
      *       day of week in the month (e.g., (2, WEDNESDAY) is the second Wednesday
-     *       of the month).
+     *       of the month).</li>
      *   <li>If dayOfWeek is positive and dayOfWeekInMonth is negative, they specify
      *       the day of week in the month counting backward from the end of the month.
-     *       (e.g., (-1, MONDAY) is the last Monday in the month)
+     *       (e.g., (-1, MONDAY) is the last Monday in the month)</li>
      *   <li>If dayOfWeek is zero and dayOfWeekInMonth is positive, dayOfWeekInMonth
      *       specifies the day of the month, regardless of what day of the week it is.
-     *       (e.g., (10, 0) is the tenth day of the month)
+     *       (e.g., (10, 0) is the tenth day of the month)</li>
      *   <li>If dayOfWeek is zero and dayOfWeekInMonth is negative, dayOfWeekInMonth
      *       specifies the day of the month counting backward from the end of the
      *       month, regardless of what day of the week it is (e.g., (-2, 0) is the
-     *       next-to-last day of the month).
+     *       next-to-last day of the month).</li>
      *   <li>If dayOfWeek is negative and dayOfWeekInMonth is positive, they specify the
      *       first specified day of the week on or after the specfied day of the month.
      *       (e.g., (15, -SUNDAY) is the first Sunday after the 15th of the month
-     *       [or the 15th itself if the 15th is a Sunday].)
+     *       [or the 15th itself if the 15th is a Sunday].)</li>
      *   <li>If dayOfWeek and DayOfWeekInMonth are both negative, they specify the
      *       last specified day of the week on or before the specified day of the month.
      *       (e.g., (-20, -TUESDAY) is the last Tuesday before the 20th of the month
-     *       [or the 20th itself if the 20th is a Tuesday].)</ul>
+     *       [or the 20th itself if the 20th is a Tuesday].)</li>
+     * </ul>
+     * @param month the daylight savings starting month. Month is 0-based.
+     * eg, 0 for January.
+     * @param dayOfWeekInMonth the daylight savings starting
+     * day-of-week-in-month. Please see the member description for an example.
+     * @param dayOfWeek the daylight savings starting day-of-week. Please see
+     * the member description for an example.
+     * @param time the daylight savings starting time. Please see the member
+     * description for an example.
+     * @param status An UErrorCode
+     * @stable ICU 2.0
+     */
+    void setStartRule(int32_t month, int32_t dayOfWeekInMonth, int32_t dayOfWeek,
+                      int32_t time, UErrorCode& status);
+    /**
+     * Sets the daylight savings starting rule. For example, in the U.S., Daylight Savings
+     * Time starts at the first Sunday in April, at 2 AM in standard time.
+     * Therefore, you can set the start rule by calling:
+     * setStartRule(TimeFields.APRIL, 1, TimeFields.SUNDAY, 2*60*60*1000);
+     * The dayOfWeekInMonth and dayOfWeek parameters together specify how to calculate
+     * the exact starting date.  Their exact meaning depend on their respective signs,
+     * allowing various types of rules to be constructed, as follows:
+     * <ul>
+     *   <li>If both dayOfWeekInMonth and dayOfWeek are positive, they specify the
+     *       day of week in the month (e.g., (2, WEDNESDAY) is the second Wednesday
+     *       of the month).</li>
+     *   <li>If dayOfWeek is positive and dayOfWeekInMonth is negative, they specify
+     *       the day of week in the month counting backward from the end of the month.
+     *       (e.g., (-1, MONDAY) is the last Monday in the month)</li>
+     *   <li>If dayOfWeek is zero and dayOfWeekInMonth is positive, dayOfWeekInMonth
+     *       specifies the day of the month, regardless of what day of the week it is.
+     *       (e.g., (10, 0) is the tenth day of the month)</li>
+     *   <li>If dayOfWeek is zero and dayOfWeekInMonth is negative, dayOfWeekInMonth
+     *       specifies the day of the month counting backward from the end of the
+     *       month, regardless of what day of the week it is (e.g., (-2, 0) is the
+     *       next-to-last day of the month).</li>
+     *   <li>If dayOfWeek is negative and dayOfWeekInMonth is positive, they specify the
+     *       first specified day of the week on or after the specfied day of the month.
+     *       (e.g., (15, -SUNDAY) is the first Sunday after the 15th of the month
+     *       [or the 15th itself if the 15th is a Sunday].)</li>
+     *   <li>If dayOfWeek and DayOfWeekInMonth are both negative, they specify the
+     *       last specified day of the week on or before the specified day of the month.
+     *       (e.g., (-20, -TUESDAY) is the last Tuesday before the 20th of the month
+     *       [or the 20th itself if the 20th is a Tuesday].)</li>
+     * </ul>
      * @param month the daylight savings starting month. Month is 0-based.
      * eg, 0 for January.
      * @param dayOfWeekInMonth the daylight savings starting
@@ -222,11 +346,8 @@ public:
      * @param mode whether the time is local wall time, local standard time,
      * or UTC time. Default is local wall time.
      * @param status An UErrorCode
-     * @stable
+     * @stable ICU 2.0
      */
-    void setStartRule(int32_t month, int32_t dayOfWeekInMonth, int32_t dayOfWeek,
-                      int32_t time, UErrorCode& status);
-
     void setStartRule(int32_t month, int32_t dayOfWeekInMonth, int32_t dayOfWeek,
                       int32_t time, TimeMode mode, UErrorCode& status);
 
@@ -238,17 +359,45 @@ public:
      * @param time          The time of that day (number of millis after midnight)
      *                      when DST takes effect in local wall time, which is
      *                      standard time in this case.
-     * @param mode whether the time is local wall time, local standard time,
-     * or UTC time. Default is local wall time.
      * @param status An UErrorCode
-     * @stable
+     * @stable ICU 2.0
      */
     void setStartRule(int32_t month, int32_t dayOfMonth, int32_t time,
                       UErrorCode& status);
-
+    /**
+     * Sets the DST start rule to a fixed date within a month.
+     *
+     * @param month         The month in which this rule occurs (0-based).
+     * @param dayOfMonth    The date in that month (1-based).
+     * @param time          The time of that day (number of millis after midnight)
+     *                      when DST takes effect in local wall time, which is
+     *                      standard time in this case.
+     * @param mode whether the time is local wall time, local standard time,
+     * or UTC time. Default is local wall time.
+     * @param status An UErrorCode
+     * @stable ICU 2.0
+     */
     void setStartRule(int32_t month, int32_t dayOfMonth, int32_t time,
                       TimeMode mode, UErrorCode& status);
 
+    /**
+     * Sets the DST start rule to a weekday before or after a give date within
+     * a month, e.g., the first Monday on or after the 8th.
+     *
+     * @param month         The month in which this rule occurs (0-based).
+     * @param dayOfMonth    A date within that month (1-based).
+     * @param dayOfWeek     The day of the week on which this rule occurs.
+     * @param time          The time of that day (number of millis after midnight)
+     *                      when DST takes effect in local wall time, which is
+     *                      standard time in this case.
+     * @param after         If true, this rule selects the first dayOfWeek on
+     *                      or after dayOfMonth.  If false, this rule selects
+     *                      the last dayOfWeek on or before dayOfMonth.
+     * @param status An UErrorCode
+     * @stable ICU 2.0
+     */
+    void setStartRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
+                      int32_t time, UBool after, UErrorCode& status);
     /**
      * Sets the DST start rule to a weekday before or after a give date within
      * a month, e.g., the first Monday on or after the 8th.
@@ -265,14 +414,36 @@ public:
      *                      or after dayOfMonth.  If false, this rule selects
      *                      the last dayOfWeek on or before dayOfMonth.
      * @param status An UErrorCode
-     * @stable
+     * @stable ICU 2.0
      */
-    void setStartRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
-                      int32_t time, UBool after, UErrorCode& status);
-
     void setStartRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
                       int32_t time, TimeMode mode, UBool after, UErrorCode& status);
 
+    /**
+     * Sets the daylight savings ending rule. For example, in the U.S., Daylight
+     * Savings Time ends at the last (-1) Sunday in October, at 2 AM in standard time.
+     * Therefore, you can set the end rule by calling:
+     * <pre>
+     * .   setEndRule(TimeFields.OCTOBER, -1, TimeFields.SUNDAY, 2*60*60*1000);
+     * </pre>
+     * Various other types of rules can be specified by manipulating the dayOfWeek
+     * and dayOfWeekInMonth parameters.  For complete details, see the documentation
+     * for setStartRule().
+     *
+     * @param month the daylight savings ending month. Month is 0-based.
+     * eg, 0 for January.
+     * @param dayOfWeekInMonth the daylight savings ending
+     * day-of-week-in-month. See setStartRule() for a complete explanation.
+     * @param dayOfWeek the daylight savings ending day-of-week. See setStartRule()
+     * for a complete explanation.
+     * @param time the daylight savings ending time. Please see the member
+     * description for an example.
+     * @param status An UErrorCode
+     * @stable ICU 2.0
+     */
+    void setEndRule(int32_t month, int32_t dayOfWeekInMonth, int32_t dayOfWeek,
+                    int32_t time, UErrorCode& status);
+
     /**
      * Sets the daylight savings ending rule. For example, in the U.S., Daylight
      * Savings Time ends at the last (-1) Sunday in October, at 2 AM in standard time.
@@ -295,14 +466,24 @@ public:
      * @param mode whether the time is local wall time, local standard time,
      * or UTC time. Default is local wall time.
      * @param status An UErrorCode
-     * @stable
+     * @stable ICU 2.0
      */
-    void setEndRule(int32_t month, int32_t dayOfWeekInMonth, int32_t dayOfWeek,
-                    int32_t time, UErrorCode& status);
-
     void setEndRule(int32_t month, int32_t dayOfWeekInMonth, int32_t dayOfWeek,
                     int32_t time, TimeMode mode, UErrorCode& status);
 
+    /**
+     * Sets the DST end rule to a fixed date within a month.
+     *
+     * @param month         The month in which this rule occurs (0-based).
+     * @param dayOfMonth    The date in that month (1-based).
+     * @param time          The time of that day (number of millis after midnight)
+     *                      when DST ends in local wall time, which is daylight
+     *                      time in this case.
+     * @param status An UErrorCode
+     * @stable ICU 2.0
+     */
+    void setEndRule(int32_t month, int32_t dayOfMonth, int32_t time, UErrorCode& status);
+
     /**
      * Sets the DST end rule to a fixed date within a month.
      *
@@ -314,13 +495,30 @@ public:
      * @param mode whether the time is local wall time, local standard time,
      * or UTC time. Default is local wall time.
      * @param status An UErrorCode
-     * @stable
+     * @stable ICU 2.0
      */
-    void setEndRule(int32_t month, int32_t dayOfMonth, int32_t time, UErrorCode& status);
-
     void setEndRule(int32_t month, int32_t dayOfMonth, int32_t time,
                     TimeMode mode, UErrorCode& status);
 
+    /**
+     * Sets the DST end rule to a weekday before or after a give date within
+     * a month, e.g., the first Monday on or after the 8th.
+     *
+     * @param month         The month in which this rule occurs (0-based).
+     * @param dayOfMonth    A date within that month (1-based).
+     * @param dayOfWeek     The day of the week on which this rule occurs.
+     * @param time          The time of that day (number of millis after midnight)
+     *                      when DST ends in local wall time, which is daylight
+     *                      time in this case.
+     * @param after         If true, this rule selects the first dayOfWeek on
+     *                      or after dayOfMonth.  If false, this rule selects
+     *                      the last dayOfWeek on or before dayOfMonth.
+     * @param status An UErrorCode
+     * @stable ICU 2.0
+     */
+    void setEndRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
+                    int32_t time, UBool after, UErrorCode& status);
+
     /**
      * Sets the DST end rule to a weekday before or after a give date within
      * a month, e.g., the first Monday on or after the 8th.
@@ -337,11 +535,8 @@ public:
      *                      or after dayOfMonth.  If false, this rule selects
      *                      the last dayOfWeek on or before dayOfMonth.
      * @param status An UErrorCode
-     * @stable
+     * @stable ICU 2.0
      */
-    void setEndRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
-                    int32_t time, UBool after, UErrorCode& status);
-
     void setEndRule(int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
                     int32_t time, TimeMode mode, UBool after, UErrorCode& status);
 
@@ -361,15 +556,13 @@ public:
      * @param day        The reference date's day-in-month (1-based)
      * @param dayOfWeek  The reference date's day-of-week (1-based; 1 is Sunday)
      * @param millis     The reference date's milliseconds in day, UTT (NOT local time).
+     * @param status     An UErrorCode to receive the status.
      * @return           The offset in milliseconds to add to GMT to get local time.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                               uint8_t dayOfWeek, int32_t millis, UErrorCode& status) const;
 
-    virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
-                              uint8_t dayOfWeek, int32_t millis) const;
-
     /**
      * Gets the time zone offset, for current date, modified in case of
      * daylight savings. This is the offset to add *to* UTC to get local time.
@@ -381,25 +574,48 @@ public:
      * @param dayOfWeek the day-of-week of the given date.
      * @param milliseconds the millis in day in <em>standard</em> local time.
      * @param monthLength the length of the given month in days.
-     * @param prevMonthLength length of the previous month in days.
+     * @param status     An UErrorCode to receive the status.
      * @return the offset to add *to* GMT to get local time.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                            uint8_t dayOfWeek, int32_t milliseconds,
                            int32_t monthLength, UErrorCode& status) const;
-
+    /**
+     * Gets the time zone offset, for current date, modified in case of
+     * daylight savings. This is the offset to add *to* UTC to get local time.
+     * @param era the era of the given date.
+     * @param year the year in the given date.
+     * @param month the month in the given date.
+     * Month is 0-based. e.g., 0 for January.
+     * @param day the day-in-month of the given date.
+     * @param dayOfWeek the day-of-week of the given date.
+     * @param milliseconds the millis in day in <em>standard</em> local time.
+     * @param monthLength the length of the given month in days.
+     * @param prevMonthLength length of the previous month in days.
+     * @param status     An UErrorCode to receive the status.
+     * @return the offset to add *to* GMT to get local time.
+     * @stable ICU 2.0
+     */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                               uint8_t dayOfWeek, int32_t milliseconds,
                               int32_t monthLength, int32_t prevMonthLength,
                               UErrorCode& status) const;
 
+    /**
+     * Redeclared TimeZone method.  This implementation simply calls
+     * the base class method, which otherwise would be hidden.
+     * @stable ICU 2.8
+     */
+    virtual void getOffset(UDate date, UBool local, int32_t& rawOffset,
+                           int32_t& dstOffset, UErrorCode& ec) const;
+
     /**
      * Returns the TimeZone's raw GMT offset (i.e., the number of milliseconds to add
      * to GMT to get local time, before taking daylight savings time into account).
      *
      * @return   The TimeZone's raw GMT offset.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t getRawOffset(void) const;
 
@@ -408,7 +624,7 @@ public:
      * to GMT to get local time, before taking daylight savings time into account).
      *
      * @param offsetMillis  The new raw GMT offset for this time zone.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void setRawOffset(int32_t offsetMillis);
 
@@ -417,30 +633,25 @@ public:
      * @param millisSavedDuringDST the number of milliseconds the time is
      * advanced with respect to standard time when the daylight savings rules
      * are in effect. A positive number, typically one hour (3600000).
-     * @stable
+     * @param status  An UErrorCode to receive the status.
+     * @stable ICU 2.0
      */
     void setDSTSavings(int32_t millisSavedDuringDST, UErrorCode& status);
 
-#ifdef ICU_SIMPLETIMEZONE_USE_DEPRECATES
-    /**
-     * @deprecated This will be removed after 2000-Dec-31. Use the other setDSTSavings().
-     */
-    void setDSTSavings(int32_t millisSavedDuringDST);
-#endif
-
     /**
      * Returns the amount of time in ms that the clock is advanced during DST.
      * @return the number of milliseconds the time is
      * advanced with respect to standard time when the daylight savings rules
      * are in effect. A positive number, typically one hour (3600000).
+     * @stable ICU 2.0
      */
-    int32_t getDSTSavings(void) const;
+    virtual int32_t getDSTSavings(void) const;
 
     /**
      * Queries if this TimeZone uses Daylight Savings Time.
      *
      * @return   True if this TimeZone uses Daylight Savings Time; false otherwise.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool useDaylightTime(void) const;
 
@@ -448,10 +659,15 @@ public:
      * Returns true if the given date is within the period when daylight savings time
      * is in effect; false otherwise.  If the TimeZone doesn't observe daylight savings
      * time, this functions always returns false.
+     * This method is wasteful since it creates a new GregorianCalendar and
+     * deletes it each time it is called. This is a deprecated method
+     * and provided only for Java compatibility.
+     *
      * @param date The date to test.
+     * @param status  An UErrorCode to receive the status.
      * @return true if the given date is in Daylight Savings Time;
      * false otherwise.
-     * @stable
+     * @deprecated ICU 2.4. Use Calendar::inDaylightTime() instead.
      */
     virtual UBool inDaylightTime(UDate date, UErrorCode& status) const;
 
@@ -459,7 +675,7 @@ public:
      * Return true if this zone has the same rules and offset as another zone.
      * @param other the TimeZone object to be compared with
      * @return true if the given zone has the same rules and offset as this one
-     * @stable
+     * @stable ICU 2.0
      */
     UBool hasSameRules(const TimeZone& other) const;
 
@@ -468,7 +684,7 @@ public:
      * the TimeZone object cloned.
      *
      * @return   A new copy of this TimeZone object.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual TimeZone* clone(void) const;
 
@@ -482,9 +698,9 @@ public:
      *
      * @return   The class ID for this object. All objects of a given class have the
      *           same class ID. Objects of other classes have different class IDs.
-     * @stable
+     * @stable ICU 2.0
      */
-    virtual UClassID getDynamicClassID(void) const { return (UClassID)&fgClassID; }
+    virtual UClassID getDynamicClassID(void) const;
 
     /**
      * Return the class ID for this class. This is useful only for comparing to a return
@@ -495,9 +711,9 @@ public:
      * .       Derived::getStaticClassID()) ...
      * </pre>
      * @return   The class ID for all objects of this class.
-     * @stable
+     * @stable ICU 2.0
      */
-    static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
+    static UClassID U_EXPORT2 getStaticClassID(void);
 
 private:
     /**
@@ -511,16 +727,26 @@ private:
         DOW_LE_DOM_MODE
     };
 
-    friend class TimeZone; // for access to these 2 constructors:
-
-    /**
-     * Construct from memory-mapped data.
-     */
-    SimpleTimeZone(const StandardZone& stdZone, const UnicodeString& id);
-    SimpleTimeZone(const DSTZone& dstZone, const UnicodeString& id);
+    SimpleTimeZone(); // default constructor not implemented
 
     /**
      * Internal construction method.
+     * @param rawOffsetGMT    The new SimpleTimeZone's raw GMT offset
+     * @param startMonth      the month DST starts
+     * @param startDay        the day DST starts
+     * @param startDayOfWeek  the DOW DST starts
+     * @param startTime       the time DST starts
+     * @param startTimeMode   Whether the start time is local wall time, local
+     *                        standard time, or UTC time. Default is local wall time.
+     * @param endMonth        the month DST ends
+     * @param endDay          the day DST ends
+     * @param endDayOfWeek    the DOW DST ends
+     * @param endTime         the time DST ends
+     * @param endTimeMode     Whether the end time is local wall time, local
+     *                        standard time, or UTC time. Default is local wall time.
+     * @param savingsDST      The number of milliseconds added to standard time
+     *                        to get DST time. Default is one hour.
+     * @param status          An UErrorCode to receive the status.
      */
     void construct(int32_t rawOffsetGMT,
                    int8_t startMonth, int8_t startDay, int8_t startDayOfWeek,
@@ -563,8 +789,6 @@ private:
     void decodeStartRule(UErrorCode& status);
     void decodeEndRule(UErrorCode& status);
 
-    static const char     fgClassID;
-
     int8_t startMonth, startDay, startDayOfWeek;   // the month, day, DOW, and time DST starts
     int32_t startTime;
     TimeMode startTimeMode, endTimeMode; // Mode for startTime, endTime; see TimeMode
@@ -573,7 +797,7 @@ private:
     int32_t startYear;  // the year these DST rules took effect
     int32_t rawOffset;  // the TimeZone's raw GMT offset
     UBool useDaylight; // flag indicating whether this TimeZone uses DST
-    static const int8_t staticMonthLength[12]; // lengths of the months
+    static const int8_t STATICMONTHLENGTH[12]; // lengths of the months
     EMode startMode, endMode;   // flags indicating what kind of rules the DST rules are
 
     /**
@@ -617,6 +841,14 @@ inline void SimpleTimeZone::setEndRule(int32_t month, int32_t dayOfMonth, int32_
     setEndRule(month, dayOfMonth, dayOfWeek, time, WALL_TIME, after, status);
 }
 
+inline void
+SimpleTimeZone::getOffset(UDate date, UBool local, int32_t& rawOffsetRef,
+                          int32_t& dstOffsetRef, UErrorCode& ec) const {
+    TimeZone::getOffset(date, local, rawOffsetRef, dstOffsetRef, ec);
+}
+
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif // _SIMPLETZ
