@@ -1,5 +1,5 @@
 /*
-* Copyright (C) {1997-2001}, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 1997-2006, International Business Machines Corporation and others. All Rights Reserved.
 *******************************************************************************
 *
 * File SMPDTFMT.H
@@ -24,6 +24,14 @@
 #define SMPDTFMT_H
 
 #include "unicode/utypes.h"
+
+/**
+ * \file 
+ * \brief C++ API: Format and parse dates in a language-independent manner.
+ */
+ 
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/datefmt.h"
 
 U_NAMESPACE_BEGIN
@@ -32,6 +40,7 @@ class DateFormatSymbols;
 class DateFormat;
 
 /**
+ *
  * SimpleDateFormat is a concrete class for formatting and parsing dates in a
  * language-independent manner. It allows for formatting (millis -> text),
  * parsing (text -> millis), and normalization. Formats/Parses a date or time,
@@ -53,30 +62,35 @@ class DateFormat;
  * as the following:
  * <pre>
  * \code
- *    Symbol   Meaning                 Presentation       Example
- *    ------   -------                 ------------       -------
- *    G        era designator          (Text)             AD
- *    y        year                    (Number)           1996
- *    Y        year/week of year       (Number)           1996
- *    M        month in year           (Text & Number)    July & 07
- *    d        day in month            (Number)           10
- *    h        hour in am/pm (1~12)    (Number)           12
- *    H        hour in day (0~23)      (Number)           0
- *    m        minute in hour          (Number)           30
- *    s        second in minute        (Number)           55
- *    S        millisecond             (Number)           978
- *    E        day of week             (Text)             Tuesday
- *    e        day of week/local (1~7) (Number)           2
- *    D        day of year             (Number)           189
- *    F        day of week in month    (Number)           2 (2nd Wed in July)
- *    w        week in year            (Number)           27
- *    W        week in month           (Number)           2
- *    a        am/pm marker            (Text)             PM
- *    k        hour in day (1~24)      (Number)           24
- *    K        hour in am/pm (0~11)    (Number)           0
- *    z        time zone               (Text)             Pacific Standard Time
- *    '        escape for text
- *    ''       single quote                               '
+ * Symbol   Meaning                 Presentation        Example
+ * ------   -------                 ------------        -------
+ * G        era designator          (Text)              AD
+ * y        year                    (Number)            1996
+ * Y        year (week of year)     (Number)            1997
+ * u        extended year           (Number)            4601
+ * M        month in year           (Text & Number)     July & 07
+ * d        day in month            (Number)            10
+ * h        hour in am/pm (1~12)    (Number)            12
+ * H        hour in day (0~23)      (Number)            0
+ * m        minute in hour          (Number)            30
+ * s        second in minute        (Number)            55
+ * S        fractional second       (Number)            978
+ * E        day of week             (Text)              Tuesday
+ * e        day of week (local 1~7) (Number)            2
+ * D        day in year             (Number)            189
+ * F        day of week in month    (Number)            2 (2nd Wed in July)
+ * w        week in year            (Number)            27
+ * W        week in month           (Number)            2
+ * a        am/pm marker            (Text)              PM
+ * k        hour in day (1~24)      (Number)            24
+ * K        hour in am/pm (0~11)    (Number)            0
+ * z        time zone               (Time)              Pacific Standard Time
+ * Z        time zone (RFC 822)     (Number)            -0800
+ * v        time zone (generic)     (Text)              Pacific Time
+ * g        Julian day              (Number)            2451334
+ * A        milliseconds in day     (Number)            69540000
+ * '        escape for text         (Delimiter)         'Date='
+ * ''       single quote            (Literal)           'o''clock'
  * \endcode
  * </pre>
  * The count of pattern letters determine the format.
@@ -88,6 +102,7 @@ class DateFormat;
  * this amount (e.g. if "m" produces "6", "mm" produces "06"). Year is handled
  * specially; that is, if the count of 'y' is 2, the Year will be truncated to 2 digits.
  * (e.g., if "yyyy" produces "1997", "yy" produces "97".)
+ * Unlike other fields, fractional seconds are padded on the right with zero.
  * <P>
  * (Text & Number): 3 or over, use text, otherwise use number.  (e.g., "M" produces "1",
  * "MM" produces "01", "MMM" produces "Jan", and "MMMM" produces "January".)
@@ -105,11 +120,11 @@ class DateFormat;
  * \code
  *    Format Pattern                         Result
  *    --------------                         -------
- *    "yyyy.MM.dd G 'at' HH:mm:ss z"    ->>  1996.07.10 AD at 15:08:56 PDT
+ *    "yyyy.MM.dd G 'at' HH:mm:ss vvvv" ->>  1996.07.10 AD at 15:08:56 Pacific Time
  *    "EEE, MMM d, ''yy"                ->>  Wed, July 10, '96
  *    "h:mm a"                          ->>  12:08 PM
  *    "hh 'o''clock' a, zzzz"           ->>  12 o'clock PM, Pacific Daylight Time
- *    "K:mm a, z"                       ->>  0:00 PM, PST
+ *    "K:mm a, vvv"                     ->>  0:00 PM, PT
  *    "yyyyy.MMMMM.dd GGG hh:mm aaa"    ->>  1996.July.10 AD 12:08 PM
  * \endcode
  * </pre>
@@ -163,6 +178,18 @@ class DateFormat;
  * interpreted literally, regardless of the number of digits.  So using the
  * pattern "MM/dd/yyyy", "01/11/12" parses to Jan 11, 12 A.D.
  *
+ * <p>
+ * When numeric fields abut one another directly, with no intervening delimiter
+ * characters, they constitute a run of abutting numeric fields.  Such runs are
+ * parsed specially.  For example, the format "HHmmss" parses the input text
+ * "123456" to 12:34:56, parses the input text "12345" to 1:23:45, and fails to
+ * parse "1234".  In other words, the leftmost field of the run is flexible,
+ * while the others keep a fixed width.  If the parse fails anywhere in the run,
+ * then the leftmost field is shortened by one character, and the entire run is
+ * parsed again. This is repeated until either the parse succeeds or the
+ * leftmost field is one character in length.  If the parse still fails at that
+ * point, the parse of the run fails.
+ *
  * <P>
  * For time zones that have no names, SimpleDateFormat uses strings GMT+hours:minutes or
  * GMT-hours:minutes.
@@ -171,6 +198,10 @@ class DateFormat;
  * year, whether hours are zero based or not (0 vs 12 or 24), and the timezone.
  * There is one common number format to handle all the numbers; the digit count
  * is handled programmatically according to the pattern.
+ *
+ * <p><em>User subclasses are not supported.</em> While clients may write
+ * subclasses, such code will not necessarily work and will not be
+ * guaranteed to work stably from release to release.
  */
 class U_I18N_API SimpleDateFormat: public DateFormat {
 public:
@@ -180,7 +211,8 @@ public:
      * <P>
      * [Note:] Not all locales support SimpleDateFormat; for full generality,
      * use the factory methods in the DateFormat class.
-     * @stable
+     * @param status    Output param set to success/failure code.
+     * @stable ICU 2.0
      */
     SimpleDateFormat(UErrorCode& status);
 
@@ -191,7 +223,9 @@ public:
      * <P>
      * [Note:] Not all locales support SimpleDateFormat; for full generality,
      * use the factory methods in the DateFormat class.
-     * @stable
+     * @param pattern    the pattern for the format.
+     * @param status     Output param set to success/failure code.
+     * @stable ICU 2.0
      */
     SimpleDateFormat(const UnicodeString& pattern,
                      UErrorCode& status);
@@ -203,7 +237,10 @@ public:
      * <P>
      * [Note:] Not all locales support SimpleDateFormat; for full generality,
      * use the factory methods in the DateFormat class.
-     * @stable
+     * @param pattern    the pattern for the format.
+     * @param locale     the given locale.
+     * @param status     Output param set to success/failure code.
+     * @stable ICU 2.0
      */
     SimpleDateFormat(const UnicodeString& pattern,
                      const Locale& locale,
@@ -213,7 +250,10 @@ public:
      * Construct a SimpleDateFormat using the given pattern and locale-specific
      * symbol data.  The formatter takes ownership of the DateFormatSymbols object;
      * the caller is no longer responsible for deleting it.
-     * @stable
+     * @param pattern           the given pattern for the format.
+     * @param formatDataToAdopt the symbols to be adopted.
+     * @param status            Output param set to success/faulure code.
+     * @stable ICU 2.0
      */
     SimpleDateFormat(const UnicodeString& pattern,
                      DateFormatSymbols* formatDataToAdopt,
@@ -223,7 +263,10 @@ public:
      * Construct a SimpleDateFormat using the given pattern and locale-specific
      * symbol data.  The DateFormatSymbols object is NOT adopted; the caller
      * remains responsible for deleting it.
-     * @stable
+     * @param pattern           the given pattern for the format.
+     * @param formatData        the formatting symbols to be use.
+     * @param status            Output param set to success/faulure code.
+     * @stable ICU 2.0
      */
     SimpleDateFormat(const UnicodeString& pattern,
                      const DateFormatSymbols& formatData,
@@ -231,33 +274,36 @@ public:
 
     /**
      * Copy constructor.
-     * @stable
+     * @stable ICU 2.0
      */
     SimpleDateFormat(const SimpleDateFormat&);
 
     /**
      * Assignment operator.
-     * @stable
+     * @stable ICU 2.0
      */
     SimpleDateFormat& operator=(const SimpleDateFormat&);
 
     /**
      * Destructor.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual ~SimpleDateFormat();
 
     /**
      * Clone this Format object polymorphically. The caller owns the result and
      * should delete it when done.
-     * @stable
+     * @return    A copy of the object.
+     * @stable ICU 2.0
      */
     virtual Format* clone(void) const;
 
     /**
      * Return true if the given Format objects are semantically equal. Objects
      * of different subclasses are considered unequal.
-     * @stable
+     * @param other    the object to be compared with.
+     * @return         true if the given Format objects are semantically equal.
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const Format& other) const;
 
@@ -268,17 +314,17 @@ public:
      * Example: using the US locale: "yyyy.MM.dd e 'at' HH:mm:ss zzz" ->>
      * 1996.07.10 AD at 15:08:56 PDT
      *
-     * @param cal           a Calendar set to the date and time to be formatted
-     *                      into a date/time string.
-     * @param toAppendTo    The result of the formatting operation is appended to this
-     *                      string.
-     * @param pos           The formatting position. On input: an alignment field,
-     *                      if desired. On output: the offsets of the alignment field.
-     * @return              A reference to 'toAppendTo'.
-     * @draft ICU 2.1
+     * @param cal       Calendar set to the date and time to be formatted
+     *                  into a date/time string.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       The formatting position. On input: an alignment field,
+     *                  if desired. On output: the offsets of the alignment field.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.1
      */
     virtual UnicodeString& format(  Calendar& cal,
-                                    UnicodeString& toAppendTo,
+                                    UnicodeString& appendTo,
                                     FieldPosition& pos) const;
 
     /**
@@ -288,43 +334,59 @@ public:
      * Example: using the US locale: "yyyy.MM.dd e 'at' HH:mm:ss zzz" ->>
      * 1996.07.10 AD at 15:08:56 PDT
      *
-     * @param obj           A Formattable containing the date-time value to be formatted
-     *                      into a date-time string.  If the type of the Formattable
-     *                      is a numeric type, it is treated as if it were an
-     *                      instance of Date.
-     * @param toAppendTo    The result of the formatting operation is appended to this
-     *                      string.
-     * @param pos           The formatting position. On input: an alignment field,
-     *                      if desired. On output: the offsets of the alignment field.
-     * @return              A reference to 'toAppendTo'.
-     * @stable
+     * @param obj       A Formattable containing the date-time value to be formatted
+     *                  into a date-time string.  If the type of the Formattable
+     *                  is a numeric type, it is treated as if it were an
+     *                  instance of Date.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       The formatting position. On input: an alignment field,
+     *                  if desired. On output: the offsets of the alignment field.
+     * @param status    Output param set to success/faulure code.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     virtual UnicodeString& format(  const Formattable& obj,
-                                    UnicodeString& toAppendTo,
+                                    UnicodeString& appendTo,
                                     FieldPosition& pos,
                                     UErrorCode& status) const;
 
     /**
      * Redeclared DateFormat method.
-     * @draft ICU 2.1
+     * @param date          the Date value to be formatted.
+     * @param appendTo      Output parameter to receive result.
+     *                      Result is appended to existing contents.
+     * @param fieldPosition The formatting position. On input: an alignment field,
+     *                      if desired. On output: the offsets of the alignment field.
+     * @return              Reference to 'appendTo' parameter.
+     * @stable ICU 2.1
      */
     UnicodeString& format(UDate date,
-                          UnicodeString& result,
+                          UnicodeString& appendTo,
                           FieldPosition& fieldPosition) const;
 
     /**
      * Redeclared DateFormat method.
-     * @stable
+     * @param obj       Object to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param status    Input/output success/failure code.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     UnicodeString& format(const Formattable& obj,
-                          UnicodeString& result,
+                          UnicodeString& appendTo,
                           UErrorCode& status) const;
 
     /**
      * Redeclared DateFormat method.
-     * @stable
+     * @param date      Date value to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
-    UnicodeString& format(UDate date, UnicodeString& result) const;
+    UnicodeString& format(UDate date, UnicodeString& appendTo) const;
 
     /**
      * Parse a date/time string beginning at the given parse position. For
@@ -337,11 +399,13 @@ public:
      * calling setLenient(false).
      *
      * @param text  The date/time string to be parsed
+     * @param cal   a Calendar set to the date and time to be formatted
+     *              into a date/time string.
      * @param pos   On input, the position at which to start parsing; on
      *              output, the position at which parsing terminated, or the
      *              start position if the parse failed.
      * @return      A valid UDate if the input could be parsed.
-     * @draft ICU 2.1
+     * @stable ICU 2.1
      */
     virtual void parse( const UnicodeString& text,
                         Calendar& cal,
@@ -364,7 +428,7 @@ public:
      *              output, the position at which parsing terminated, or the
      *              start position if the parse failed.
      * @return      A valid UDate if the input could be parsed.
-     * @stable
+     * @stable ICU 2.0
      */
     UDate parse( const UnicodeString& text,
                  ParsePosition& pos) const;
@@ -383,7 +447,7 @@ public:
      * @param status Filled in with U_ZERO_ERROR if the parse was successful, and with
      *              an error value if there was a parse error.
      * @return      A valid UDate if the input could be parsed.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UDate parse( const UnicodeString& text,
                         UErrorCode& status) const;
@@ -399,7 +463,10 @@ public:
      * <P>
      * By default, the two digit start date is set to 80 years before the current
      * time at which a SimpleDateFormat object is created.
-     * @stable
+     * @param d      start UDate used to interpret two-digit year strings.
+     * @param status Filled in with U_ZERO_ERROR if the parse was successful, and with
+     *               an error value if there was a parse error.
+     * @stable ICU 2.0
      */
     virtual void set2DigitYearStart(UDate d, UErrorCode& status);
 
@@ -414,13 +481,17 @@ public:
      * <P>
      * By default, the two digit start date is set to 80 years before the current
      * time at which a SimpleDateFormat object is created.
-     * @stable
+     * @param status Filled in with U_ZERO_ERROR if the parse was successful, and with
+     *               an error value if there was a parse error.
+     * @stable ICU 2.0
      */
     UDate get2DigitYearStart(UErrorCode& status) const;
 
     /**
      * Return a pattern string describing this date format.
-     * @stable
+     * @param result Output param to receive the pattern.
+     * @return       A reference to 'result'.
+     * @stable ICU 2.0
      */
     virtual UnicodeString& toPattern(UnicodeString& result) const;
 
@@ -437,7 +508,8 @@ public:
      * @param status    Output param set to success/failure code on
      *                  exit. If the pattern is invalid, this will be
      *                  set to a failure result.
-     * @stable
+     * @return          A reference to 'result'.
+     * @stable ICU 2.0
      */
     virtual UnicodeString& toLocalizedPattern(UnicodeString& result,
                                               UErrorCode& status) const;
@@ -448,7 +520,7 @@ public:
      * the new pattern)
      *
      * @param pattern   The pattern to be applied.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void applyPattern(const UnicodeString& pattern);
 
@@ -460,7 +532,7 @@ public:
      * @param status    Output param set to success/failure code on
      *                  exit. If the pattern is invalid, this will be
      *                  set to a failure result.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void applyLocalizedPattern(const UnicodeString& pattern,
                                        UErrorCode& status);
@@ -471,22 +543,22 @@ public:
      * names and abbreviations, time zone names, AM/PM strings, etc.)
      * @return a copy of the date-time formatting data associated
      * with this date-time formatter.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual const DateFormatSymbols* getDateFormatSymbols(void) const;
 
     /**
      * Set the date/time formatting symbols.  The caller no longer owns the
      * DateFormatSymbols object and should not delete it after making this call.
-     * @param newFormatData the given date-time formatting data.
-     * @stable
+     * @param newFormatSymbols the given date-time formatting symbols to copy.
+     * @stable ICU 2.0
      */
     virtual void adoptDateFormatSymbols(DateFormatSymbols* newFormatSymbols);
 
     /**
      * Set the date/time formatting data.
-     * @param newFormatData the given date-time formatting data.
-     * @stable
+     * @param newFormatSymbols the given date-time formatting symbols to copy.
+     * @stable ICU 2.0
      */
     virtual void setDateFormatSymbols(const DateFormatSymbols& newFormatSymbols);
 
@@ -499,9 +571,9 @@ public:
      * .       erived::getStaticClassID()) ...
      * </pre>
      * @return          The class ID for all objects of this class.
-     * @stable
+     * @stable ICU 2.0
      */
-    static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
+    static UClassID U_EXPORT2 getStaticClassID(void);
 
     /**
      * Returns a unique class ID POLYMORPHICALLY. Pure virtual override. This
@@ -512,37 +584,35 @@ public:
      * @return          The class ID for this object. All objects of a
      *                  given class have the same class ID.  Objects of
      *                  other classes have different class IDs.
-     * @stable
+     * @stable ICU 2.0
      */
-    virtual UClassID getDynamicClassID(void) const { return getStaticClassID(); }
-
-private:
-    static const char fgClassID;
-
-    static const char fgDateTimePatternsTag[];   // resource bundle tag for default date and time patterns
-
-    static const UChar fgDefaultPattern[];    // date/time pattern of last resort
-
-    friend class DateFormat;
+    virtual UClassID getDynamicClassID(void) const;
 
     /**
-     * Gets the index for the given time zone ID to obtain the timezone strings
-     * for formatting. The time zone ID is just for programmatic lookup. NOT
-     * LOCALIZED!!!
+     * Set the calendar to be used by this date format. Initially, the default
+     * calendar for the specified or default locale is used.  The caller should
+     * not delete the Calendar object after it is adopted by this call.
+     * Adopting a new calendar will change to the default symbols.
      *
-     * @param DateFormatSymbols     a DateFormatSymbols object contianing the time zone names
-     * @param ID        the given time zone ID.
-     * @return          the index of the given time zone ID.  Returns -1 if
-     *                  the given time zone ID can't be located in the
-     *                  DateFormatSymbols object.
-     * @see SimpleTimeZone
+     * @param calendarToAdopt    Calendar object to be adopted.
+     * @stable ICU 2.0
      */
-    //int32_t getZoneIndex(const DateFormatSymbols&, const UnicodeString& ID) const;
+    virtual void adoptCalendar(Calendar* calendarToAdopt);
+
+private:
+    friend class DateFormat;
 
     void initializeDefaultCentury(void);
 
+    SimpleDateFormat(); // default constructor not implemented
+
     /**
      * Used by the DateFormat factory methods to construct a SimpleDateFormat.
+     * @param timeStyle the time style.
+     * @param dateStyle the date style.
+     * @param locale    the given locale.
+     * @param status    Output param set to success/failure code on
+     *                  exit.
      */
     SimpleDateFormat(EStyle timeStyle, EStyle dateStyle, const Locale& locale, UErrorCode& status);
 
@@ -550,51 +620,75 @@ private:
      * Construct a SimpleDateFormat for the given locale.  If no resource data
      * is available, create an object of last resort, using hard-coded strings.
      * This is an internal method, called by DateFormat.  It should never fail.
+     * @param locale    the given locale.
+     * @param status    Output param set to success/failure code on
+     *                  exit.
      */
     SimpleDateFormat(const Locale& locale, UErrorCode& status); // Use default pattern
 
     /**
      * Called by format() to format a single field.
      *
-     * @param result    Filled in with the result.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
      * @param ch        The format character we encountered in the pattern.
      * @param count     Number of characters in the current pattern symbol (e.g.,
      *                  "yyyy" in the pattern would result in a call to this function
      *                  with ch equal to 'y' and count equal to 4)
-     * @param beginOffset   Tells where the text returned by this function will go in
-     *                  the finished string.  Used when this function needs to fill
-     *                  in a FieldPosition
      * @param pos       The FieldPosition being filled in by the format() call.  If
      *                  this function is formatting the field specfied by pos, it
-     *                  will fill in pos will the beginning and ending offsets of the
+     *                  will fill in pos with the beginning and ending offsets of the
      *                  field.
      * @param status    Receives a status code, which will be U_ZERO_ERROR if the operation
      *                  succeeds.
-     * @return A reference to "result".
      */
-    UnicodeString& subFormat(   UnicodeString& result,
+    void subFormat(             UnicodeString &appendTo,
                                 UChar ch,
                                 int32_t count,
-                                int32_t beginOffset,
                                 FieldPosition& pos,
                                 Calendar& cal,
                                 UErrorCode& status) const; // in case of illegal argument
 
     /**
-     * Used by subFormat() to format a numeric value.  Fills in "result" with a string
-     * representation of "value" having a number of digits between "minDigits" and
+     * Used by subFormat() to format a numeric value.
+     * Appends to toAppendTo a string representation of "value"
+     * having a number of digits between "minDigits" and
      * "maxDigits".  Uses the DateFormat's NumberFormat.
-     * @param result    Filled in with the formatted number.
+     *
+     * @param appendTo  Output parameter to receive result.
+     *                  Formatted number is appended to existing contents.
      * @param value     Value to format.
      * @param minDigits Minimum number of digits the result should have
      * @param maxDigits Maximum number of digits the result should have
-     * @return A reference to "result".
      */
-    UnicodeString& zeroPaddingNumber(UnicodeString& result,
+    void zeroPaddingNumber(          UnicodeString &appendTo,
                                      int32_t value,
                                      int32_t minDigits,
                                      int32_t maxDigits) const;
 
+    /**
+     * Return true if the given format character, occuring count
+     * times, represents a numeric field.
+     */
+    static UBool isNumeric(UChar formatChar, int32_t count);
+
+    /**
+     * initializes fCalendar from parameters.  Returns fCalendar as a convenience.
+     * @param adoptZone  Zone to be adopted, or NULL for TimeZone::createDefault().
+     * @param locale Locale of the calendar
+     * @param status Error code
+     * @return the newly constructed fCalendar
+     */
+    Calendar *initializeCalendar(TimeZone* adoptZone, const Locale& locale, UErrorCode& status);
+
+    /**
+     * initializes fSymbols from parameters.
+     * @param locale Locale of the symbols
+     * @param calendar Alias to Calendar that will be used.
+     * @param status Error code
+     */
+    void initializeSymbols(const Locale& locale, Calendar* calendar, UErrorCode& status);
+
     /**
      * Called by several of the constructors to load pattern data and formatting symbols
      * out of a resource bundle and initialize the locale based on it.
@@ -619,13 +713,31 @@ private:
      * @param text the time text being parsed.
      * @param start where to start parsing.
      * @param field the date field being parsed.
-     * @param data the string array to parsed.
+     * @param stringArray the string array to parsed.
+     * @param stringArrayCount the size of the array.
+     * @param cal a Calendar set to the date and time to be formatted
+     *            into a date/time string.
      * @return the new start position if matching succeeded; a negative number
      * indicating matching failure, otherwise.
      */
-    int32_t matchString(const UnicodeString& text, int32_t start, Calendar::EDateFields field,
+    int32_t matchString(const UnicodeString& text, int32_t start, UCalendarDateFields field,
                         const UnicodeString* stringArray, int32_t stringArrayCount, Calendar& cal) const;
 
+    /**
+     * Private code-size reduction function used by subParse.
+     * @param text the time text being parsed.
+     * @param start where to start parsing.
+     * @param field the date field being parsed.
+     * @param stringArray the string array to parsed.
+     * @param stringArrayCount the size of the array.
+     * @param cal a Calendar set to the date and time to be formatted
+     *            into a date/time string.
+     * @return the new start position if matching succeeded; a negative number
+     * indicating matching failure, otherwise.
+     */
+    int32_t matchQuarterString(const UnicodeString& text, int32_t start, UCalendarDateFields field,
+                               const UnicodeString* stringArray, int32_t stringArrayCount, Calendar& cal) const;
+
     /**
      * Private member function that converts the parsed date strings into
      * timeFields. Returns -start (for ParsePosition) if failed.
@@ -634,86 +746,86 @@ private:
      * @param ch the pattern character for the date field text to be parsed.
      * @param count the count of a pattern character.
      * @param obeyCount if true then the count is strictly obeyed.
+     * @param ambiguousYear If true then the two-digit year == the default start year.
+     * @param cal a Calendar set to the date and time to be formatted
+     *            into a date/time string.
      * @return the new start position if matching succeeded; a negative number
      * indicating matching failure, otherwise.
      */
     int32_t subParse(const UnicodeString& text, int32_t& start, UChar ch, int32_t count,
-                     UBool obeyCount, UBool ambiguousYear[], Calendar& cal) const;
+                     UBool obeyCount, UBool allowNegative, UBool ambiguousYear[], Calendar& cal) const;
 
-    /**
-     * Parse the given text, at the given position, as a numeric value, using
-     * this object's NumberFormat. Return the corresponding long value in the
-     * fill-in parameter 'value'. If the parse fails, this method leaves pos
-     * unchanged and returns FALSE; otherwise it advances pos and
-     * returns TRUE.
-     */
-    //UBool subParseLong(const UnicodeString& text, ParsePosition& pos, int32_t& value) const;
+    void parseInt(const UnicodeString& text,
+                  Formattable& number,
+                  ParsePosition& pos,
+                  UBool allowNegative) const;
 
     /**
      * Translate a pattern, mapping each character in the from string to the
      * corresponding character in the to string. Return an error if the original
      * pattern contains an unmapped character, or if a quote is unmatched.
      * Quoted (single quotes only) material is not translated.
+     * @param originalPattern   the original pattern.
+     * @param translatedPattern Output param to receive the translited pattern.
+     * @param from              the characters to be translited from.
+     * @param to                the characters to be translited to.
+     * @param status            Receives a status code, which will be U_ZERO_ERROR
+     *                          if the operation succeeds.
      */
     static void translatePattern(const UnicodeString& originalPattern,
                                 UnicodeString& translatedPattern,
                                 const UnicodeString& from,
                                 const UnicodeString& to,
                                 UErrorCode& status);
-    /**
-     * Given a zone ID, try to locate it in our time zone array. Return the
-     * index (row index) of the found time zone, or -1 if we can't find it.
-     */
-    //int32_t getZoneIndex(const UnicodeString& ID) const;
 
     /**
      * Sets the starting date of the 100-year window that dates with 2-digit years
      * are considered to fall within.
+     * @param startDate the start date
+     * @param status    Receives a status code, which will be U_ZERO_ERROR
+     *                  if the operation succeeds.
      */
     void         parseAmbiguousDatesAsAfter(UDate startDate, UErrorCode& status);
 
     /**
-     * Returns the beginning date of the 100-year window that dates with 2-digit years
-     * are considered to fall within.
-     */
-    UDate         internalGetDefaultCenturyStart(void) const;
+     * Given text, a start in the text, and a row index, return the column index that
+     * of the zone name that matches (case insensitive) at start, or 0 if none matches.
+     *
+    int32_t      matchZoneString(const UnicodeString& text, int32_t start, int32_t zi) const;
+    */
 
     /**
-     * Returns the first year of the 100-year window that dates with 2-digit years
-     * are considered to fall within.
+     * Given text, a start in the text, and a calendar, return the next offset in the text
+     * after matching the zone string.  If we fail to match, return 0.  Update the calendar
+     * as appropriate.
      */
-    int32_t          internalGetDefaultCenturyStartYear(void) const;
-
+    int32_t      subParseZoneString(const UnicodeString& text, int32_t start, Calendar& cal, UErrorCode& status) const;
+    
     /**
-     * Initializes the 100-year window that dates with 2-digit years are considered
-     * to fall within so that its start date is 80 years before the current time.
+     * append the gmt string
      */
-    static void  initializeSystemDefaultCentury(void);
-
-    /**
-     * Last-resort string to use for "GMT" when constructing time zone strings.
-     */
-    // For time zones that have no names, use strings GMT+minutes and
-    // GMT-minutes. For instance, in France the time zone is GMT+60.
-    static const UChar fgGmtPlus[];
-    static const UChar fgGmtMinus[];
-    static const UChar fgGmt[];
+    inline void appendGMT(UnicodeString &appendTo, Calendar& cal, UErrorCode& status) const;
 
     /**
      * Used to map pattern characters to Calendar field identifiers.
      */
-    static const Calendar::EDateFields fgPatternIndexToCalendarField[];
+    static const UCalendarDateFields fgPatternIndexToCalendarField[];
 
     /**
      * Map index into pattern character string to DateFormat field number
      */
-    static const DateFormat::EField fgPatternIndexToDateFormatField[];
+    static const UDateFormatField fgPatternIndexToDateFormatField[];
 
     /**
      * The formatting pattern for this formatter.
      */
     UnicodeString       fPattern;
 
+    /**
+     * The original locale used (for reloading symbols)
+     */
+    Locale              fLocale;
+
     /**
      * A pointer to an object containing the strings to use in formatting (e.g.,
      * month and day names, AM and PM strings, time zone names, etc.)
@@ -735,30 +847,9 @@ private:
      */
     /*transient*/ int32_t   fDefaultCenturyStartYear;
 
-    /**
-     * The system maintains a static default century start date.  This is initialized
-     * the first time it is used.  Before then, it is set to SYSTEM_DEFAULT_CENTURY to
-     * indicate an uninitialized state.  Once the system default century date and year
-     * are set, they do not change.
-     */
-    static UDate         fgSystemDefaultCenturyStart;
+    /*transient*/ TimeZone* parsedTimeZone; // here to avoid api change
 
-    /**
-     * See documentation for systemDefaultCenturyStart.
-     */
-    static int32_t          fgSystemDefaultCenturyStartYear;
-
-    /**
-     * Default value that indicates the defaultCenturyStartYear is unitialized
-     */
-    static const int32_t    fgSystemDefaultCenturyYear;
-
-public:
-    /**
-     * If a start date is set to this value, that indicates that the system default
-     * start is in effect for this instance.
-     */
-    static const UDate        fgSystemDefaultCentury;
+    UBool fHaveDefaultCentury;
 };
 
 inline UDate
@@ -769,28 +860,30 @@ SimpleDateFormat::get2DigitYearStart(UErrorCode& /*status*/) const
 
 inline UnicodeString&
 SimpleDateFormat::format(const Formattable& obj,
-                         UnicodeString& result,
+                         UnicodeString& appendTo,
                          UErrorCode& status) const {
     // Don't use Format:: - use immediate base class only,
     // in case immediate base modifies behavior later.
-    return DateFormat::format(obj, result, status);
+    return DateFormat::format(obj, appendTo, status);
 }
 
 inline UnicodeString&
 SimpleDateFormat::format(UDate date,
-                         UnicodeString& result,
+                         UnicodeString& appendTo,
                          FieldPosition& fieldPosition) const {
     // Don't use Format:: - use immediate base class only,
     // in case immediate base modifies behavior later.
-    return DateFormat::format(date, result, fieldPosition);
+    return DateFormat::format(date, appendTo, fieldPosition);
 }
 
 inline UnicodeString&
-SimpleDateFormat::format(UDate date, UnicodeString& result) const {
-    return DateFormat::format(date, result);
+SimpleDateFormat::format(UDate date, UnicodeString& appendTo) const {
+    return DateFormat::format(date, appendTo);
 }
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif // _SMPDTFMT
 //eof
