@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001 IBM and others. All rights reserved.
+*   Copyright (C) 2001-2006 IBM and others. All rights reserved.
 **********************************************************************
 *   Date        Name        Description
 *  03/22/2000   helena      Creation.
@@ -10,25 +10,35 @@
 #ifndef STSEARCH_H
 #define STSEARCH_H
 
+#include "unicode/utypes.h"
+
+/**
+ * \file 
+ * \brief C++ API: Service for searching text based on RuleBasedCollator.
+ */
+ 
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/tblcoll.h"
 #include "unicode/coleitr.h"
 #include "unicode/search.h"
 
 U_NAMESPACE_BEGIN
 
-/**
+/** 
+ *
  * <tt>StringSearch</tt> is a <tt>SearchIterator</tt> that provides
  * language-sensitive text searching based on the comparison rules defined
  * in a {@link RuleBasedCollator} object.
  * StringSearch ensures that language eccentricity can be 
- * handled, e.g. for the German collator, characters  and SS will be matched 
- * if case is chosen to be ignored. 
- * See the <a href=http://oss.software.ibm.com/icu/develop/collation/ICU_collation_design.htm>
+ * handled, e.g. for the German collator, characters &szlig; and SS will be matched 
+ * if case is chosen to be ignored.
+ * See the <a href="http://dev.icu-project.org/cgi-bin/viewcvs.cgi/~checkout~/icuhtml/design/collation/ICU_collation_design.htm">
  * "ICU Collation Design Document"</a> for more information.
  * <p> 
  * The algorithm implemented is a modified form of the Boyer Moore's search.
  * For more information  see 
- * <a href=http://oss.software.ibm.com/icu/docs/papers/text-search.html>
+ * <a href="http://icu.sourceforge.net/docs/papers/efficient_text_searching_in_java.html">
  * "Efficient Text Searching in Java"</a>, published in <i>Java Report</i> 
  * in February, 1999, for further information on the algorithm.
  * <p>
@@ -45,7 +55,7 @@ U_NAMESPACE_BEGIN
  *           there exists no non-ignorable combining mark before or after S? 
  *           in S respectively. 
  * </pre>
- * Option 2. will be the default
+ * Option 2. will be the default.
  * <p>
  * This search has APIs similar to that of other text iteration mechanisms 
  * such as the break iterators in <tt>BreakIterator</tt>. Using these 
@@ -75,7 +85,7 @@ U_NAMESPACE_BEGIN
  *      contracting sequence except the first will fail. Vice versa if a 
  *      preceding pattern match is to be found, a invalid starting point 
  *      would be any character within a contracting sequence except the last.
- * <\ul>
+ * </ul>
  * <p>
  * A breakiterator can be used if only matches at logical breaks are desired.
  * Using a breakiterator will only give you results that exactly matches the
@@ -107,14 +117,17 @@ U_NAMESPACE_BEGIN
  * UnicodeString target("The quick brown fox jumped over the lazy fox");
  * UnicodeString pattern("fox");
  *
- * SearchIterator *iter = new StringSearch(pattern, target);
- *
- * for (int pos = iter->first(); pos != USEARCH_DONE; 
- *                               pos = iter->next()) {
+ * SearchIterator *iter  = new StringSearch(pattern, target);
+ * UErrorCode      error = U_ZERO_ERROR;
+ * for (int pos = iter->first(error); pos != USEARCH_DONE; 
+ *                               pos = iter->next(error)) {
  *     printf("Found match at %d pos, length is %d\n", pos, 
  *                                             iter.getMatchLength());
  * }
  * </code></pre>
+ * <p>
+ * Note, StringSearch is not to be subclassed.
+ * </p>
  * @see SearchIterator
  * @see RuleBasedCollator
  * @since ICU 2.0
@@ -129,7 +142,7 @@ public:
     /**
      * Creating a <tt>StringSearch</tt> instance using the argument locale 
      * language rule set. A collator will be created in the process, which 
-     * will be owned by this instance and will be deleted in during 
+     * will be owned by this instance and will be deleted during 
      * destruction
      * @param pattern The text for which this object will search.
      * @param text    The text in which to search for the pattern.
@@ -145,7 +158,7 @@ public:
      * @param status  for errors if any. If pattern or text is NULL, or if
      *               either the length of pattern or text is 0 then an 
      *               U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     StringSearch(const UnicodeString &pattern, const UnicodeString &text,
                  const Locale        &locale,       
@@ -171,7 +184,7 @@ public:
      *                <tt>NULL</tt> can be passed in instead.
      * @param status for errors if any. If either the length of pattern or 
      *               text is 0 then an U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     StringSearch(const UnicodeString     &pattern, 
                  const UnicodeString     &text,
@@ -182,7 +195,7 @@ public:
     /**
      * Creating a <tt>StringSearch</tt> instance using the argument locale 
      * language rule set. A collator will be created in the process, which 
-     * will be owned by this instance and will be deleted in during 
+     * will be owned by this instance and will be deleted during 
      * destruction
      * <p>
      * Note: No parsing of the text within the <tt>CharacterIterator</tt> 
@@ -202,7 +215,7 @@ public:
      *                <tt>NULL</tt> can be passed in instead.
      * @param status for errors if any. If either the length of pattern or 
      *               text is 0 then an U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     StringSearch(const UnicodeString &pattern, CharacterIterator &text,
                  const Locale        &locale, 
@@ -232,7 +245,7 @@ public:
      *                <tt>NULL</tt> can be passed in instead.
      * @param status for errors if any. If either the length of pattern or 
      *               text is 0 then an U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     StringSearch(const UnicodeString     &pattern, CharacterIterator &text,
                        RuleBasedCollator *coll, 
@@ -243,24 +256,37 @@ public:
      * Copy constructor that creates a StringSearch instance with the same 
      * behavior, and iterating over the same text.
      * @param that StringSearch instance to be copied.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     StringSearch(const StringSearch &that);
 
     /**
     * Destructor. Cleans up the search iterator data struct.
     * If a collator is created in the constructor, it will be destroyed here.
-    * @draft ICU 2.0
+    * @stable ICU 2.0
     */
     virtual ~StringSearch(void);
 
+    /**
+     * Clone this object.
+     * Clones can be used concurrently in multiple threads.
+     * If an error occurs, then NULL is returned.
+     * The caller must delete the clone.
+     *
+     * @return a clone of this object
+     *
+     * @see getDynamicClassID
+     * @stable ICU 2.8
+     */
+    StringSearch *clone() const;
+
     // operator overloading ---------------------------------------------
 
     /**
      * Assignment operator. Sets this iterator to have the same behavior,
      * and iterate over the same text, as the one passed in.
      * @param that instance to be copied.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     StringSearch & operator=(const StringSearch &that);
 
@@ -270,7 +296,7 @@ public:
      * @return TRUE if both instances have the same attributes, 
      *         breakiterators, collators and iterate over the same text 
      *         while looking for the same pattern.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const SearchIterator &that) const;
 
@@ -283,19 +309,21 @@ public:
      * This method takes the argument index and sets the position in the text 
      * string accordingly without checking if the index is pointing to a 
      * valid starting point to begin searching. 
-     * @param position within the text to be set
+     * @param position within the text to be set. If position is less
+     *          than or greater than the text range for searching, 
+     *          an U_INDEX_OUTOFBOUNDS_ERROR will be returned
      * @param status for errors if it occurs
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual void setOffset(int32_t position, UErrorCode &status);
 
     /**
      * Return the current index in the text being searched.
      * If the iteration has gone past the end of the text
-     * (or past the beginning for a backwards search), {@link #USEARCH_DONE} 
+     * (or past the beginning for a backwards search), USEARCH_DONE
      * is returned.
      * @return current index in the text being searched.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual int32_t getOffset(void) const;
 
@@ -308,7 +336,7 @@ public:
      * @param text text string to be searched
      * @param status for errors if any. If the text length is 0 then an 
      *        U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual void setText(const UnicodeString &text, UErrorCode &status);
     
@@ -324,18 +352,18 @@ public:
      * @param text text string to be searched
      * @param status for errors if any. If the text length is 0 then an 
      *        U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual void setText(CharacterIterator &text, UErrorCode &status);
 
     /**
-     * Gets the collator used for the language rules. 
+     * Gets the collator used for the language rules.
      * <p>
-     * Deleting the returned <tt>RuleBasedCollator</tt> before calling 
-     * the destructor would cause the string search to fail.
-     * The destructor will delete the collator if this instance owns it
+     * Caller may modify but <b>must not</b> delete the <tt>RuleBasedCollator</tt>!
+     * Modifications to this collator will affect the original collator passed in to 
+     * the <tt>StringSearch></tt> constructor or to setCollator, if any.
      * @return collator used for string search
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     RuleBasedCollator * getCollator() const;
     
@@ -347,7 +375,7 @@ public:
      * unchanged.
      * @param coll    collator 
      * @param status  for errors if any
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     void setCollator(RuleBasedCollator *coll, UErrorCode &status);
     
@@ -358,14 +386,14 @@ public:
      * @param pattern search pattern to be found
      * @param status for errors if any. If the pattern length is 0 then an 
      *               U_ILLEGAL_ARGUMENT_ERROR is returned.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     void setPattern(const UnicodeString &pattern, UErrorCode &status);
     
     /**
      * Gets the search pattern.
      * @return pattern used for matching
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     const UnicodeString & getPattern() const;
 
@@ -377,7 +405,7 @@ public:
      * iteration is initiated before a backwards iteration. Otherwise if 
      * a backwards iteration is initiated before a forwards iteration, the 
      * search will begin at the end of the text string.
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual void reset();
 
@@ -387,10 +415,24 @@ public:
      * replicated, except for the user-specified collator and the
      * breakiterator.
      * @return cloned object
-     * @draft ICU 2.0
+     * @stable ICU 2.0
      */
     virtual SearchIterator * safeClone(void) const;
     
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @stable ICU 2.2
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @stable ICU 2.2
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
+
 protected:
 
     // protected method -------------------------------------------------
@@ -398,15 +440,15 @@ protected:
     /**
      * Search forward for matching text, starting at a given location.
      * Clients should not call this method directly; instead they should 
-     * call {@link SearchIterator#next}.
+     * call {@link SearchIterator#next }.
      * <p>
      * If a match is found, this method returns the index at which the match
-     * starts and calls {@link SearchIterator#setMatchLength} with the number 
+     * starts and calls {@link SearchIterator#setMatchLength } with the number 
      * of characters in the target text that make up the match. If no match 
      * is found, the method returns <tt>USEARCH_DONE</tt>.
      * <p>
      * The <tt>StringSearch</tt> is adjusted so that its current index 
-     * (as returned by {@link #getOffset}) is the match position if one was 
+     * (as returned by {@link #getOffset }) is the match position if one was 
      * found.
      * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
      * the <tt>StringSearch</tt> will be adjusted to the index USEARCH_DONE.
@@ -415,6 +457,7 @@ protected:
      * @param status for errors if any occurs
      * @return The index at which the matched text in the target starts, or 
      *         USEARCH_DONE if no match was found.
+     * @stable ICU 2.0
      */
     virtual int32_t handleNext(int32_t position, UErrorCode &status);
 
@@ -424,12 +467,12 @@ protected:
      * <tt>SearchIterator.previous()</tt>, which this method overrides.
      * <p>
      * If a match is found, this method returns the index at which the match
-     * starts and calls {@link SearchIterator#setMatchLength} with the number 
+     * starts and calls {@link SearchIterator#setMatchLength } with the number 
      * of characters in the target text that make up the match. If no match 
      * is found, the method returns <tt>USEARCH_DONE</tt>.
      * <p>
      * The <tt>StringSearch</tt> is adjusted so that its current index 
-     * (as returned by {@link #getOffset}) is the match position if one was 
+     * (as returned by {@link #getOffset }) is the match position if one was 
      * found.
      * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
      * the <tt>StringSearch</tt> will be adjusted to the index USEARCH_DONE.
@@ -438,32 +481,36 @@ protected:
      * @param status for errors if any occurs
      * @return The index at which the matched text in the target starts, or 
      *         USEARCH_DONE if no match was found.
+     * @stable ICU 2.0
      */
     virtual int32_t handlePrev(int32_t position, UErrorCode &status);
     
 private :
+    StringSearch(); // default constructor not implemented
 
     // private data members ----------------------------------------------
 
     /**
     * RuleBasedCollator, contains exactly the same UCollator * in m_strsrch_
+    * @stable ICU 2.0
     */
     RuleBasedCollator  m_collator_;
     /**
     * Pattern text
+    * @stable ICU 2.0
     */
     UnicodeString      m_pattern_;
     /**
-    * Corresponding collation rules
-    */
-    UnicodeString      m_collation_rules_;
-    /**
     * String search struct data
+    * @stable ICU 2.0
     */
     UStringSearch     *m_strsrch_;
+
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 #endif
 
