@@ -1,6 +1,7 @@
 /*
 *******************************************************************************
-* Copyright (C) 1996-2001, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 1996-2006, International Business Machines Corporation and others.
+* All Rights Reserved.
 *******************************************************************************
 */
 
@@ -8,6 +9,9 @@
 #define UDAT_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/ucal.h"
 #include "unicode/unum.h"
 /**
@@ -35,7 +39,7 @@
  *  UErrorCode status = U_ZERO_ERROR;
  *  UChar *myString;
  *  int32_t myStrlen = 0;
- *  UDateFormat* dfmt = udat_open(UCAL_DEFAULT, UCAL_DEFAULT, NULL, "PST", &status);
+ *  UDateFormat* dfmt = udat_open(UDAT_DEFAULT, UDAT_DEFAULT, NULL, NULL, -1, NULL, -1, &status);
  *  myStrlen = udat_format(dfmt, myDate, NULL, myStrlen, NULL, &status);
  *  if (status==U_BUFFER_OVERFLOW_ERROR){
  *      status=U_ZERO_ERROR;
@@ -54,7 +58,7 @@
  *  UChar* myString;
  *  char buffer[1024];
  *  UDate myDateArr[] = { 0.0, 100000000.0, 2000000000.0 }; // test values
- *  UDateFormat* df = udat_open(UCAL_DEFAULT, UCAL_DEFAULT, NULL, "GMT", &status);
+ *  UDateFormat* df = udat_open(UDAT_DEFAULT, UDAT_DEFAULT, NULL, NULL, -1, NULL, 0, &status);
  *  for (i = 0; i < 3; i++) {
  *      myStrlen = udat_format(df, myDateArr[i], NULL, myStrlen, NULL, &status);
  *      if(status == U_BUFFER_OVERFLOW_ERROR){
@@ -78,7 +82,7 @@
  *  char buffer[1024];
  *
  *  pos.field = 1;  // Same as the DateFormat::EField enum
- *  UDateFormat* dfmt = udat_open(UCAL_DEFAULT, UCAL_DEFAULT, NULL, "PST", &status);
+ *  UDateFormat* dfmt = udat_open(UDAT_DEFAULT, UDAT_DEFAULT, NULL, -1, NULL, 0, &status);
  *  myStrlen = udat_format(dfmt, myDate, NULL, myStrlen, &pos, &status);
  *  if (status==U_BUFFER_OVERFLOW_ERROR){
  *      status=U_ZERO_ERROR;
@@ -94,7 +98,7 @@
  * udat_open()
  * <pre>
  * \code
- *        UDateFormat* df = udat_open(UDAT_SHORT, UDAT_SHORT, "fr_FR", "GMT", &status);
+ *        UDateFormat* df = udat_open(UDAT_SHORT, UDAT_SHORT, "fr_FR", NULL, -1, NULL, 0, &status);
  * \endcode
  * </pre>
  * You can use a DateFormat API udat_parse() to parse.
@@ -129,11 +133,14 @@
 
 /** A date formatter.
  *  For usage in C programs.
+ *  @stable ICU 2.6
  */
 typedef void* UDateFormat;
 
-/** The possible date/time format styles */
-enum UDateFormatStyle {
+/** The possible date/time format styles 
+ *  @stable ICU 2.6
+ */
+typedef enum UDateFormatStyle {
     /** Full style */
     UDAT_FULL,
     /** Long style */
@@ -149,31 +156,268 @@ enum UDateFormatStyle {
     /** for internal API use only */
     UDAT_IGNORE = -2
 
-};
-typedef enum UDateFormatStyle UDateFormatStyle;
+} UDateFormatStyle;
+
+/**
+ * FieldPosition and UFieldPosition selectors for format fields
+ * defined by DateFormat and UDateFormat.
+ * @stable ICU 3.0
+ */
+typedef enum UDateFormatField {
+    /**
+     * FieldPosition and UFieldPosition selector for 'G' field alignment,
+     * corresponding to the UCAL_ERA field.
+     * @stable ICU 3.0
+     */
+    UDAT_ERA_FIELD = 0,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'y' field alignment,
+     * corresponding to the UCAL_YEAR field.
+     * @stable ICU 3.0
+     */
+    UDAT_YEAR_FIELD = 1,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'M' field alignment,
+     * corresponding to the UCAL_MONTH field.
+     * @stable ICU 3.0
+     */
+    UDAT_MONTH_FIELD = 2,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'd' field alignment,
+     * corresponding to the UCAL_DATE field.
+     * @stable ICU 3.0
+     */
+    UDAT_DATE_FIELD = 3,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'k' field alignment,
+     * corresponding to the UCAL_HOUR_OF_DAY field.
+     * UDAT_HOUR_OF_DAY1_FIELD is used for the one-based 24-hour clock.
+     * For example, 23:59 + 01:00 results in 24:59.
+     * @stable ICU 3.0
+     */
+    UDAT_HOUR_OF_DAY1_FIELD = 4,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'H' field alignment,
+     * corresponding to the UCAL_HOUR_OF_DAY field.
+     * UDAT_HOUR_OF_DAY0_FIELD is used for the zero-based 24-hour clock.
+     * For example, 23:59 + 01:00 results in 00:59.
+     * @stable ICU 3.0
+     */
+    UDAT_HOUR_OF_DAY0_FIELD = 5,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'm' field alignment,
+     * corresponding to the UCAL_MINUTE field.
+     * @stable ICU 3.0
+     */
+    UDAT_MINUTE_FIELD = 6,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 's' field alignment,
+     * corresponding to the UCAL_SECOND field.
+     * @stable ICU 3.0
+     */
+    UDAT_SECOND_FIELD = 7,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'S' field alignment,
+     * corresponding to the UCAL_MILLISECOND field.
+     * @stable ICU 3.0
+     */
+    UDAT_FRACTIONAL_SECOND_FIELD = 8,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'E' field alignment,
+     * corresponding to the UCAL_DAY_OF_WEEK field.
+     * @stable ICU 3.0
+     */
+    UDAT_DAY_OF_WEEK_FIELD = 9,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'D' field alignment,
+     * corresponding to the UCAL_DAY_OF_YEAR field.
+     * @stable ICU 3.0
+     */
+    UDAT_DAY_OF_YEAR_FIELD = 10,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'F' field alignment,
+     * corresponding to the UCAL_DAY_OF_WEEK_IN_MONTH field.
+     * @stable ICU 3.0
+     */
+    UDAT_DAY_OF_WEEK_IN_MONTH_FIELD = 11,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'w' field alignment,
+     * corresponding to the UCAL_WEEK_OF_YEAR field.
+     * @stable ICU 3.0
+     */
+    UDAT_WEEK_OF_YEAR_FIELD = 12,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'W' field alignment,
+     * corresponding to the UCAL_WEEK_OF_MONTH field.
+     * @stable ICU 3.0
+     */
+    UDAT_WEEK_OF_MONTH_FIELD = 13,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'a' field alignment,
+     * corresponding to the UCAL_AM_PM field.
+     * @stable ICU 3.0
+     */
+    UDAT_AM_PM_FIELD = 14,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'h' field alignment,
+     * corresponding to the UCAL_HOUR field.
+     * UDAT_HOUR1_FIELD is used for the one-based 12-hour clock.
+     * For example, 11:30 PM + 1 hour results in 12:30 AM.
+     * @stable ICU 3.0
+     */
+    UDAT_HOUR1_FIELD = 15,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'K' field alignment,
+     * corresponding to the UCAL_HOUR field.
+     * UDAT_HOUR0_FIELD is used for the zero-based 12-hour clock.
+     * For example, 11:30 PM + 1 hour results in 00:30 AM.
+     * @stable ICU 3.0
+     */
+    UDAT_HOUR0_FIELD = 16,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'z' field alignment,
+     * corresponding to the UCAL_ZONE_OFFSET and
+     * UCAL_DST_OFFSET fields.
+     * @stable ICU 3.0
+     */
+    UDAT_TIMEZONE_FIELD = 17,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'Y' field alignment,
+     * corresponding to the UCAL_YEAR_WOY field.
+     * @stable ICU 3.0
+     */
+    UDAT_YEAR_WOY_FIELD = 18,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'e' field alignment,
+     * corresponding to the UCAL_DOW_LOCAL field.
+     * @stable ICU 3.0
+     */
+    UDAT_DOW_LOCAL_FIELD = 19,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'u' field alignment,
+     * corresponding to the UCAL_EXTENDED_YEAR field.
+     * @stable ICU 3.0
+     */
+    UDAT_EXTENDED_YEAR_FIELD = 20,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'g' field alignment,
+     * corresponding to the UCAL_JULIAN_DAY field.
+     * @stable ICU 3.0
+     */
+    UDAT_JULIAN_DAY_FIELD = 21,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'A' field alignment,
+     * corresponding to the UCAL_MILLISECONDS_IN_DAY field.
+     * @stable ICU 3.0
+     */
+    UDAT_MILLISECONDS_IN_DAY_FIELD = 22,
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'Z' field alignment,
+     * corresponding to the UCAL_ZONE_OFFSET and
+     * UCAL_DST_OFFSET fields.
+     * @stable ICU 3.0
+     */
+    UDAT_TIMEZONE_RFC_FIELD = 23,
+
+#ifndef U_HIDE_DRAFT_API
+
+    /**
+     * FieldPosition and UFieldPosition selector for 'v' field alignment,
+     * corresponding to the UCAL_ZONE_OFFSET field.
+     * @draft ICU 3.4
+     */
+    UDAT_TIMEZONE_GENERIC_FIELD = 24,
+    /**
+     * FieldPosition selector for 'c' field alignment,
+     * corresponding to the {@link Calendar#DAY} field. 
+     * This displays the stand alone day name, if available.
+     * @draft ICU 3.4
+     */
+    UDAT_STANDALONE_DAY_FIELD = 25,
+    
+    /**
+     * FieldPosition selector for 'L' field alignment,
+     * corresponding to the {@link Calendar#MONTH} field.  
+     * This displays the stand alone month name, if available.
+     * @draft ICU 3.4
+     */
+    UDAT_STANDALONE_MONTH_FIELD = 26,
+
+    /**
+     * FieldPosition selector for "Q" field alignment,
+     * corresponding to quarters. This is implemented
+     * using the {@link Calendar#MONTH} field. This
+     * displays the quarter.
+     * @draft ICU 3.6
+     */
+    UDAT_QUARTER_FIELD = 27,
+
+    /**
+     * FieldPosition selector for the "q" field alignment,
+     * corresponding to stand-alone quarters. This is
+     * implemented using the {@link Calendar#MONTH} field.
+     * This displays the stand-alone quarter.
+     * @draft ICU 3.6
+     */
+    UDAT_STANDALONE_QUARTER_FIELD = 28,
+
+#endif /*U_HIDE_DRAFT_API*/
+   
+   /**
+     * Number of FieldPosition and UFieldPosition selectors for 
+     * DateFormat and UDateFormat.
+     * Valid selectors range from 0 to UDAT_FIELD_COUNT-1.
+     * This value is subject to change if new fields are defined
+     * in the future.
+     * @stable ICU 3.0
+     */
+    UDAT_FIELD_COUNT = 29
+
+} UDateFormatField;
 
 /**
  * Open a new UDateFormat for formatting and parsing dates and times.
- * A UDateFormat may be used to format dates in calls to \Ref{udat_format},
- * and to parse dates in calls to \Ref{udat_parse}.
- * @param timeStyle The style used to format times; one of UDAT_FULL_STYLE, UDAT_LONG_STYLE,
- * UDAT_MEDIUM_STYLE, UDAT_SHORT_STYLE, or UDAT_DEFAULT_STYLE
- * @param dateStyle The style used to format dates; one of UDAT_FULL_STYLE, UDAT_LONG_STYLE,
- * UDAT_MEDIUM_STYLE, UDAT_SHORT_STYLE, or UDAT_DEFAULT_STYLE
+ * A UDateFormat may be used to format dates in calls to {@link #udat_format },
+ * and to parse dates in calls to {@link #udat_parse }.
+ * @param timeStyle The style used to format times; one of UDAT_FULL, UDAT_LONG,
+ * UDAT_MEDIUM, UDAT_SHORT, or UDAT_DEFAULT
+ * @param dateStyle The style used to format dates; one of UDAT_FULL, UDAT_LONG,
+ * UDAT_MEDIUM, UDAT_SHORT, or UDAT_DEFAULT
  * @param locale The locale specifying the formatting conventions
  * @param tzID A timezone ID specifying the timezone to use.  If 0, use
  * the default timezone.
  * @param tzIDLength The length of tzID, or -1 if null-terminated.
- * @param status A pointer to an UErrorCode to receive any errors
  * @param pattern A pattern specifying the format to use.
  * @param patternLength The number of characters in the pattern, or -1 if null-terminated.
- * @param locale The locale specifying the formatting conventions
  * @param status A pointer to an UErrorCode to receive any errors
  * @return A pointer to a UDateFormat to use for formatting dates and times, or 0 if
  * an error occurred.
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI UDateFormat* U_EXPORT2 
+U_STABLE UDateFormat* U_EXPORT2 
 udat_open(UDateFormatStyle  timeStyle,
           UDateFormatStyle  dateStyle,
           const char        *locale,
@@ -187,10 +431,10 @@ udat_open(UDateFormatStyle  timeStyle,
 /**
 * Close a UDateFormat.
 * Once closed, a UDateFormat may no longer be used.
-* @param fmt The formatter to close.
-* @stable
+* @param format The formatter to close.
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_close(UDateFormat* format);
 
 /**
@@ -199,16 +443,15 @@ udat_close(UDateFormat* format);
  * @param fmt The format to copy
  * @param status A pointer to an UErrorCode to receive any errors.
  * @return A pointer to a UDateFormat identical to fmt.
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI UDateFormat* U_EXPORT2 
+U_STABLE UDateFormat* U_EXPORT2 
 udat_clone(const UDateFormat *fmt,
        UErrorCode *status);
 
 /**
 * Format a date using an UDateFormat.
-* The date will be formatted using the conventions specified in \Ref{udat_open}
-* or \Ref{udat_openPattern}
+* The date will be formatted using the conventions specified in {@link #udat_open }
 * @param format The formatter to use
 * @param dateToFormat The date to format
 * @param result A pointer to a buffer to receive the formatted number.
@@ -222,9 +465,9 @@ udat_clone(const UDateFormat *fmt,
 * @return The total buffer size needed; if greater than resultLength, the output was truncated.
 * @see udat_parse
 * @see UFieldPosition
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 udat_format(    const    UDateFormat*    format,
                         UDate           dateToFormat,
                         UChar*          result,
@@ -234,9 +477,8 @@ udat_format(    const    UDateFormat*    format,
 
 /**
 * Parse a string into an date/time using a UDateFormat.
-* The date will be parsed using the conventions specified in \Ref{udat_open}
-* or \Ref{udat_openPattern}
-* @param fmt The formatter to use.
+* The date will be parsed using the conventions specified in {@link #udat_open }
+* @param format The formatter to use.
 * @param text The text to parse.
 * @param textLength The length of text, or -1 if null-terminated.
 * @param parsePos If not 0, on input a pointer to an integer specifying the offset at which
@@ -244,9 +486,9 @@ udat_format(    const    UDateFormat*    format,
 * @param status A pointer to an UErrorCode to receive any errors
 * @return The value of the parsed date/time
 * @see udat_format
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI UDate U_EXPORT2 
+U_STABLE UDate U_EXPORT2 
 udat_parse(    const    UDateFormat*    format,
             const    UChar*          text,
                     int32_t         textLength,
@@ -255,9 +497,8 @@ udat_parse(    const    UDateFormat*    format,
 
 /**
 * Parse a string into an date/time using a UDateFormat.
-* The date will be parsed using the conventions specified in \Ref{udat_open}
-* or \Ref{udat_openPattern}
-* @param fmt The formatter to use.
+* The date will be parsed using the conventions specified in {@link #udat_open }
+* @param format The formatter to use.
 * @param calendar The calendar in which to store the parsed data.
 * @param text The text to parse.
 * @param textLength The length of text, or -1 if null-terminated.
@@ -265,9 +506,9 @@ udat_parse(    const    UDateFormat*    format,
 * to begin parsing.  If not 0, on output the offset at which parsing ended.
 * @param status A pointer to an UErrorCode to receive any errors
 * @see udat_format
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_parseCalendar(const    UDateFormat*    format,
                             UCalendar*      calendar,
                    const    UChar*          text,
@@ -282,9 +523,9 @@ udat_parseCalendar(const    UDateFormat*    format,
 * @param fmt The formatter to query
 * @return TRUE if fmt is set to perform lenient parsing, FALSE otherwise.
 * @see udat_setLenient
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI UBool U_EXPORT2 
+U_STABLE UBool U_EXPORT2 
 udat_isLenient(const UDateFormat* fmt);
 
 /**
@@ -294,9 +535,9 @@ udat_isLenient(const UDateFormat* fmt);
 * @param fmt The formatter to set
 * @param isLenient TRUE if fmt should perform lenient parsing, FALSE otherwise.
 * @see dat_isLenient
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_setLenient(    UDateFormat*    fmt,
                     UBool          isLenient);
 
@@ -307,9 +548,9 @@ udat_setLenient(    UDateFormat*    fmt,
 * @param fmt The formatter to query.
 * @return A pointer to the UCalendar used by fmt.
 * @see udat_setCalendar
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI const UCalendar* U_EXPORT2 
+U_STABLE const UCalendar* U_EXPORT2 
 udat_getCalendar(const UDateFormat* fmt);
 
 /**
@@ -319,9 +560,9 @@ udat_getCalendar(const UDateFormat* fmt);
 * @param fmt The formatter to set.
 * @param calendarToSet A pointer to an UCalendar to be used by fmt.
 * @see udat_setCalendar
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_setCalendar(            UDateFormat*    fmt,
                     const   UCalendar*      calendarToSet);
 
@@ -332,9 +573,9 @@ udat_setCalendar(            UDateFormat*    fmt,
 * @param fmt The formatter to query.
 * @return A pointer to the UNumberFormat used by fmt to format numbers.
 * @see udat_setNumberFormat
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI const UNumberFormat* U_EXPORT2 
+U_STABLE const UNumberFormat* U_EXPORT2 
 udat_getNumberFormat(const UDateFormat* fmt);
 
 /**
@@ -344,9 +585,9 @@ udat_getNumberFormat(const UDateFormat* fmt);
 * @param fmt The formatter to set.
 * @param numberFormatToSet A pointer to the UNumberFormat to be used by fmt to format numbers.
 * @see udat_getNumberFormat
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_setNumberFormat(            UDateFormat*    fmt,
                         const   UNumberFormat*  numberFormatToSet);
 
@@ -357,20 +598,20 @@ udat_setNumberFormat(            UDateFormat*    fmt,
 * @param index The index of the desired locale.
 * @return A locale for which date/time formatting patterns are available, or 0 if none.
 * @see udat_countAvailable
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI const char* U_EXPORT2 
+U_STABLE const char* U_EXPORT2 
 udat_getAvailable(int32_t index);
 
 /**
 * Determine how many locales have date/time  formatting patterns available.
 * This function is most useful as determining the loop ending condition for
-* calls to \Ref{udat_getAvailable}.
+* calls to {@link #udat_getAvailable }.
 * @return The number of locales for which date/time formatting patterns are available.
 * @see udat_getAvailable
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 udat_countAvailable(void);
 
 /**
@@ -381,9 +622,9 @@ udat_countAvailable(void);
 * @param status A pointer to an UErrorCode to receive any errors
 * @return The year relative to which all 2-digit years are interpreted.
 * @see udat_Set2DigitYearStart
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI UDate U_EXPORT2 
+U_STABLE UDate U_EXPORT2 
 udat_get2DigitYearStart(    const   UDateFormat     *fmt,
                                     UErrorCode      *status);
 
@@ -395,9 +636,9 @@ udat_get2DigitYearStart(    const   UDateFormat     *fmt,
 * @param d The year relative to which all 2-digit years will be interpreted.
 * @param status A pointer to an UErrorCode to receive any errors
 * @see udat_Set2DigitYearStart
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_set2DigitYearStart(    UDateFormat     *fmt,
                             UDate           d,
                             UErrorCode      *status);
@@ -412,9 +653,9 @@ udat_set2DigitYearStart(    UDateFormat     *fmt,
 * @param status A pointer to an UErrorCode to receive any errors
 * @return The total buffer size needed; if greater than resultLength, the output was truncated.
 * @see udat_applyPattern
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 udat_toPattern(    const   UDateFormat     *fmt,
                         UBool          localized,
                         UChar           *result,
@@ -424,21 +665,24 @@ udat_toPattern(    const   UDateFormat     *fmt,
 /**
 * Set the pattern used by an UDateFormat.
 * The pattern should follow the pattern syntax rules.
-* @param fmt The formatter to set.
+* @param format The formatter to set.
 * @param localized TRUE if the pattern is localized, FALSE otherwise.
 * @param pattern The new pattern
 * @param patternLength The length of pattern, or -1 if null-terminated.
 * @see udat_toPattern
-* @draft
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_applyPattern(            UDateFormat     *format,
                             UBool          localized,
                     const   UChar           *pattern,
                             int32_t         patternLength);
 
-/** The possible types of date format symbols */
-enum UDateFormatSymbolType {
+/** 
+ * The possible types of date format symbols 
+ * @stable ICU 2.6
+ */
+typedef enum UDateFormatSymbolType {
     /** The era names, for example AD */
     UDAT_ERAS,
     /** The month names, for example February */
@@ -452,14 +696,36 @@ enum UDateFormatSymbolType {
     /** The AM/PM names, for example AM */
     UDAT_AM_PMS,
     /** The localized characters */
-    UDAT_LOCALIZED_CHARS
-};
-typedef enum UDateFormatSymbolType UDateFormatSymbolType;
+    UDAT_LOCALIZED_CHARS,
+    /** The long era names, for example Anno Domini */
+    UDAT_ERA_NAMES,
+    /** The narrow month names, for example F */
+    UDAT_NARROW_MONTHS,
+    /** The narrow weekday names, for example N */
+    UDAT_NARROW_WEEKDAYS,
+    /** Standalone context versions of months */
+    UDAT_STANDALONE_MONTHS,
+    UDAT_STANDALONE_SHORT_MONTHS,
+    UDAT_STANDALONE_NARROW_MONTHS,
+    /** Standalone context versions of weekdays */
+    UDAT_STANDALONE_WEEKDAYS,
+    UDAT_STANDALONE_SHORT_WEEKDAYS,
+    UDAT_STANDALONE_NARROW_WEEKDAYS,
+    /** The quarters, for example 1st Quarter */
+    UDAT_QUARTERS,
+    /** The short quarter names, for example Q1 */
+    UDAT_SHORT_QUARTERS,
+    /** Standalone context versions of quarters */
+    UDAT_STANDALONE_QUARTERS,
+    UDAT_STANDALONE_SHORT_QUARTERS
 
+} UDateFormatSymbolType;
+
+struct UDateFormatSymbols;
 /** Date format symbols.
  *  For usage in C programs.
+ *  @stable ICU 2.6
  */
-struct UDateFormatSymbols;
 typedef struct UDateFormatSymbols UDateFormatSymbols;
 
 /**
@@ -476,9 +742,9 @@ typedef struct UDateFormatSymbols UDateFormatSymbols;
 * @return The total buffer size needed; if greater than resultLength, the output was truncated.
 * @see udat_countSymbols
 * @see udat_setSymbols
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 udat_getSymbols(const   UDateFormat             *fmt,
                         UDateFormatSymbolType   type,
                         int32_t                 index,
@@ -489,16 +755,16 @@ udat_getSymbols(const   UDateFormat             *fmt,
 /**
 * Count the number of particular symbols for an UDateFormat.
 * This function is most useful as for detemining the loop termination condition
-* for calls to \Ref{udat_getSymbols}.
+* for calls to {@link #udat_getSymbols }.
 * @param fmt The formatter to query.
 * @param type The type of symbols to count.  One of UDAT_ERAS, UDAT_MONTHS, UDAT_SHORT_MONTHS,
 * UDAT_WEEKDAYS, UDAT_SHORT_WEEKDAYS, UDAT_AM_PMS, or UDAT_LOCALIZED_CHARS
 * @return The number of symbols of type type.
 * @see udat_getSymbols
 * @see udat_setSymbols
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 udat_countSymbols(    const    UDateFormat                *fmt,
                             UDateFormatSymbolType    type);
 
@@ -506,7 +772,7 @@ udat_countSymbols(    const    UDateFormat                *fmt,
 * Set the symbols associated with an UDateFormat.
 * The symbols are what a UDateFormat uses to represent locale-specific data,
 * for example month or day names.
-* @param fmt The formatter to set
+* @param format The formatter to set
 * @param type The type of symbols to set.  One of UDAT_ERAS, UDAT_MONTHS, UDAT_SHORT_MONTHS,
 * UDAT_WEEKDAYS, UDAT_SHORT_WEEKDAYS, UDAT_AM_PMS, or UDAT_LOCALIZED_CHARS
 * @param index The index of the symbol to set of type type.
@@ -515,9 +781,9 @@ udat_countSymbols(    const    UDateFormat                *fmt,
 * @param status A pointer to an UErrorCode to receive any errors
 * @see udat_getSymbols
 * @see udat_countSymbols
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 udat_setSymbols(    UDateFormat             *format,
                     UDateFormatSymbolType   type,
                     int32_t                 index,
@@ -525,22 +791,20 @@ udat_setSymbols(    UDateFormat             *format,
                     int32_t                 valueLength,
                     UErrorCode              *status);
 
-/********************* Deprecated API ************************************/
 /**
- *@deprecated Remove after Aug 2002
+ * Get the locale for this date format object.
+ * You can choose between valid and actual locale.
+ * @param fmt The formatter to get the locale from
+ * @param type type of the locale we're looking for (valid or actual) 
+ * @param status error code for the operation
+ * @return the locale name
+ * @stable ICU 2.8
  */
-#ifdef U_USE_DEPRECATED_FORMAT_API
-#if ((U_ICU_VERSION_MAJOR_NUM != 2) || (U_ICU_VERSION_MINOR_NUM !=1))
-#   error "ICU version has changed. Please redefine the macros under U_USE_DEPRECATED_FORMAT_API pre-processor definition"
-#else 
-    static UDateFormat*
-    udat_openPattern(const UChar* pattern,int32_t patternLength,const char* locale,UErrorCode *status)
-    {
-        return udat_open(UDAT_IGNORE,UDAT_IGNORE,locale,NULL,0,pattern,patternLength,status);
-    }
+U_STABLE const char* U_EXPORT2
+udat_getLocaleByType(const UDateFormat *fmt,
+                     ULocDataLocaleType type,
+                     UErrorCode* status); 
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
 
-#   define udat_open_2_1(timeStyle,dateStyle,locale,tzId,tzIdLength,status) udat_open(timeStyle,dateStyle,locale,tzId,tzIdLength,NULL,0,status)
-#endif
-#endif
-/********************* End **********************************************/
 #endif
