@@ -1,8 +1,7 @@
 
 /*
- * @(#)LESwaps.h	1.3 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2005 - All Rights Reserved
  *
  */
 
@@ -11,14 +10,44 @@
 
 #include "LETypes.h"
 
+/**
+ * \file 
+ * \brief C++ API: Endian independent access to data for LayoutEngine
+ */
+
 U_NAMESPACE_BEGIN
 
 /**
- * These are convenience macros which invoke the swap functions
+ * A convenience macro which invokes the swapWord member function
  * from a concise call.
+ *
+ * @stable ICU 2.8
  */
-#define SWAPW(value) (LESwaps::isBigEndian() ? (value) : LESwaps::swapWord(value))
-#define SWAPL(value) (LESwaps::isBigEndian() ? (value) : LESwaps::swapLong(value))
+#if defined(U_IS_BIG_ENDIAN)
+    #if U_IS_BIG_ENDIAN
+        #define SWAPW(value) (value)
+    #else
+        #define SWAPW(value) LESwaps::swapWord(value)
+    #endif
+#else
+    #define SWAPW(value) (LESwaps::isBigEndian() ? (value) : LESwaps::swapWord(value))
+#endif
+
+/**
+ * A convenience macro which invokes the swapLong member function
+ * from a concise call.
+ *
+ * @stable ICU 2.8
+ */
+#if defined(U_IS_BIG_ENDIAN)
+    #if U_IS_BIG_ENDIAN
+        #define SWAPL(value) (value)
+    #else
+        #define SWAPL(value) LESwaps::swapLong(value)
+    #endif
+#else
+    #define SWAPL(value) (LESwaps::isBigEndian() ? (value) : LESwaps::swapLong(value))
+#endif
 
 /**
  * This class is used to access data which stored in big endian order
@@ -28,11 +57,13 @@ U_NAMESPACE_BEGIN
  *
  * All methods are static and inline in an attempt to induce the compiler
  * to do most of the calculations at compile time.
+ *
+ * @stable ICU 2.8
  */
-class LESwaps
-{
+class U_LAYOUT_API LESwaps /* not : public UObject because all methods are static */ {
 public:
 
+#if !defined(U_IS_BIG_ENDIAN)
     /**
      * This method detects the endian-ness of the platform by
      * casting a pointer to a word to a pointer to a byte. On
@@ -40,22 +71,27 @@ public:
      * lowest address. On little endian platforms, the FF will
      * be in the byte with the highest address.
      *
-     * @return true if the platform is big endian
+     * @return TRUE if the platform is big endian
+     *
+     * @stable ICU 2.8
      */
-    static le_bool isBigEndian()
+    static le_uint8 isBigEndian()
     {
-        static const le_uint16 word = 0xFF00;
+        const le_uint16 word = 0xFF00;
 
         return *((le_uint8 *) &word);
     };
+#endif
 
     /**
-     * This method the byte swap required on little endian platforms
-     * to correctly access a word.
+     * This method does the byte swap required on little endian platforms
+     * to correctly access a (16-bit) word.
      *
      * @param value - the word to be byte swapped
      *
      * @return the byte swapped word
+     *
+     * @stable ICU 2.8
      */
     static le_uint16 swapWord(le_uint16 value)
     {
@@ -64,16 +100,21 @@ public:
 
     /**
      * This method does the byte swapping required on little endian platforms
-     * to correctly access a long.
+     * to correctly access a (32-bit) long.
      *
      * @param value - the long to be byte swapped
      *
      * @return the byte swapped long
+     *
+     * @stable ICU 2.8
      */
     static le_uint32 swapLong(le_uint32 value)
     {
         return swapWord((le_uint16) (value >> 16)) | (swapWord((le_uint16) value) << 16);
     };
+
+private:
+    LESwaps() {} // private - forbid instantiation
 };
 
 U_NAMESPACE_END
