@@ -1,8 +1,5 @@
-
 /*
- * %W% %E%
- *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001, 2002 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2006 - All Rights Reserved
  *
  */
 
@@ -42,8 +39,10 @@ U_NAMESPACE_BEGIN
  * such as kerning and accent positioning.
  *
  * @see LayoutEngine
+ *
+ * @internal
  */
-class OpenTypeLayoutEngine : public LayoutEngine
+class U_LAYOUT_API OpenTypeLayoutEngine : public LayoutEngine
 {
 public:
     /**
@@ -59,9 +58,11 @@ public:
      *
      * @see LayoutEngine::layoutEngineFactory
      * @see ScriptAndLangaugeTags.h for script and language codes
+     *
+     * @internal
      */
     OpenTypeLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode,
-                            const GlyphSubstitutionTableHeader *gsubTable);
+                            le_int32 typoFlags, const GlyphSubstitutionTableHeader *gsubTable);
 
     /**
      * This constructor is used when the font requires a "canned" GSUB table which can't be known
@@ -70,11 +71,16 @@ public:
      * @param fontInstance - the font
      * @param scriptCode - the script
      * @param langaugeCode - the language
+     *
+     * @internal
      */
-    OpenTypeLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode);
+    OpenTypeLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode,
+			 le_int32 typoFlags);
 
     /**
      * The destructor, virtual for correct polymorphic invocation.
+     *
+     * @internal
      */
     virtual ~OpenTypeLayoutEngine();
 
@@ -85,6 +91,8 @@ public:
      * @param scriptCode - the script code
      *
      * @return the four byte script tag
+     *
+     * @internal
      */
     static LETag getScriptTag(le_int32 scriptCode);
 
@@ -95,9 +103,32 @@ public:
      * @param languageCode - the language code
      *
      * @return the four byte language tag
+     *
+     * @internal
      */
     static LETag getLangSysTag(le_int32 languageCode);
 
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @stable ICU 2.8
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @stable ICU 2.8
+     */
+    static UClassID getStaticClassID();
+
+    /**
+     * The array of language tags, indexed by language code.
+     *
+     * @internal
+     */
+    static const LETag languageTags[];
+
 private:
 
     /**
@@ -113,31 +144,56 @@ private:
 
 protected:
     /**
-     * An array of pointers to four byte feature tags.
-     * Each pointer points to a list of tags, terminated
-     * by a special empty tag.
+     * A set of "default" features. The default characterProcessing method
+     * will apply all of these features to every glyph.
+     *
+     * @internal
      */
-    const LETag **fFeatureTags;
+    FeatureMask fFeatureMask;
 
-        /**
-         * A list of tags in the order in which the features in
-         * the font should be applied, as opposed to using the
-         * order of the lookups in the font.
-         */
-    const LETag *fFeatureOrder;
+    /**
+     * A set of mappings from feature tags to feature masks. These may
+     * be in the order in which the featues should be applied, but they
+     * don't need to be.
+     *
+     * @internal
+     */
+    const FeatureMap *fFeatureMap;
+
+    /**
+     * The length of the feature map.
+     *
+     * @internal
+     */
+    le_int32 fFeatureMapCount;
+
+    /**
+     * <code>TRUE</code> if the features in the
+     * feature map are in the order in which they
+     * must be applied.
+     *
+     * @internal
+     */
+    le_bool fFeatureOrder;
 
     /**
      * The address of the GSUB table.
+     *
+     * @internal
      */
     const GlyphSubstitutionTableHeader *fGSUBTable;
 
     /**
      * The address of the GDEF table.
+     *
+     * @internal
      */
     const GlyphDefinitionTableHeader   *fGDEFTable;
 
     /**
      * The address of the GPOS table.
+     *
+     * @internal
      */
     const GlyphPositioningTableHeader  *fGPOSTable;
 
@@ -146,23 +202,37 @@ protected:
      * preformed by the GSUB table. This is used for some
      * "canned" GSUB tables to restrict substitutions to
      * glyphs that are in the font.
+     *
+     * @internal
      */
     LEGlyphFilter *fSubstitutionFilter;
 
     /**
      * The four byte script tag.
+     *
+     * @internal
      */
     LETag fScriptTag;
 
     /**
      * The four byte language tag
+     *
+     * @internal
      */
     LETag fLangSysTag;
 
+    /**
+     * <code>TRUE</code> if <code>mapCharsToGlyphs</code> should replace ZWJ / ZWNJ with a glyph
+     * with no contours.
+     *
+     * @internal
+     */
+    le_bool fFilterZeroWidth;
+
     /**
      * This method does the OpenType character processing. It assigns the OpenType feature
      * tags to the characters, and may generate output characters that differ from the input
-     * charcters dueto insertions, deletions, or reorderings. In such cases, it will also
+     * charcters due to insertions, deletions, or reorderings. In such cases, it will also
      * generate an output character index array reflecting these changes.
      *
      * Subclasses must override this method.
@@ -172,7 +242,7 @@ protected:
      * @param offset - the index of the first character to process
      * @param count - the number of characters to process
      * @param max - the number of characters in the input context
-     * @param rightToLeft - true if the characters are in a right to left directional run
+     * @param rightToLeft - TRUE if the characters are in a right to left directional run
      *
      * Output parameters:
      * @param outChars - the output character array, if different from the input
@@ -181,21 +251,11 @@ protected:
      * @param success - set to an error code if the operation fails
      *
      * @return the output character count (input character count if no change)
+     *
+     * @internal
      */
-    virtual le_int32 characterProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
-            LEUnicode *&outChars, le_int32 *&charIndices, const LETag **&featureTags, LEErrorCode &success) /*= 0;*/
-    {
-        if (LE_FAILURE(success)) {
-            return 0;
-        }
-
-        if (offset < 0 || count < 0 || max < 0 || offset >= max || offset + count > max) {
-            success = LE_ILLEGAL_ARGUMENT_ERROR;
-            return 0;
-        }
-
-        return count;
-    };
+    virtual le_int32 characterProcessing(const LEUnicode /*chars*/[], le_int32 offset, le_int32 count, le_int32 max, le_bool /*rightToLeft*/,
+            LEUnicode *&/*outChars*/, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method does character to glyph mapping, and applies the GSUB table. The
@@ -211,7 +271,7 @@ protected:
      * @param offset - the index of the first character to process
      * @param count - the number of characters to process
      * @param max - the number of characters in the input context
-     * @param rightToLeft - true if the characters are in a right to left directional run
+     * @param rightToLeft - TRUE if the characters are in a right to left directional run
      * @param featureTags - the feature tag array
      *
      * Output parameters:
@@ -223,9 +283,11 @@ protected:
      *
      * Note: if the character index array was already set by the characterProcessing
      * method, this method won't change it.
+     *
+     * @internal
      */
     virtual le_int32 glyphProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
-            const LETag **featureTags, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
+            LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method does any processing necessary to convert "fake"
@@ -249,19 +311,10 @@ protected:
      * @param success - set to an error code if the operation fails
      *
      * @return the number of glyph indices in the output glyph index array
+     *
+     * @internal
      */
-    virtual le_int32 glyphPostProcessing(LEGlyphID tempGlyphs[], le_int32 tempCharIndices[], le_int32 tempGlyphCount,
-                    LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success)
-    {
-        if (LE_FAILURE(success)) {
-            return 0;
-        }
-
-        glyphs = tempGlyphs;
-        charIndices = tempCharIndices;
-
-        return tempGlyphCount;
-    };
+    virtual le_int32 glyphPostProcessing(LEGlyphStorage &tempGlyphStorage, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method applies the characterProcessing, glyphProcessing and glyphPostProcessing
@@ -272,7 +325,7 @@ protected:
      * @param offset - the index of the first character to process
      * @param count - the number of characters to process
      * @param max - the number of characters in the input context
-     * @param rightToLeft - true if the text is in a right to left directional run
+     * @param rightToLeft - TRUE if the text is in a right to left directional run
      *
      * Output parameters:
      * @param glyphs - the glyph index array
@@ -282,8 +335,10 @@ protected:
      * @return the number of glyphs in the glyph index array
      *
      * @see LayoutEngine::computeGlyphs
+     *
+     * @internal
      */
-    virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
+    virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method uses the GPOS table, if there is one, to adjust the glyph positions.
@@ -297,13 +352,17 @@ protected:
      * Output parameters:
      * @param positions - the output X and Y positions (two entries per glyph)
      * @param success - set to an error code if the operation fails
+     *
+     * @internal
      */
-    virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, LEGlyphID glyphs[], le_int32 glyphCount, float positions[], LEErrorCode &success);
+    virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method frees the feature tag array so that the
      * OpenTypeLayoutEngine can be reused for different text.
      * It is also called from our destructor.
+     *
+     * @internal
      */
     virtual void reset();
 };
