@@ -1,8 +1,7 @@
 
 /*
- * @(#)ThaiLayoutEngine.cpp	1.2 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2005 - All Rights Reserved
  *
  */
 
@@ -10,13 +9,16 @@
 #include "LayoutEngine.h"
 #include "ThaiLayoutEngine.h"
 #include "ScriptAndLanguageTags.h"
+#include "LEGlyphStorage.h"
 
 #include "ThaiShaping.h"
 
 U_NAMESPACE_BEGIN
 
-ThaiLayoutEngine::ThaiLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode)
-    : LayoutEngine(fontInstance, scriptCode, languageCode)
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(ThaiLayoutEngine)
+
+ThaiLayoutEngine::ThaiLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, le_int32 typoFlags)
+    : LayoutEngine(fontInstance, scriptCode, languageCode, typoFlags)
 {
     fErrorChar = 0x25CC;
 
@@ -49,7 +51,7 @@ ThaiLayoutEngine::~ThaiLayoutEngine()
 // Output: glyphs, char indices
 // Returns: the glyph count
 // NOTE: this assumes that ThaiShaping::compose will allocate the outChars array...
-le_int32 ThaiLayoutEngine::computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success)
+le_int32 ThaiLayoutEngine::computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool /*rightToLeft*/, LEGlyphStorage &glyphStorage, LEErrorCode &success)
 {
     if (LE_FAILURE(success)) {
         return 0;
@@ -65,26 +67,27 @@ le_int32 ThaiLayoutEngine::computeGlyphs(const LEUnicode chars[], le_int32 offse
     
     // This is enough room for the worst-case expansion
     // (it says here...)
-    outChars = new LEUnicode[count * 2];
+    outChars = LE_NEW_ARRAY(LEUnicode, count * 2);
 
     if (outChars == NULL) {
         success = LE_MEMORY_ALLOCATION_ERROR;
         return 0;
     }
 
-    charIndices = new le_int32[count * 2];
+    glyphStorage.allocateGlyphArray(count * 2, FALSE, success);
 
-    if (charIndices == NULL) {
-        delete[] outChars;
+    if (LE_FAILURE(success)) {
+        LE_DELETE_ARRAY(outChars);
         success = LE_MEMORY_ALLOCATION_ERROR;
         return 0;
     }
 
-    glyphCount = ThaiShaping::compose(chars, offset, count, fGlyphSet, fErrorChar, outChars, charIndices);
-    mapCharsToGlyphs(outChars, 0, glyphCount, false, false, glyphs, charIndices, success);
+    glyphCount = ThaiShaping::compose(chars, offset, count, fGlyphSet, fErrorChar, outChars, glyphStorage);
+    mapCharsToGlyphs(outChars, 0, glyphCount, FALSE, FALSE, TRUE, glyphStorage, success);
 
-    delete[] outChars;
+    LE_DELETE_ARRAY(outChars);
 
+    glyphStorage.adoptGlyphCount(glyphCount);
     return glyphCount;
 }
 
