@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2000, International Business Machines
+*   Copyright (C) 1999-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -20,6 +20,7 @@
  * TODO: make a complete i18n layout for this program.
  ******************************************************************************/
 
+#include "unicode/putil.h"
 #include "unicode/ures.h"
 #include "unicode/ustdio.h"
 #include "unicode/uloc.h"
@@ -27,6 +28,7 @@
 #include "uoptions.h"
 #include "toolutil.h"
 
+#include <string.h>
 #include <stdlib.h>
 #ifdef WIN32
 #include <direct.h>
@@ -78,6 +80,7 @@ main(int argc, char* argv[]) {
     UFILE *out = NULL;
     int32_t i = 0;
     const char* arg;
+    char resPathBuffer[1024];
 #ifdef WIN32
     currdir = _getcwd(NULL, 0);
 #else
@@ -92,9 +95,9 @@ main(int argc, char* argv[]) {
             "error in command line argument \"%s\"\n",
             argv[-argc]);
     }
-    if(argc<0 || options[0].doesOccur || options[1].doesOccur) {
+    if(argc<2 || options[0].doesOccur || options[1].doesOccur) {
         fprintf(stderr,
-            "usage: %s [-options]\n",
+            "usage: %s [-options] locale(s)\n",
             argv[0]);
         return argc<0 ? U_ILLEGAL_ARGUMENT_ERROR : U_ZERO_ERROR;
     }
@@ -118,7 +121,10 @@ main(int argc, char* argv[]) {
             resPath = NULL; /* we'll use ICU system resources for dumping */
         }
     } else {
-        resPath = currdir; /* we'll just dump uresb samples resources */
+        strcpy(resPathBuffer, currdir);
+        /*strcat(resPathBuffer, U_FILE_SEP_STRING);
+        strcat(resPathBuffer, "uresb");*/
+        resPath = resPathBuffer; /* we'll just dump uresb samples resources */
     }
 
     if(options[5].doesOccur) {
@@ -133,23 +139,17 @@ main(int argc, char* argv[]) {
     }
 
     if(options[6].doesOccur) {
-      VERBOSE = TRUE;
+        VERBOSE = TRUE;
     }
 
     outerr = u_finit(stderr, locale, encoding);
     out = u_finit(stdout, locale, encoding); 
 
-/*
-    for(i = 0; i<20; i++) {
-        reportError(&i);
-    }
-*/
-
     for(i = 1; i < argc; ++i) {
         status = U_ZERO_ERROR;
         arg = getLongPathname(argv[i]);
 
-        printf("uresb: processing file \"%s\"\n", arg);
+        u_fprintf(out, "uresb: processing file \"%s\" in path \"%s\"\n", arg, resPath);
         bundle = ures_open(resPath, arg, &status);
         if(U_SUCCESS(status)) {
             u_fprintf(out, "%s\n", arg);
@@ -219,12 +219,11 @@ static UChar *quotedString(const UChar *string) {
 }
 
 void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErrorCode *status) {
-    int32_t noOfElements = ures_getSize(resource);
     int32_t i = 0;
     const char *key = ures_getKey(resource);
 
     switch(ures_getType(resource)) {
-    case RES_STRING :
+    case URES_STRING :
         {
             int32_t len=0;
             const UChar*thestr = ures_getString(resource, &len, status);
@@ -240,9 +239,9 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
             */
             printIndent(out, indent);
             if(key != NULL) {
-                u_fprintf(out, "%s { \"%U\" } ", key, string);
+                u_fprintf(out, "%s { \"%S\" } ", key, string);
             } else {
-                u_fprintf(out, "\"%U\",", string);
+                u_fprintf(out, "\"%S\",", string);
             }
             if(VERBOSE) {
                 u_fprintf(out, " // STRING");
@@ -251,7 +250,7 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
             free(string);
         }
         break;
-    case RES_INT :
+    case URES_INT :
         printIndent(out, indent);
         if(key != NULL) {
             u_fprintf(out, "%s", key);
@@ -263,7 +262,7 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
         }
         u_fprintf(out, "\n");
         break;
-    case RES_BINARY :
+    case URES_BINARY :
         {
             int32_t len = 0;
             const int8_t *data = (const int8_t *)ures_getBinary(resource, &len, status);
@@ -292,7 +291,7 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
             }
         }
         break;
-    case RES_INT_VECTOR :
+    case URES_INT_VECTOR :
       {
           int32_t len = 0;
           const int32_t *data = ures_getIntVector(resource, &len, status);
@@ -319,8 +318,8 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
           }
       }
       break;
-    case RES_TABLE :
-    case RES_ARRAY :
+    case URES_TABLE :
+    case URES_ARRAY :
         {
             UResourceBundle *t = NULL;
             ures_resetIterator(resource);
@@ -330,7 +329,7 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
             }
             u_fprintf(out, "{");
             if(VERBOSE) {
-                if(ures_getType(resource) == RES_TABLE) {
+                if(ures_getType(resource) == URES_TABLE) {
                     u_fprintf(out, " // TABLE");
                 } else {
                     u_fprintf(out, " // ARRAY");
@@ -355,7 +354,7 @@ void printOutBundle(UFILE *out, UResourceBundle *resource, int32_t indent, UErro
 }
 
 void reportError(UErrorCode *status) {
-    u_fprintf(outerr, "Error %d : %U happened!\n", *status, getErrorName(*status));
+    u_fprintf(outerr, "Error %d(%s) : %U happened!\n", *status, u_errorName(*status), getErrorName(*status));
 }
 
 
@@ -379,3 +378,4 @@ const UChar *getErrorName(UErrorCode errorNumber) {
     }
 
 }
+
