@@ -21,12 +21,17 @@ void addTestEuroRegression(TestNode** root);
 void addTestConverterFallBack(TestNode** root);
 void addExtraTests(TestNode** root);
 
+/* bocu1tst.c */
+U_CFUNC void
+addBOCU1Tests(TestNode** root);
+
 void addConvert(TestNode** root);
 
 void addConvert(TestNode** root)
-{    
+{
     addTestConvert(root);
     addTestNewConvert(root);
+    addBOCU1Tests(root);
     addTestConvertErrorCallBack(root);
     addTestEuroRegression(root);
     addTestConverterFallBack(root);
