@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2006, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -14,8 +14,14 @@
 */
 /* C API TEST FOR MESSAGE FORMAT */
 
-#include "unicode/uloc.h"
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
+#include <stdlib.h>
+#include <string.h>
+#include <stdarg.h>
+#include "unicode/uloc.h"
 #include "unicode/umsg.h"
 #include "unicode/udat.h"
 #include "unicode/umsg.h"
@@ -23,8 +29,6 @@
 #include "cintltst.h"
 #include "cmsgtst.h"
 #include "cformtst.h"
-#include "cstring.h"
-#include "cmemory.h"
 
 static const char* const txt_testCasePatterns[] = {
    "Quotes '', '{', a {0,number,integer} '{'0}",
@@ -38,7 +42,7 @@ static const char* const txt_testResultStrings[] = {
     "Quotes ', {, a 1 {0}",
     "Quotes ', {, a 1 {0}",
     "You deposited 1 times an amount of $3,456.00 on 1/12/70",
-    "{2,time,full}, for 3,456, 1 is 5:46:40 AM PST and full date is Monday, January 12, 1970",
+    "{2,time,full}, for 3,456, 1 is 5:46:40 AM PT and full date is Monday, January 12, 1970",
     "{1,number,percent} for 1 is 345,600%"
 };
 
@@ -57,12 +61,14 @@ static void InitStrings( void )
         return;
 
     for (i=0; i < cnt_testCases; i++ ) {
-        testCasePatterns[i]=(UChar*)malloc(sizeof(UChar) * (strlen(txt_testCasePatterns[i]) + 1));
-        u_uastrcpy(testCasePatterns[i], txt_testCasePatterns[i] );
+        uint32_t strSize = (uint32_t)strlen(txt_testCasePatterns[i]) + 1;
+        testCasePatterns[i]=(UChar*)malloc(sizeof(UChar) * strSize);
+        u_uastrncpy(testCasePatterns[i], txt_testCasePatterns[i], strSize);
     }
     for (i=0; i < cnt_testCases; i++ ) {
-        testResultStrings[i] = (UChar*)malloc(sizeof(UChar) * (strlen(txt_testResultStrings[i]) + 1));
-        u_uastrcpy(testResultStrings[i], txt_testResultStrings[i] );
+        uint32_t strSize = (uint32_t)strlen(txt_testResultStrings[i]) + 1;
+        testResultStrings[i] = (UChar*)malloc(sizeof(UChar) * strSize);
+        u_uastrncpy(testResultStrings[i], txt_testResultStrings[i], strSize);
     }
 
     strings_initialized = TRUE;
@@ -83,6 +89,16 @@ static void FreeStrings( void )
     strings_initialized = FALSE;
 }
 
+/* Platform dependent test to detect if this type will return NULL when interpreted as a pointer. */
+static UBool returnsNullForType(int firstParam, ...) {
+    UBool isNULL;
+    va_list marker;
+    va_start(marker, firstParam);
+    isNULL = (UBool)(va_arg(marker, void*) == NULL);
+    va_end(marker);
+    return isNULL;
+}
+
 /* Test u_formatMessage() with various test patterns() */
 static void MessageFormatTest( void ) 
 {
@@ -91,11 +107,14 @@ static void MessageFormatTest( void )
     int32_t resultLengthOut,resultlength,i, patternlength;
     UErrorCode status = U_ZERO_ERROR;
     UDate d1=1000000000.0;
+
+    ctest_setTimeZone(NULL, &status);
+
     str=(UChar*)malloc(sizeof(UChar) * 7);
-    u_uastrcpy(str, "MyDisk");
+    u_uastrncpy(str, "MyDisk", 7);
     resultlength=1;
     result=(UChar*)malloc(sizeof(UChar) * 1);
-    log_verbose("Testing u_formatMessage90\n");
+    log_verbose("Testing u_formatMessage()\n");
     InitStrings();
     for (i = 0; i < cnt_testCases; i++) {
         status=U_ZERO_ERROR;
@@ -122,6 +141,7 @@ static void MessageFormatTest( void )
         }
     }
     free(result);
+    result = NULL;
     free(str);
     {
 
@@ -136,7 +156,7 @@ static void MessageFormatTest( void )
             {
                 status=U_ZERO_ERROR;
                 resultlength=resultLengthOut+1;
-                result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+                result=(UChar*)malloc(sizeof(UChar) * resultlength);
                 u_formatMessage( "en_US",testCasePatterns[i], patternlength, result, resultlength, 
                     &status, 1, 3456.00, d1);
             }
@@ -151,7 +171,7 @@ static void MessageFormatTest( void )
                 log_err("FAIL: Error in MessageFormat on testcase : %d\n GOT %s EXPECTED %s\n", i, 
                     austrdup(result), austrdup(testResultStrings[i]) );
             }
-            uprv_free(result);
+            free(result);
             result=NULL;
         }
     }
@@ -184,11 +204,11 @@ static void MessageFormatTest( void )
             resultLength = umsg_format(formatter,result,resultLength,&ec,1,3456.00,d1);
             if(ec==U_BUFFER_OVERFLOW_ERROR){
                 ec=U_ZERO_ERROR;
-                result = (UChar*) uprv_malloc(U_SIZEOF_UCHAR*resultLength+2);
+                result = (UChar*) malloc(U_SIZEOF_UCHAR*resultLength+2);
                 resultLength =  umsg_format(formatter,result,resultLength+2,&ec,1,3456.00,d1);
                 if(U_FAILURE(ec)){
                       log_err("ERROR: failure in message format on testcase %d:  %s\n", i, u_errorName(status) );
-                      uprv_free(result);
+                      free(result);
                       return;
                 }
             
@@ -200,37 +220,44 @@ static void MessageFormatTest( void )
                         austrdup(result), austrdup(testResultStrings[i]) );
                 }
 
- /* ###TODO: Fix this after ICU 2.1. */
-#if !defined(HPUX)
+                if (returnsNullForType(1, (double)2.0)) {
+                    /* HP/UX and possibly other platforms don't properly check for this case.
+                    We pass in a UDate, but the function expects a UDate *.  When va_arg is used,
+                    most compilers will return NULL, but HP-UX won't do that and will return 2
+                    in this case.  This is a platform dependent test.
 
-                umsg_parse(formatter,result,resultLength,&count,&ec,one,two,d2);
-                if(ec!=U_ILLEGAL_ARGUMENT_ERROR){
-                    log_err("FAIL: Did not get expected error for umsg_parse(). Expected: U_ILLEGAL_ARGUMENT_ERROR Got: %s \n",u_errorName(ec));
-                }else{
-                    ec = U_ZERO_ERROR;
+                    This relies upon "undefined" behavior, as indicated by C99 7.15.1.1 paragraph 2
+                    */
+                    umsg_parse(formatter,result,resultLength,&count,&ec,one,two,d2);
+                    if(ec!=U_ILLEGAL_ARGUMENT_ERROR){
+                        log_err("FAIL: Did not get expected error for umsg_parse(). Expected: U_ILLEGAL_ARGUMENT_ERROR Got: %s \n",u_errorName(ec));
+                    }else{
+                        ec = U_ZERO_ERROR;
+                    }
+                }
+                else {
+                    log_verbose("Warning: Returning NULL for a mismatched va_arg type isn't supported on this platform.\n", i);
                 }
 
-#else
-#if !((U_ICU_VERSION_MAJOR_NUM == 2) && (U_ICU_VERSION_MINOR_NUM ==1))
-                log_err("FAIL: Fix umsg_parse on HP/UX\n");
-#endif
-#endif            
                 umsg_parse(formatter,result,resultLength,&count,&ec,&one,&two,&d2);
                 if(U_FAILURE(ec)){
                     log_err("umsg_parse could not parse the pattern. Error: %s.\n",u_errorName(ec));
                 }
-                uprv_free(result);
+                free(result);
             }else{
                 log_err("FAIL: Expected U_BUFFER_OVERFLOW error while preflighting got: %s for testCasePatterns[%d]",u_errorName(ec),i);
             }
         }
+        umsg_close(formatter);
     }
     FreeStrings();
+
+    ctest_resetTimeZone();
 }
 
 
 /*test u_formatMessage() with sample patterns */
-static void TestSampleMessageFormat()
+static void TestSampleMessageFormat(void)
 {
     UChar *str;
     UChar *result;
@@ -238,6 +265,9 @@ static void TestSampleMessageFormat()
     int32_t resultLengthOut, resultlength;
     UDate d = 837039928046.0;
     UErrorCode status = U_ZERO_ERROR;
+
+    ctest_setTimeZone(NULL, &status);
+
     str=(UChar*)malloc(sizeof(UChar) * 15);
     u_uastrcpy(str, "abc");    
     
@@ -322,6 +352,8 @@ static void TestSampleMessageFormat()
     
     free(result);
     free(str);
+
+    ctest_resetTimeZone();
 }
 
 /* Test umsg_format() and umsg_parse() , format and parse sequence and round trip */
@@ -336,11 +368,14 @@ static void TestNewFormatAndParseAPI(void)
     UDate d1,d;
     UDateFormat *def1;
     UErrorCode status = U_ZERO_ERROR;
-    double value;
+    int32_t value = 0;
     UChar ret[30];
     UParseError parseError;
     UMessageFormat* fmt = NULL;
     int32_t count=0;
+
+    ctest_setTimeZone(NULL, &status);
+
     log_verbose("Testing format and parse with parse error\n");
 
     str=(UChar*)malloc(sizeof(UChar) * 25);
@@ -397,7 +432,7 @@ static void TestNewFormatAndParseAPI(void)
     if(U_FAILURE(status)){
         log_err("ERROR: error in parsing: test#5: %s\n", myErrorName(status));
     }
-    if(value!=7.00 && u_strcmp(str,ret)!=0)
+    if(value!=7 && u_strcmp(str,ret)!=0)
         log_err("FAIL: Error in parseMessage on test#5 \n");
     else
         log_verbose("PASS: parseMessage successful on test#5\n");
@@ -415,13 +450,15 @@ static void TestNewFormatAndParseAPI(void)
                 austrdup(myDateFormat(def1,d)), austrdup(myDateFormat(def1,d1)) );
         }
     }
+    umsg_close(fmt);
     udat_close(def1);
     ucal_close(cal);
 
     free(result);
     free(str);
     free(tzID);
-    
+
+    ctest_resetTimeZone();
 }
 
 /* Test u_formatMessageWithError() and u_parseMessageWithError() , format and parse sequence and round trip */
@@ -430,16 +467,19 @@ static void TestSampleFormatAndParseWithError(void)
 
     UChar *result, *tzID, *str;
     UChar pattern[100];
+
     UChar expected[100];
     int32_t resultLengthOut, resultlength;
     UCalendar *cal;
     UDate d1,d;
     UDateFormat *def1;
     UErrorCode status = U_ZERO_ERROR;
-    double value;
+    int32_t value = 0;
     UChar ret[30];
     UParseError parseError;
 
+    ctest_setTimeZone(NULL, &status);
+
     log_verbose("Testing format and parse with parse error\n");
 
     str=(UChar*)malloc(sizeof(UChar) * 25);
@@ -488,7 +528,7 @@ static void TestSampleFormatAndParseWithError(void)
     if(U_FAILURE(status)){
         log_err("ERROR: error in parsing: test#5: %s\n", myErrorName(status));
     }
-    if(value!=7.00 && u_strcmp(str,ret)!=0)
+    if(value!=7 && u_strcmp(str,ret)!=0)
         log_err("FAIL: Error in parseMessage on test#5 \n");
     else
         log_verbose("PASS: parseMessage successful on test#5\n");
@@ -513,10 +553,11 @@ static void TestSampleFormatAndParseWithError(void)
     free(str);
     free(tzID);
     
+    ctest_resetTimeZone();
 }
 
 /* Test u_formatMessage() and u_parseMessage() , format and parse sequence and round trip */
-static void TestSampleFormatAndParse()
+static void TestSampleFormatAndParse(void)
 {
 
     UChar *result, *tzID, *str;
@@ -527,8 +568,11 @@ static void TestSampleFormatAndParse()
     UDate d1,d;
     UDateFormat *def1;
     UErrorCode status = U_ZERO_ERROR;
-    double value;
+    int32_t value = 0;
     UChar ret[30];
+
+    ctest_setTimeZone(NULL, &status);
+
     log_verbose("Testing format and parse\n");
 
     str=(UChar*)malloc(sizeof(UChar) * 25);
@@ -577,7 +621,7 @@ static void TestSampleFormatAndParse()
     if(U_FAILURE(status)){
         log_err("ERROR: error in parsing: test#5: %s\n", myErrorName(status));
     }
-    if(value!=7.00 && u_strcmp(str,ret)!=0)
+    if(value!=7 && u_strcmp(str,ret)!=0)
         log_err("FAIL: Error in parseMessage on test#5 \n");
     else
         log_verbose("PASS: parseMessage successful on test#5\n");
@@ -602,10 +646,11 @@ static void TestSampleFormatAndParse()
     free(str);
     free(tzID);
     
+    ctest_resetTimeZone();
 }
 
 /* test message format with a choice option */
-static void TestMsgFormatChoice()
+static void TestMsgFormatChoice(void)
 {
     UChar* str;
     UErrorCode status = U_ZERO_ERROR;
@@ -692,7 +737,7 @@ static void TestMsgFormatChoice()
 }
 
 /*test u_parseMessage() with various test patterns */
-static void TestParseMessage()
+static void TestParseMessage(void)
 {
     UChar pattern[100];
     UChar source[100];
@@ -757,6 +802,9 @@ static void TestMessageFormatWithValist( void )
     int32_t resultLengthOut,resultlength,i, patternlength;
     UErrorCode status = U_ZERO_ERROR;
     UDate d1=1000000000.0;
+
+    ctest_setTimeZone(NULL, &status);
+
     str=(UChar*)malloc(sizeof(UChar) * 7);
     u_uastrcpy(str, "MyDisk");
     resultlength=1;
@@ -790,6 +838,8 @@ static void TestMessageFormatWithValist( void )
     free(result);
     free(str);
     FreeStrings();
+
+    ctest_resetTimeZone();
 }
 
 static void CallParseMessage(const char* locale, UChar* pattern, int32_t patternLength, 
@@ -856,6 +906,8 @@ static void TestJ904(void) {
     const char* PAT = "Number {1,number,#0.000}, String {0}, Date {2,date,12:mm:ss.SSS}";
     const char* EXP = "Number 0,143, String foo, Date 12:34:56.789";
 
+    ctest_setTimeZone(NULL, &status);
+
     u_uastrcpy(string, "foo");
     /* Slight hack here -- instead of date pattern HH:mm:ss.SSS, use
      * 12:mm:ss.SSS.  Why?  So this test generates the same output --
@@ -867,18 +919,20 @@ static void TestJ904(void) {
                              string, 1/7.0,
                              789.0+1000*(56+60*(34+60*12)));
 
-    u_austrcpy(cresult, result);
+    u_austrncpy(cresult, result, sizeof(cresult));
 
     /* This test passes if it DOESN'T CRASH.  However, we test the
      * output anyway.  If the string doesn't match in the date part,
      * check to see that the machine doesn't have an unusual time zone
      * offset, that is, one with a non-zero minutes/seconds offset
      * from GMT -- see above. */
-    if (uprv_strcmp(cresult, EXP) == 0) {
+    if (strcmp(cresult, EXP) == 0) {
         log_verbose("Ok: \"%s\"\n", cresult);
     } else {
         log_err("FAIL: got \"%s\", expected \"%s\"\n", cresult, EXP);
     }
+
+    ctest_resetTimeZone();
 }
 
 static void OpenMessageFormatTest(void)
@@ -894,7 +948,7 @@ static void OpenMessageFormatTest(void)
     int32_t length=0;
     UErrorCode status = U_ZERO_ERROR;
 
-    u_uastrcpy(pattern, PAT);
+    u_uastrncpy(pattern, PAT, sizeof(pattern)/sizeof(pattern[0]));
 
     /* Test umsg_open                   */
     f1 = umsg_open(pattern,length,NULL,NULL,&status);
@@ -934,7 +988,7 @@ static void OpenMessageFormatTest(void)
 
     /* Test umsg_applyPattern           */
     status = U_ZERO_ERROR;
-    umsg_applyPattern(f1,pattern,strlen(PAT),NULL,&status);
+    umsg_applyPattern(f1,pattern,(int32_t)strlen(PAT),NULL,&status);
     if(U_FAILURE(status))
     {
         log_err("umsg_applyPattern failed. Error %s \n",u_errorName(status));
@@ -944,10 +998,11 @@ static void OpenMessageFormatTest(void)
     umsg_toPattern(f1,result,256,&status);
     if(U_FAILURE(status) ){
         log_err("umsg_toPattern method failed. Error: %s \n",u_errorName(status));
-    }
-    if(u_strcmp(result,pattern)!=0){
-        u_UCharsToChars(result,cresult,256);
-        log_err("umsg_toPattern method failed. Expected: %s Got: %s \n",PAT,cresult);
+    } else {
+        if(u_strcmp(result,pattern)!=0){
+            u_UCharsToChars(result,cresult,256);
+            log_err("umsg_toPattern method failed. Expected: %s Got: %s \n",PAT,cresult);
+        }
     }
     /* umsg_format umsg_parse */
 
@@ -956,6 +1011,41 @@ static void OpenMessageFormatTest(void)
     umsg_close(f3);
 }
 
+static void MessageLength(void)
+{
+    UErrorCode status = U_ZERO_ERROR;
+    const char patChars[] = {"123{0}456{0}"};
+    const char expectedChars[] = {"123abc"};
+    UChar pattern[sizeof(patChars)];
+    UChar arg[] = {0x61,0x62,0x63,0};
+    UChar result[128] = {0};
+    UChar expected[sizeof(expectedChars)];
+
+    u_uastrncpy(pattern, patChars, sizeof(pattern)/sizeof(pattern[0]));
+    u_uastrncpy(expected, expectedChars, sizeof(expected)/sizeof(expected[0]));
+
+    u_formatMessage("en_US", pattern, 6, result, sizeof(result)/sizeof(result[0]), &status, arg);
+    if (U_FAILURE(status)) {
+        log_err("u_formatMessage method failed. Error: %s \n",u_errorName(status));
+    }
+    if (u_strcmp(result, expected) != 0) {
+        log_err("u_formatMessage didn't return expected result\n");
+    }
+}
+
+static void TestErrorChaining(void) {
+    UErrorCode status = U_USELESS_COLLATOR_ERROR;
+
+    umsg_open(NULL, 0, NULL, NULL, &status);
+    umsg_applyPattern(NULL, NULL, 0, NULL, &status);
+    umsg_clone(NULL, &status);
+    umsg_close(NULL);
+
+    /* All of this code should have done nothing. */
+    if (status != U_USELESS_COLLATOR_ERROR) {
+        log_err("Status got changed to %s\n", u_errorName(status));
+    }
+}
 
 void addMsgForTest(TestNode** root);
 
@@ -972,5 +1062,8 @@ void addMsgForTest(TestNode** root)
     addTest(root, &TestMessageFormatWithValist, "tsformat/cmsgtst/TestMessageFormatWithValist");
     addTest(root, &TestParseMessageWithValist, "tsformat/cmsgtst/TestParseMessageWithValist");
     addTest(root, &TestJ904, "tsformat/cmsgtst/TestJ904");
-
+    addTest(root, &MessageLength, "tsformat/cmsgtst/MessageLength");
+    addTest(root, &TestErrorChaining, "tsformat/cmsgtst/TestErrorChaining");
 }
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
