@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2004, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -14,19 +14,27 @@
 *     synwee                      added test for checkFCD
 *********************************************************************************/
 /*tests for u_normalization*/
-#include <stdlib.h>
 #include "unicode/utypes.h"
-#include "unicode/ucol.h"
-#include "unicode/uloc.h"
+#include "unicode/unorm.h"
+#include "unormimp.h"
 #include "cintltst.h"
-#include "cnormtst.h"
-#include "ccolltst.h"
-#include "unicode/ustring.h"
+
+#if UCONFIG_NO_NORMALIZATION
+
+void addNormTest(TestNode** root) {
+    /* no normalization - nothing to do */
+}
+
+#else
+
+#include <stdlib.h>
 #include <time.h>
+#include "unicode/uchar.h"
+#include "unicode/ustring.h"
+#include "unicode/unorm.h"
+#include "cnormtst.h"
 
-#define ARRAY_LENGTH(array) (sizeof (array) / sizeof (*array))
-
-static UCollator *myCollation;
+#define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof ((array)[0]))
 
 static void
 TestAPI(void);
@@ -40,6 +48,17 @@ TestConcatenate(void);
 static void
 TestNextPrevious(void);
 
+static void TestIsNormalized(void);
+
+static void
+TestFCNFKCClosure(void);
+
+static void
+TestQuickCheckPerCP(void);
+
+static void
+TestComposition(void);
+
 const static char* canonTests[][3] = {
     /* Input*/                    /*Decomposed*/                /*Composed*/
     { "cat",                    "cat",                        "cat"                    },
@@ -99,6 +118,8 @@ const static char* compatTests[][3] = {
     
 };
 
+void addNormTest(TestNode** root);
+
 void addNormTest(TestNode** root)
 {
     addTest(root, &TestAPI, "tscoll/cnormtst/TestAPI");
@@ -108,10 +129,14 @@ void addNormTest(TestNode** root)
     addTest(root, &TestCompatDecompCompose, "tscoll/cnormtst/CompatDecompCompose");
     addTest(root, &TestNull, "tscoll/cnormtst/TestNull");
     addTest(root, &TestQuickCheck, "tscoll/cnormtst/TestQuickCheck");
+    addTest(root, &TestQuickCheckPerCP, "tscoll/cnormtst/TestQuickCheckPerCP");
+    addTest(root, &TestIsNormalized, "tscoll/cnormtst/TestIsNormalized");
     addTest(root, &TestCheckFCD, "tscoll/cnormtst/TestCheckFCD");
     addTest(root, &TestNormCoverage, "tscoll/cnormtst/TestNormCoverage");
     addTest(root, &TestConcatenate, "tscoll/cnormtst/TestConcatenate");
     addTest(root, &TestNextPrevious, "tscoll/cnormtst/TestNextPrevious");
+    addTest(root, &TestFCNFKCClosure, "tscoll/cnormtst/TestFCNFKCClosure");
+    addTest(root, &TestComposition, "tscoll/cnormtst/TestComposition");
 }
 
 void TestDecomp() 
@@ -120,32 +145,27 @@ void TestDecomp()
     int32_t x, neededLen, resLen;
     UChar *source=NULL, *result=NULL; 
     status = U_ZERO_ERROR;
-    myCollation = ucol_open("en_US", &status);
-    if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
-        return;
-    }
     resLen=0;
     log_verbose("Testing unorm_normalize with  Decomp canonical\n");
-    for(x=0; x < ARRAY_LENGTH(canonTests); x++)
+    for(x=0; x < LENGTHOF(canonTests); x++)
     {
         source=CharsToUChars(canonTests[x][0]);
-        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFD, UCOL_IGNORE_HANGUL, NULL, 0, &status); 
+        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFD, 0, NULL, 0, &status); 
         if(status==U_BUFFER_OVERFLOW_ERROR)
         {
             status=U_ZERO_ERROR;
             resLen=neededLen+1;
             result=(UChar*)malloc(sizeof(UChar*) * resLen);
-            unorm_normalize(source, u_strlen(source), UNORM_NFD, UCOL_IGNORE_HANGUL, result, resLen, &status); 
+            unorm_normalize(source, u_strlen(source), UNORM_NFD, 0, result, resLen, &status); 
         }
         if(U_FAILURE(status)){
             log_err("ERROR in unorm_normalize at %s:  %s\n", austrdup(source), myErrorName(status) );
+        } else {
+          assertEqual(result, canonTests[x][1], x);
         }
-        assertEqual(result, canonTests[x][1], x);
         free(result);
         free(source);
     }
-    ucol_close(myCollation);
 }
 
 void TestCompatDecomp() 
@@ -154,32 +174,27 @@ void TestCompatDecomp()
     int32_t x, neededLen, resLen;
     UChar *source=NULL, *result=NULL; 
     status = U_ZERO_ERROR;
-    myCollation = ucol_open("en_US", &status);
-    if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
-        return;
-    }
     resLen=0;
     log_verbose("Testing unorm_normalize with  Decomp compat\n");
-    for(x=0; x < ARRAY_LENGTH(compatTests); x++)
+    for(x=0; x < LENGTHOF(compatTests); x++)
     {
         source=CharsToUChars(compatTests[x][0]);
-        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFKD, UCOL_IGNORE_HANGUL, NULL, 0, &status); 
+        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFKD, 0, NULL, 0, &status); 
         if(status==U_BUFFER_OVERFLOW_ERROR)
         {
             status=U_ZERO_ERROR;
             resLen=neededLen+1;
             result=(UChar*)malloc(sizeof(UChar*) * resLen);
-            unorm_normalize(source, u_strlen(source), UNORM_NFKD,UCOL_IGNORE_HANGUL, result, resLen, &status); 
+            unorm_normalize(source, u_strlen(source), UNORM_NFKD, 0, result, resLen, &status); 
         }
         if(U_FAILURE(status)){
             log_err("ERROR in unorm_normalize at %s:  %s\n", austrdup(source), myErrorName(status) );
+        } else {
+          assertEqual(result, compatTests[x][1], x);
         }
-        assertEqual(result, compatTests[x][1], x);
         free(result);
         free(source);
     }
-    ucol_close(myCollation);            
 }
 
 void TestCanonDecompCompose() 
@@ -188,32 +203,27 @@ void TestCanonDecompCompose()
     int32_t x, neededLen, resLen;
     UChar *source=NULL, *result=NULL; 
     status = U_ZERO_ERROR;
-    myCollation = ucol_open("en_US", &status);
-    if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
-        return;
-    }
     resLen=0;
     log_verbose("Testing unorm_normalize with Decomp can compose compat\n");
-    for(x=0; x < ARRAY_LENGTH(canonTests); x++)
+    for(x=0; x < LENGTHOF(canonTests); x++)
     {
         source=CharsToUChars(canonTests[x][0]);
-        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFC, UCOL_IGNORE_HANGUL, NULL, 0, &status); 
+        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFC, 0, NULL, 0, &status); 
         if(status==U_BUFFER_OVERFLOW_ERROR)
         {
             status=U_ZERO_ERROR;
             resLen=neededLen+1;
             result=(UChar*)malloc(sizeof(UChar*) * resLen);
-            unorm_normalize(source, u_strlen(source), UNORM_NFC, UCOL_IGNORE_HANGUL, result, resLen, &status); 
+            unorm_normalize(source, u_strlen(source), UNORM_NFC, 0, result, resLen, &status); 
         }
         if(U_FAILURE(status)){
             log_err("ERROR in unorm_normalize at %s:  %s\n", austrdup(source),myErrorName(status) );
+        } else {
+          assertEqual(result, canonTests[x][2], x);
         }
-        assertEqual(result, canonTests[x][2], x);
         free(result);
         free(source);
     }
-    ucol_close(myCollation);            
 }
 
 void TestCompatDecompCompose() 
@@ -222,32 +232,27 @@ void TestCompatDecompCompose()
     int32_t x, neededLen, resLen;
     UChar *source=NULL, *result=NULL;
     status = U_ZERO_ERROR;
-    myCollation = ucol_open("en_US", &status);
-    if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
-        return;
-    }
     resLen=0;
     log_verbose("Testing unorm_normalize with compat decomp compose can\n");
-    for(x=0; x < ARRAY_LENGTH(compatTests); x++)
+    for(x=0; x < LENGTHOF(compatTests); x++)
     {
         source=CharsToUChars(compatTests[x][0]);
-        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFKC, UCOL_IGNORE_HANGUL, NULL, 0, &status); 
+        neededLen= unorm_normalize(source, u_strlen(source), UNORM_NFKC, 0, NULL, 0, &status); 
         if(status==U_BUFFER_OVERFLOW_ERROR)
         {
             status=U_ZERO_ERROR;
             resLen=neededLen+1;
             result=(UChar*)malloc(sizeof(UChar*) * resLen);
-            unorm_normalize(source, u_strlen(source), UNORM_NFKC, UCOL_IGNORE_HANGUL, result, resLen, &status); 
+            unorm_normalize(source, u_strlen(source), UNORM_NFKC, 0, result, resLen, &status); 
         }
         if(U_FAILURE(status)){
             log_err("ERROR in unorm_normalize at %s:  %s\n", austrdup(source), myErrorName(status) );
+        } else {
+          assertEqual(result, compatTests[x][2], x);
         }
-        assertEqual(result, compatTests[x][2], x);
         free(result);
         free(source);
     }
-    ucol_close(myCollation);            
 }
 
 
@@ -508,7 +513,7 @@ static void TestQuickCheckStringResult()
   UChar *c = NULL;
   UErrorCode error = U_ZERO_ERROR;
 
-  for (count = 0; count < ARRAY_LENGTH(canonTests); count ++)
+  for (count = 0; count < LENGTHOF(canonTests); count ++)
   {
     d = CharsToUChars(canonTests[count][1]);
     c = CharsToUChars(canonTests[count][2]);
@@ -530,7 +535,7 @@ static void TestQuickCheckStringResult()
     free(c);
   }
 
-  for (count = 0; count < ARRAY_LENGTH(compatTests); count ++)
+  for (count = 0; count < LENGTHOF(compatTests); count ++)
   {
     d = CharsToUChars(compatTests[count][1]);
     c = CharsToUChars(compatTests[count][2]);
@@ -561,6 +566,75 @@ void TestQuickCheck()
   TestQuickCheckStringResult(); 
 }
 
+/*
+ * The intltest/NormalizerConformanceTest tests a lot of strings that _are_
+ * normalized, and some that are not.
+ * Here we pick some specific cases and test the C API.
+ */
+static void TestIsNormalized(void) {
+    static const UChar notNFC[][8]={            /* strings that are not in NFC */
+        { 0x62, 0x61, 0x300, 0x63, 0 },         /* 0061 0300 compose */
+        { 0xfb1d, 0 },                          /* excluded from composition */
+        { 0x0627, 0x0653, 0 },                  /* 0627 0653 compose */
+        { 0x3071, 0x306f, 0x309a, 0x3073, 0 }   /* 306F 309A compose */
+    };
+    static const UChar notNFKC[][8]={           /* strings that are not in NFKC */
+        { 0x1100, 0x1161, 0 },                  /* Jamo compose */
+        { 0x1100, 0x314f, 0 },                  /* compatibility Jamo compose */
+        { 0x03b1, 0x1f00, 0x0345, 0x03b3, 0 }   /* 1F00 0345 compose */
+    };
+
+    int32_t i;
+    UErrorCode errorCode;
+
+    /* API test */
+
+    /* normal case with length>=0 (length -1 used for special cases below) */
+    errorCode=U_ZERO_ERROR;
+    if(!unorm_isNormalized(notNFC[0]+2, 1, UNORM_NFC, &errorCode) || U_FAILURE(errorCode)) {
+        log_err("error: !isNormalized(<U+0300>, NFC) (%s)\n", u_errorName(errorCode));
+    }
+
+    /* incoming U_FAILURE */
+    errorCode=U_TRUNCATED_CHAR_FOUND;
+    (void)unorm_isNormalized(notNFC[0]+2, 1, UNORM_NFC, &errorCode);
+    if(errorCode!=U_TRUNCATED_CHAR_FOUND) {
+        log_err("error: isNormalized(U_TRUNCATED_CHAR_FOUND) changed the error code to %s\n", u_errorName(errorCode));
+    }
+
+    /* NULL source */
+    errorCode=U_ZERO_ERROR;
+    (void)unorm_isNormalized(NULL, 1, UNORM_NFC, &errorCode);
+    if(errorCode!=U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("error: isNormalized(NULL) did not set U_ILLEGAL_ARGUMENT_ERROR but %s\n", u_errorName(errorCode));
+    }
+
+    /* bad length */
+    errorCode=U_ZERO_ERROR;
+    (void)unorm_isNormalized(notNFC[0]+2, -2, UNORM_NFC, &errorCode);
+    if(errorCode!=U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("error: isNormalized([-2]) did not set U_ILLEGAL_ARGUMENT_ERROR but %s\n", u_errorName(errorCode));
+    }
+
+    /* specific cases */
+    for(i=0; i<LENGTHOF(notNFC); ++i) {
+        errorCode=U_ZERO_ERROR;
+        if(unorm_isNormalized(notNFC[i], -1, UNORM_NFC, &errorCode) || U_FAILURE(errorCode)) {
+            log_err("error: isNormalized(notNFC[%d], NFC) is wrong (%s)\n", i, u_errorName(errorCode));
+        }
+        errorCode=U_ZERO_ERROR;
+        if(unorm_isNormalized(notNFC[i], -1, UNORM_NFKC, &errorCode) || U_FAILURE(errorCode)) {
+            log_err("error: isNormalized(notNFC[%d], NFKC) is wrong (%s)\n", i, u_errorName(errorCode));
+        }
+    }
+    for(i=0; i<LENGTHOF(notNFKC); ++i) {
+        errorCode=U_ZERO_ERROR;
+        if(unorm_isNormalized(notNFKC[i], -1, UNORM_NFKC, &errorCode) || U_FAILURE(errorCode)) {
+            log_err("error: isNormalized(notNFKC[%d], NFKC) is wrong (%s)\n", i, u_errorName(errorCode));
+        }
+    }
+}
+
 void TestCheckFCD() 
 {
   UErrorCode status = U_ZERO_ERROR;
@@ -634,7 +708,7 @@ void TestCheckFCD()
     while (size != 19) {
       data[size] = datachar[(rand() * 50) / RAND_MAX];
       log_verbose("0x%x", data[size]);
-      normsize += unorm_normalize(data + size, 1, UNORM_NFD, UCOL_IGNORE_HANGUL, 
+      normsize += unorm_normalize(data + size, 1, UNORM_NFD, 0, 
                                   norm + normsize, 100 - normsize, &status);       
       if (U_FAILURE(status)) {
         log_err("unorm_quickCheck(FCD) failed: exception occured at data generation\n");
@@ -644,7 +718,7 @@ void TestCheckFCD()
     }
     log_verbose("\n");
 
-    nfdsize = unorm_normalize(data, size, UNORM_NFD, UCOL_IGNORE_HANGUL, 
+    nfdsize = unorm_normalize(data, size, UNORM_NFD, 0, 
                               nfd, 100, &status);       
     if (U_FAILURE(status)) {
       log_err("unorm_quickCheck(FCD) failed: exception occured at normalized data generation\n");
@@ -1272,3 +1346,171 @@ TestNextPrevious() {
         return;
     }
 }
+
+static void
+TestFCNFKCClosure(void) {
+    static const struct {
+        UChar32 c;
+        const UChar s[6];
+    } tests[]={
+        { 0x037A, { 0x0020, 0x03B9, 0 } },
+        { 0x03D2, { 0x03C5, 0 } },
+        { 0x20A8, { 0x0072, 0x0073, 0 } },
+        { 0x210B, { 0x0068, 0 } },
+        { 0x210C, { 0x0068, 0 } },
+        { 0x2121, { 0x0074, 0x0065, 0x006C, 0 } },
+        { 0x2122, { 0x0074, 0x006D, 0 } },
+        { 0x2128, { 0x007A, 0 } },
+        { 0x1D5DB, { 0x0068, 0 } },
+        { 0x1D5ED, { 0x007A, 0 } },
+        { 0x0061, { 0 } }
+    };
+
+    UChar buffer[8];
+    UErrorCode errorCode;
+    int32_t i, length;
+
+    for(i=0; i<LENGTHOF(tests); ++i) {
+        errorCode=U_ZERO_ERROR;
+        length=u_getFC_NFKC_Closure(tests[i].c, buffer, LENGTHOF(buffer), &errorCode);
+        if(U_FAILURE(errorCode) || length!=u_strlen(buffer) || 0!=u_strcmp(tests[i].s, buffer)) {
+            log_err("u_getFC_NFKC_Closure(U+%04lx) is wrong (%s)\n", tests[i].c, u_errorName(errorCode));
+        }
+    }
+
+    /* error handling */
+    errorCode=U_ZERO_ERROR;
+    length=u_getFC_NFKC_Closure(0x5c, NULL, LENGTHOF(buffer), &errorCode);
+    if(errorCode!=U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("u_getFC_NFKC_Closure(dest=NULL) is wrong (%s)\n", u_errorName(errorCode));
+    }
+
+    length=u_getFC_NFKC_Closure(0x5c, buffer, LENGTHOF(buffer), &errorCode);
+    if(errorCode!=U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("u_getFC_NFKC_Closure(U_FAILURE) is wrong (%s)\n", u_errorName(errorCode));
+    }
+}
+
+static void
+TestQuickCheckPerCP() {
+    UErrorCode errorCode;
+    UChar32 c, lead, trail;
+    UChar s[U16_MAX_LENGTH], nfd[16];
+    int32_t length, lccc1, lccc2, tccc1, tccc2;
+    UNormalizationCheckResult qc1, qc2;
+
+    if(
+        u_getIntPropertyMaxValue(UCHAR_NFD_QUICK_CHECK)!=(int32_t)UNORM_YES ||
+        u_getIntPropertyMaxValue(UCHAR_NFKD_QUICK_CHECK)!=(int32_t)UNORM_YES ||
+        u_getIntPropertyMaxValue(UCHAR_NFC_QUICK_CHECK)!=(int32_t)UNORM_MAYBE ||
+        u_getIntPropertyMaxValue(UCHAR_NFKC_QUICK_CHECK)!=(int32_t)UNORM_MAYBE ||
+        u_getIntPropertyMaxValue(UCHAR_LEAD_CANONICAL_COMBINING_CLASS)!=u_getIntPropertyMaxValue(UCHAR_CANONICAL_COMBINING_CLASS) ||
+        u_getIntPropertyMaxValue(UCHAR_TRAIL_CANONICAL_COMBINING_CLASS)!=u_getIntPropertyMaxValue(UCHAR_CANONICAL_COMBINING_CLASS)
+    ) {
+        log_err("wrong result from one of the u_getIntPropertyMaxValue(UCHAR_NF*_QUICK_CHECK) or UCHAR_*_CANONICAL_COMBINING_CLASS\n");
+    }
+
+    /*
+     * compare the quick check property values for some code points
+     * to the quick check results for checking same-code point strings
+     */
+    errorCode=U_ZERO_ERROR;
+    c=0;
+    while(c<0x110000) {
+        length=0;
+        U16_APPEND_UNSAFE(s, length, c);
+
+        qc1=u_getIntPropertyValue(c, UCHAR_NFC_QUICK_CHECK);
+        qc2=unorm_quickCheck(s, length, UNORM_NFC, &errorCode);
+        if(qc1!=qc2) {
+            log_err("u_getIntPropertyValue(NFC)=%d != %d=unorm_quickCheck(NFC) for U+%04x\n", qc1, qc2, c);
+        }
+
+        qc1=u_getIntPropertyValue(c, UCHAR_NFD_QUICK_CHECK);
+        qc2=unorm_quickCheck(s, length, UNORM_NFD, &errorCode);
+        if(qc1!=qc2) {
+            log_err("u_getIntPropertyValue(NFD)=%d != %d=unorm_quickCheck(NFD) for U+%04x\n", qc1, qc2, c);
+        }
+
+        qc1=u_getIntPropertyValue(c, UCHAR_NFKC_QUICK_CHECK);
+        qc2=unorm_quickCheck(s, length, UNORM_NFKC, &errorCode);
+        if(qc1!=qc2) {
+            log_err("u_getIntPropertyValue(NFKC)=%d != %d=unorm_quickCheck(NFKC) for U+%04x\n", qc1, qc2, c);
+        }
+
+        qc1=u_getIntPropertyValue(c, UCHAR_NFKD_QUICK_CHECK);
+        qc2=unorm_quickCheck(s, length, UNORM_NFKD, &errorCode);
+        if(qc1!=qc2) {
+            log_err("u_getIntPropertyValue(NFKD)=%d != %d=unorm_quickCheck(NFKD) for U+%04x\n", qc1, qc2, c);
+        }
+
+        length=unorm_normalize(s, length, UNORM_NFD, 0, nfd, LENGTHOF(nfd), &errorCode);
+        U16_GET(nfd, 0, 0, length, lead);
+        U16_GET(nfd, 0, length-1, length, trail);
+
+        lccc1=u_getIntPropertyValue(c, UCHAR_LEAD_CANONICAL_COMBINING_CLASS);
+        lccc2=u_getCombiningClass(lead);
+        tccc1=u_getIntPropertyValue(c, UCHAR_TRAIL_CANONICAL_COMBINING_CLASS);
+        tccc2=u_getCombiningClass(trail);
+
+        if(lccc1!=lccc2) {
+            log_err("u_getIntPropertyValue(lccc)=%d != %d=u_getCombiningClass(lead) for U+%04x\n",
+                    lccc1, lccc2, c);
+        }
+        if(tccc1!=tccc2) {
+            log_err("u_getIntPropertyValue(tccc)=%d != %d=u_getCombiningClass(trail) for U+%04x\n",
+                    tccc1, tccc2, c);
+        }
+
+        /* skip some code points */
+        c=(20*c)/19+1;
+    }
+}
+
+static void
+TestComposition(void) {
+    static const struct {
+        UNormalizationMode mode;
+        uint32_t options;
+        UChar input[12];
+        UChar expect[12];
+    } cases[]={
+        /*
+         * special cases for UAX #15 bug
+         * see Unicode Public Review Issue #29
+         * at http://www.unicode.org/review/resolved-pri.html#pri29
+         */
+        { UNORM_NFC, 0, { 0x1100, 0x0300, 0x1161, 0x0327 },         { 0x1100, 0x0300, 0x1161, 0x0327 } },
+        { UNORM_NFC, 0, { 0x1100, 0x0300, 0x1161, 0x0327, 0x11a8 }, { 0x1100, 0x0300, 0x1161, 0x0327, 0x11a8 } },
+        { UNORM_NFC, 0, { 0xac00, 0x0300, 0x0327, 0x11a8 },         { 0xac00, 0x0327, 0x0300, 0x11a8 } },
+        { UNORM_NFC, 0, { 0x0b47, 0x0300, 0x0b3e },                 { 0x0b47, 0x0300, 0x0b3e } },
+
+        { UNORM_NFC, UNORM_BEFORE_PRI_29, { 0x1100, 0x0300, 0x1161, 0x0327 },           { 0xac00, 0x0300, 0x0327 } },
+        { UNORM_NFC, UNORM_BEFORE_PRI_29, { 0x1100, 0x0300, 0x1161, 0x0327, 0x11a8 },   { 0xac01, 0x0300, 0x0327 } },
+        { UNORM_NFC, UNORM_BEFORE_PRI_29, { 0xac00, 0x0300, 0x0327, 0x11a8 },           { 0xac01, 0x0327, 0x0300 } },
+        { UNORM_NFC, UNORM_BEFORE_PRI_29, { 0x0b47, 0x0300, 0x0b3e },                   { 0x0b4b, 0x0300 } }
+
+        /* TODO: add test cases for UNORM_FCC here (j2151) */
+    };
+
+    UChar output[16];
+    UErrorCode errorCode;
+    int32_t i, length;
+
+    for(i=0; i<LENGTHOF(cases); ++i) {
+        errorCode=U_ZERO_ERROR;
+        length=unorm_normalize(
+                    cases[i].input, -1,
+                    cases[i].mode, cases[i].options,
+                    output, LENGTHOF(output),
+                    &errorCode);
+        if( U_FAILURE(errorCode) ||
+            length!=u_strlen(cases[i].expect) ||
+            0!=u_memcmp(output, cases[i].expect, length)
+        ) {
+            log_err("unexpected result for case %d\n", i);
+        }
+    }
+}
+
+#endif /* #if !UCONFIG_NO_NORMALIZATION */
