@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1998-2001, International Business Machines Corporation and
+ * Copyright (c) 1998-2006, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*
@@ -19,6 +19,7 @@
 #include "unicode/putil.h"
 #include "unicode/ustring.h"
 #include "cstring.h"
+#include "putilimp.h"
 
 static UBool compareWithNAN(double x, double y);
 static void doAssert(double expect, double got, const char *message);
@@ -89,13 +90,6 @@ static void TestPUtilAPI(void){
     log_verbose("Testing the API uprv_pow10()\n");
     doAssert(uprv_pow10(4), 10000, "uprv_pow10(4) failed.");
 
-    log_verbose("Testing the API uprv_log10()\n");
-    doAssert(uprv_log10(3456), 3, "uprv_log10(3456) failed.");
-#ifdef OS390
-    doAssert(uprv_log10(1.0e55), 55, "uprv_log10(1.0e55) failed.");
-#else
-    doAssert(uprv_log10(1.0e300), 300, "uprv_log10(1.0e300) failed.");
-#endif          
     log_verbose("Testing the API uprv_isNegativeInfinity()\n");
     isTrue=uprv_isNegativeInfinity(uprv_getInfinity() * -1);
     if(isTrue != TRUE){
@@ -112,6 +106,7 @@ static void TestPUtilAPI(void){
         log_err("ERROR: uprv_isInfinite failed.\n");
     }
 
+#if 0
     log_verbose("Testing the API uprv_digitsAfterDecimal()....\n");
     doAssert(uprv_digitsAfterDecimal(value1), 3, "uprv_digitsAfterDecimal() failed.");
     doAssert(uprv_digitsAfterDecimal(1.2345E2), 2, "uprv_digitsAfterDecimal(1.2345E2) failed.");
@@ -122,6 +117,7 @@ static void TestPUtilAPI(void){
     doAssert(uprv_digitsAfterDecimal(-0.021), 3, "uprv_digitsAfterDecimal(-0.021) failed.");
     doAssert(uprv_digitsAfterDecimal(23.0), 0, "uprv_digitsAfterDecimal(23.0) failed.");
     doAssert(uprv_digitsAfterDecimal(0.022223333321), 9, "uprv_digitsAfterDecimal(0.022223333321) failed.");
+#endif
 
 
     log_verbose("Testing the API u_versionToString().....\n");
@@ -205,6 +201,7 @@ static void TestPUtilAPI(void){
 
     {
         const char* dataDirectory;
+        int32_t dataDirectoryLen;
         UChar *udataDir=0;
         UChar temp[100];
         char *charvalue=0;
@@ -215,22 +212,23 @@ static void TestPUtilAPI(void){
         /*dataDirectory=u_getDataDirectory();*/
 
         dataDirectory="directory1";  /*no backslashes*/
-        udataDir=(UChar*)uprv_malloc(sizeof(UChar) * (strlen(dataDirectory) + 1));
-        u_charsToUChars(dataDirectory, udataDir, (strlen(dataDirectory)+1));
+        dataDirectoryLen=(int32_t)strlen(dataDirectory);
+        udataDir=(UChar*)malloc(sizeof(UChar) * (dataDirectoryLen + 1));
+        u_charsToUChars(dataDirectory, udataDir, (dataDirectoryLen + 1));
         u_uastrcpy(temp, dataDirectory);
        
         if(u_strcmp(temp, udataDir) != 0){
             log_err("ERROR: u_charsToUChars failed. Expected %s, Got %s\n", austrdup(temp), austrdup(udataDir));
         }
         log_verbose("Testing UChars to chars\n");
-        charvalue=(char*)uprv_malloc(sizeof(char) * (u_strlen(udataDir) + 1));
+        charvalue=(char*)malloc(sizeof(char) * (u_strlen(udataDir) + 1));
 
         u_UCharsToChars(udataDir, charvalue, (u_strlen(udataDir)+1));
         if(strcmp(charvalue, dataDirectory) != 0){
             log_err("ERROR: u_UCharsToChars failed. Expected %s, Got %s\n", charvalue, dataDirectory);
         }
-        uprv_free(charvalue);
-        uprv_free(udataDir);
+        free(charvalue);
+        free(udataDir);
     }
    
     log_verbose("Testing uprv_timezone()....\n");
@@ -243,42 +241,9 @@ static void TestPUtilAPI(void){
         if ((tzoffset % 1800 != 0)) {
             log_err("FAIL: t_timezone may be incorrect. It is not a multiple of 30min.");
         }
-        tzoffset=uprv_getUTCtime();
+        /*tzoffset=uprv_getUTCtime();*/
 
     }
-
-/* We can deprecate the test code right away; the API itself goes 2002-Jun-30 */
-#ifdef ICU_ENABLE_DEPRECATED_NEXTDOUBLE
-    log_verbose("Testing uprv_nextDouble() where the value is NaN ...\n");
-    expn1=uprv_nextDouble(uprv_getNaN(), TRUE);
-    doAssert(expn1, uprv_getNaN(), "uprv_nextDouble(uprv_getNaN(), TRUE) failed.");
-
-#if defined(OS400)
-    /* This is oddly enough the same value as DBL_MIN on Windows. */
-    /* This number is used because really small IEEE 754 numbers get an underflow exception on this platform */
-    y1=2.2250738585072014e-308;
-#elif IEEE_754
-    /* OS390 may define IEEE_754, so check it first */
-    y1=4.9406564584125e-324;
-#elif defined(OS390) || defined(XP_MAC)
-    y1=4.9406564584125e-78;
-#else
-    /* This is the default IEEE754 value. The test should fail if it's not correct. */
-    y1=4.9406564584125e-324;
-#endif
-    doAssert(uprv_nextDouble(0, TRUE),   y1, "uprv_nextDouble(0, TRUE) failed.");
-    doAssert(uprv_nextDouble(0, FALSE), -y1, "uprv_nextDouble(0, FALSE) failed.");
-       
-    expn1=uprv_nextDouble(1.0, TRUE);
-    if(expn1 < 1.0 || expn1 > 1.0001){
-        log_err("ERROR: uprv_nextDouble failed. Expected : %lf, Got: %f\n", y1, expn1);
-    }
-    expn1=uprv_nextDouble(1.0, FALSE);
-    if(expn1 > 1.0 || expn1 < 0.9999){
-        log_err("ERROR: uprv_nextDouble failed. Expected : %lf, Got: %f\n", y1, expn1);
-    }
-#endif
-
 }
 
 #if 0
