@@ -1,7 +1,6 @@
 /********************************************************************
- * Copyright (c) 2001, 
- * International Business Machines Corporation and others. 
- * All Rights Reserved.
+ * Copyright (c) 2001-2004 International Business Machines 
+ * Corporation and others. All Rights Reserved.
  ********************************************************************
  * File usrchtst.c
  * Modification History:
@@ -9,20 +8,41 @@
  * synwee         July 19 2001     creation
  ********************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/usearch.h"
 #include "unicode/ustring.h"
 #include "ccolltst.h"
 #include "cmemory.h"
 #include <stdio.h>
 #include "usrchdat.c"
+#include "unicode/ubrk.h"
 
 static UBool      TOCLOSE_ = TRUE;
 static UCollator *EN_US_; 
 static UCollator *FR_FR_;
 static UCollator *DE_;
 static UCollator *ES_;
+
+/**
+ * CHECK_BREAK(char *brk)
+ *     Test if a break iterator is passed in AND break iteration is disabled. 
+ *     Skip the test if so.
+ * CHECK_BREAK_BOOL(char *brk)
+ *     Same as above, but returns 'TRUE' as a passing result
+ */
+
+#if !UCONFIG_NO_BREAK_ITERATION
 static UBreakIterator *EN_WORDBREAKER_;
 static UBreakIterator *EN_CHARACTERBREAKER_;
+#define CHECK_BREAK(x)
+#define CHECK_BREAK_BOOL(x)
+#else
+#define CHECK_BREAK(x)  if(x) { log_info("Skipping test on %s:%d because UCONFIG_NO_BREAK_ITERATION is on\n", __FILE__, __LINE__); return; }
+#define CHECK_BREAK_BOOL(x)  if(x) { log_info("Skipping test on %s:%d because UCONFIG_NO_BREAK_ITERATION is on\n", __FILE__, __LINE__); return TRUE; }
+#endif
 
 /**
 * Opening all static collators and break iterators
@@ -35,6 +55,13 @@ static void open(void)
         int32_t    rulelength = 0;
 
         EN_US_ = ucol_open("en_US", &status);
+        if(status == U_FILE_ACCESS_ERROR) {
+          log_data_err("Is your data around?\n");
+          return;
+        } else if(U_FAILURE(status)) {
+          log_err("Error opening collator\n");
+          return;
+        }
         FR_FR_ = ucol_open("fr_FR", &status);
         DE_ = ucol_open("de_DE", &status);
         ES_ = ucol_open("es_ES", &status);
@@ -52,9 +79,11 @@ static void open(void)
         ucol_close(ES_);
         ES_ = ucol_openRules(rules, u_strlen(rules), UCOL_ON, UCOL_TERTIARY,
                              NULL, &status); 
+#if !UCONFIG_NO_BREAK_ITERATION
         EN_WORDBREAKER_     = ubrk_open(UBRK_WORD, "en_US", NULL, 0, &status);
         EN_CHARACTERBREAKER_ = ubrk_open(UBRK_CHARACTER, "en_US", NULL, 0, 
                                         &status);
+#endif
         TOCLOSE_ = TRUE;
     }
 }
@@ -78,8 +107,10 @@ static void close(void)
         ucol_close(FR_FR_);
         ucol_close(DE_);
         ucol_close(ES_);
+#if !UCONFIG_NO_BREAK_ITERATION
         ubrk_close(EN_WORDBREAKER_);
         ubrk_close(EN_CHARACTERBREAKER_);
+#endif
     }
     TOCLOSE_ = FALSE;
 }
@@ -110,20 +141,8 @@ static char *toCharString(const UChar* unichars)
             *temp ++ = (char)ch;
         }
         else {
-            char digit[5];
-            int  zerosize;
-            *temp = 0;
-            strcat(temp, "\\u");
-            temp = temp + 2;
-            sprintf(digit, "%x", ch);
-            zerosize = 4 - strlen(digit);
-            while (zerosize != 0) {
-                *temp ++ = '0';
-                zerosize --;
-            }
-            *temp = 0;
-            strcat(temp, digit);
-            temp = temp + strlen(digit);
+            sprintf(temp, "\\u%04x", ch);
+            temp += 6; /* \uxxxx */
         }
     }
     *temp = 0;
@@ -161,12 +180,16 @@ static UBreakIterator *getBreakIterator(const char *breaker)
     if (breaker == NULL) {
         return NULL;
     }
+#if !UCONFIG_NO_BREAK_ITERATION
     if (strcmp(breaker, "wordbreaker") == 0) {
         return EN_WORDBREAKER_;
     }
     else {
         return EN_CHARACTERBREAKER_;
     }
+#else
+    return NULL;
+#endif
 }
 
 static void TestOpenClose(void) 
@@ -175,8 +198,10 @@ static void TestOpenClose(void)
           UStringSearch  *result;
     const UChar           pattern[] = {0x61, 0x62, 0x63, 0x64, 0x65, 0x66};
     const UChar           text[] = {0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67};
+#if !UCONFIG_NO_BREAK_ITERATION
           UBreakIterator *breakiter = ubrk_open(UBRK_WORD, "en_US", 
                                                 text, 6, &status);
+#endif
     /* testing null arguments */
     result = usearch_open(NULL, 0, NULL, 0, NULL, NULL, &status);
     if (U_SUCCESS(status) || result != NULL) {
@@ -231,7 +256,10 @@ static void TestOpenClose(void)
         usearch_close(result);
     }
 
+
     status = U_ZERO_ERROR;
+#if !UCONFIG_NO_BREAK_ITERATION
+
     result = usearch_open(pattern, 3, text, 6, "en_US", breakiter, &status);
     if (U_FAILURE(status) || result == NULL) {
         log_err("Error: Break iterator is valid for opening search\n");
@@ -249,6 +277,7 @@ static void TestOpenClose(void)
         usearch_close(result);
     }
     ubrk_close(breakiter);
+#endif
     close();
 }
 
@@ -257,6 +286,7 @@ static void TestInitialization(void)
           UErrorCode      status = U_ZERO_ERROR;
           UChar           pattern[512];
     const UChar           text[] = {0x61, 0x62, 0x63, 0x64, 0x65, 0x66};
+    int32_t i = 0;
     UStringSearch  *result;
 
     /* simple test on the pattern ce construction */
@@ -271,7 +301,10 @@ static void TestInitialization(void)
     usearch_close(result);
 
     /* testing if an extremely large pattern will fail the initialization */
-    uprv_memset(pattern, 0x41, 512);
+    for(i = 0; i < 512; i++) {
+      pattern[i] = 0x41;
+    }
+    /*uprv_memset(pattern, 0x41, 512);*/
     result = usearch_openFromCollator(pattern, 512, text, 3, EN_US_, NULL, 
                                       &status);
     if (U_FAILURE(status)) {
@@ -390,6 +423,8 @@ static UBool assertEqual(const SearchData search)
     UBreakIterator *breaker  = getBreakIterator(search.breaker);
     UStringSearch  *strsrch; 
     
+    CHECK_BREAK_BOOL(search.breaker);
+
     u_unescape(search.text, text, 128);
     u_unescape(search.pattern, pattern, 32);
     ucol_setStrength(collator, search.strength);
@@ -419,6 +454,7 @@ static UBool assertCanonicalEqual(const SearchData search)
     UBreakIterator *breaker  = getBreakIterator(search.breaker);
     UStringSearch  *strsrch; 
     
+    CHECK_BREAK_BOOL(search.breaker);
     u_unescape(search.text, text, 128);
     u_unescape(search.pattern, pattern, 32);
     ucol_setStrength(collator, search.strength);
@@ -452,6 +488,7 @@ static UBool assertEqualWithAttribute(const SearchData            search,
     UBreakIterator *breaker  = getBreakIterator(search.breaker);
     UStringSearch  *strsrch; 
     
+    CHECK_BREAK_BOOL(search.breaker);
     u_unescape(search.text, text, 128);
     u_unescape(search.pattern, pattern, 32);
     ucol_setStrength(collator, search.strength);
@@ -543,6 +580,9 @@ static void TestBreakIterator(void) {
     UChar           pattern[32];
     int             count = 0;
 
+    CHECK_BREAK("x");
+
+#if !UCONFIG_NO_BREAK_ITERATION
     open();
     if (usearch_getBreakIterator(NULL) != NULL) {
         log_err("Expected NULL breakiterator from NULL string search\n");
@@ -631,6 +671,7 @@ static void TestBreakIterator(void) {
     
 ENDTESTBREAKITERATOR:
     close();
+#endif
 }
 
 static void TestVariable(void) 
@@ -687,6 +728,13 @@ static void TestOverlap(void)
         u_unescape(search->pattern, pattern, 32);
         strsrch = usearch_openFromCollator(pattern, -1, text, -1, collator, 
                                            NULL, &status);
+        if(status == U_FILE_ACCESS_ERROR) {
+          log_data_err("Is your data around?\n");
+          return;
+        } else if(U_FAILURE(status)) {
+          log_err("Error opening searcher\n");
+          return;
+        }
         usearch_setAttribute(strsrch, USEARCH_OVERLAP, USEARCH_ON, &status);
         if (U_FAILURE(status) ||
             usearch_getAttribute(strsrch, USEARCH_OVERLAP) != USEARCH_ON) {
@@ -732,6 +780,13 @@ static void TestCollator(void)
     pattern[0] = 0x62;
     pattern[1] = 0x63;
     strsrch  = usearch_open(pattern, 2, text, 5, "en_US",  NULL,  &status);
+    if(status == U_FILE_ACCESS_ERROR) {
+      log_data_err("Is your data around?\n");
+      return;
+    } else if(U_FAILURE(status)) {
+      log_err("Error opening searcher\n");
+      return;
+    }
     tailored = usearch_getCollator(strsrch);
     if (usearch_next(strsrch, &status) != -1) {
         log_err("Error: Found case insensitive match, when we shouldn't\n");
@@ -818,6 +873,13 @@ static void TestPattern(void)
     ucol_setStrength(EN_US_, PATTERN[0].strength);
     strsrch = usearch_openFromCollator(pattern, -1, text, -1, EN_US_, 
                                        NULL, &status);
+    if(status == U_FILE_ACCESS_ERROR) {
+      log_data_err("Is your data around?\n");
+      return;
+    } else if(U_FAILURE(status)) {
+      log_err("Error opening searcher\n");
+      return;
+    }
 
     status = U_ZERO_ERROR;
     usearch_setPattern(strsrch, NULL, 3, &status);
@@ -983,6 +1045,8 @@ static void TestGetSetOffset(void)
     UChar          text[128];
     UErrorCode     status  = U_ZERO_ERROR;
     UStringSearch *strsrch;
+    memset(pattern, 0, 32*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
 
     open();
     if (usearch_getOffset(NULL) != USEARCH_DONE) {
@@ -1004,12 +1068,14 @@ static void TestGetSetOffset(void)
         SearchData  search      = BASIC[index ++];
         int32_t matchindex  = search.offset[count];
         int32_t     textlength;
-    
+        
         u_unescape(search.text, text, 128);
         u_unescape(search.pattern, pattern, 32);
         status = U_ZERO_ERROR;
         usearch_setText(strsrch, text, -1, &status);
         usearch_setPattern(strsrch, pattern, -1, &status);
+        ucol_setStrength(usearch_getCollator(strsrch), search.strength);
+        usearch_reset(strsrch);
         while (U_SUCCESS(status) && matchindex >= 0) {
             uint32_t matchlength = search.size[count];
             usearch_next(strsrch, &status);
@@ -1065,6 +1131,7 @@ static void TestGetSetOffset(void)
             return;
         }
     }
+    ucol_setStrength(usearch_getCollator(strsrch), UCOL_TERTIARY);
     usearch_close(strsrch);
     close();
 }
@@ -1075,6 +1142,9 @@ static void TestGetSetAttribute(void)
     UChar           pattern[32];
     UChar           text[128];
     UStringSearch  *strsrch;
+
+    memset(pattern, 0, 32*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
           
     open();
     if (usearch_getAttribute(NULL, USEARCH_OVERLAP) != USEARCH_DEFAULT ||
@@ -1389,12 +1459,19 @@ static void TestContraction(void)
     UErrorCode     status = U_ZERO_ERROR;
     int            count = 0;
     UStringSearch *strsrch;
+    memset(rules, 0, 128*sizeof(UChar));
+    memset(pattern, 0, 128*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
 
     u_unescape(CONTRACTIONRULE, rules, 128);
     collator = ucol_openRules(rules, u_strlen(rules), UCOL_ON, 
                               UCOL_TERTIARY, NULL, &status); 
-    if (U_FAILURE(status)) {
-        log_err("Error opening collator %s\n", u_errorName(status));
+    if(status == U_FILE_ACCESS_ERROR) {
+      log_data_err("Is your data around?\n");
+      return;
+    } else if(U_FAILURE(status)) {
+      log_err("Error opening collator %s\n", u_errorName(status));
+      return;
     }
     strsrch = usearch_openFromCollator(pattern, 1, text, 1, collator, NULL, 
                                        &status);
@@ -1426,11 +1503,19 @@ static void TestIgnorable(void)
     UStringSearch *strsrch;
     uint32_t       count = 0;
 
+    memset(rules, 0, 128*sizeof(UChar));
+    memset(pattern, 0, 128*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
+
     u_unescape(IGNORABLERULE, rules, 128);
     collator = ucol_openRules(rules, u_strlen(rules), UCOL_ON, 
                               IGNORABLE[count].strength, NULL, &status); 
-    if (U_FAILURE(status)) {
+    if(status == U_FILE_ACCESS_ERROR) {
+      log_data_err("Is your data around?\n");
+      return;
+    } else if(U_FAILURE(status)) {
         log_err("Error opening collator %s\n", u_errorName(status));
+        return;
     }
     strsrch = usearch_openFromCollator(pattern, 1, text, 1, collator, NULL, 
                                        &status);
@@ -1499,6 +1584,10 @@ static void TestBreakIteratorCanonical(void) {
     UErrorCode      status      = U_ZERO_ERROR;
     int             count = 0;
 
+    CHECK_BREAK("x");
+
+#if !UCONFIG_NO_BREAK_ITERATION
+
     open();
     while (count < 4) {
         /* 0-3 test are fixed */
@@ -1515,6 +1604,13 @@ static void TestBreakIteratorCanonical(void) {
         
         strsrch = usearch_openFromCollator(pattern, -1, text, -1, collator, 
                                            breaker, &status);
+        if(status == U_FILE_ACCESS_ERROR) {
+          log_data_err("Is your data around?\n");
+          return;
+        } else if(U_FAILURE(status)) {
+          log_err("Error opening searcher\n");
+          return;
+        }
         usearch_setAttribute(strsrch, USEARCH_CANONICAL_MATCH, USEARCH_ON, 
                              &status);
         if (U_FAILURE(status) || 
@@ -1559,6 +1655,7 @@ static void TestBreakIteratorCanonical(void) {
     
 ENDTESTBREAKITERATOR:
     close();
+#endif
 }
 
 static void TestVariableCanonical(void) 
@@ -1615,6 +1712,13 @@ static void TestOverlapCanonical(void)
         u_unescape(search->pattern, pattern, 32);
         strsrch = usearch_openFromCollator(pattern, -1, text, -1, collator, 
                                            NULL, &status);
+        if(status == U_FILE_ACCESS_ERROR) {
+          log_data_err("Is your data around?\n");
+          return;
+        } else if(U_FAILURE(status)) {
+          log_err("Error opening searcher\n");
+          return;
+        }
         usearch_setAttribute(strsrch, USEARCH_CANONICAL_MATCH, USEARCH_ON, 
                              &status);
         usearch_setAttribute(strsrch, USEARCH_OVERLAP, USEARCH_ON, &status);
@@ -1660,6 +1764,13 @@ static void TestCollatorCanonical(void)
 
     strsrch = usearch_openFromCollator(pattern, -1, text, -1, EN_US_, 
                                        NULL, &status);
+    if(status == U_FILE_ACCESS_ERROR) {
+      log_data_err("Is your data around?\n");
+      return;
+    } else if(U_FAILURE(status)) {
+      log_err("Error opening searcher\n");
+      return;
+    }
     usearch_setAttribute(strsrch, USEARCH_CANONICAL_MATCH, USEARCH_ON, 
                          &status);
     if (U_FAILURE(status)) {
@@ -1863,6 +1974,9 @@ static void TestGetSetOffsetCanonical(void)
     UErrorCode     status  = U_ZERO_ERROR;
     UStringSearch *strsrch;
 
+    memset(pattern, 0, 32*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
+
     open();
     strsrch = usearch_openFromCollator(pattern, 16, text, 32, EN_US_, NULL, 
                                        &status);
@@ -1955,16 +2069,23 @@ static void TestContractionCanonical(void)
     UChar          rules[128];
     UChar          pattern[128];
     UChar          text[128];
-    UCollator     *collator;
+    UCollator     *collator = NULL;
     UErrorCode     status = U_ZERO_ERROR;
     int            count = 0;
-    UStringSearch *strsrch;
+    UStringSearch *strsrch = NULL;
+    memset(rules, 0, 128*sizeof(UChar));
+    memset(pattern, 0, 128*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
 
     u_unescape(CONTRACTIONRULE, rules, 128);
     collator = ucol_openRules(rules, u_strlen(rules), UCOL_ON,
                               UCOL_TERTIARY, NULL, &status); 
-    if (U_FAILURE(status)) {
-        log_err("Error opening collator %s\n", u_errorName(status));
+    if(status == U_FILE_ACCESS_ERROR) {
+      log_data_err("Is your data around?\n");
+      return;
+    } else if(U_FAILURE(status)) {
+      log_err("Error opening collator %s\n", u_errorName(status));
+      return;
     }
     strsrch = usearch_openFromCollator(pattern, 1, text, 1, collator, NULL, 
                                        &status);
@@ -1989,6 +2110,37 @@ static void TestContractionCanonical(void)
     ucol_close(collator);
 }
 
+static void TestNumeric(void) {
+    UCollator     *coll = NULL;
+    UStringSearch *strsrch = NULL;
+    UErrorCode     status = U_ZERO_ERROR;
+
+    UChar          pattern[128];
+    UChar          text[128];
+    memset(pattern, 0, 128*sizeof(UChar));
+    memset(text, 0, 128*sizeof(UChar));
+
+    coll = ucol_open("", &status);
+    if(U_FAILURE(status)) {
+        log_data_err("Could not open UCA. Is your data around?\n");
+        return;
+    }
+
+    ucol_setAttribute(coll, UCOL_NUMERIC_COLLATION, UCOL_ON, &status);
+
+    strsrch = usearch_openFromCollator(pattern, 1, text, 1, coll, NULL, &status);
+
+    if(status != U_UNSUPPORTED_ERROR || U_SUCCESS(status)) {
+        log_err("Expected U_UNSUPPORTED_ERROR when trying to instantiate a search object from a CODAN collator, got %s instead\n", u_errorName(status));
+        if(strsrch) {
+            usearch_close(strsrch);
+        }
+    }
+
+    ucol_close(coll);
+
+}
+
 void addSearchTest(TestNode** root)
 {
     addTest(root, &TestStart, "tscoll/usrchtst/TestStart");
@@ -2038,5 +2190,7 @@ void addSearchTest(TestNode** root)
     addTest(root, &TestContractionCanonical, 
                                  "tscoll/usrchtst/TestContractionCanonical");
     addTest(root, &TestEnd, "tscoll/usrchtst/TestEnd");
+    addTest(root, &TestNumeric, "tscoll/usrchtst/TestNumeric");
 }
 
+#endif /* #if !UCONFIG_NO_COLLATION */
