@@ -1,10 +1,13 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2006, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
- 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "dtfmrgts.h"
 
 #include "unicode/timezone.h"
@@ -50,6 +53,7 @@ DateFormatRegressionTest::runIndexedTest( int32_t index, UBool exec, const char*
         CASE(22,Test4182066)
         CASE(23,Test4210209)
         CASE(24,Test714)
+        CASE(25,Test1684)
         default: name = ""; break;
     }
 }
@@ -65,8 +69,15 @@ void DateFormatRegressionTest::Test4029195(void)
     logln((UnicodeString) "today: " + today);
 
     SimpleDateFormat *sdf = (SimpleDateFormat*) DateFormat::createDateInstance();
-    failure(status, "SimpleDateFormat::createDateInstance");
+    if (failure(status, "SimpleDateFormat::createDateInstance")) {
+        return;
+    }
     UnicodeString pat;
+    if(sdf == NULL){
+        dataerrln("Error calling DateFormat::createDateTimeInstance");
+        return;
+    }
+
     pat = sdf->toPattern(pat);
     logln("pattern: " + pat);
     UnicodeString fmtd;
@@ -110,7 +121,12 @@ void DateFormatRegressionTest::Test4052408(void)
 
     DateFormat *fmt = DateFormat::createDateTimeInstance(DateFormat::SHORT,
                                                 DateFormat::SHORT, Locale::getUS());
-    UDate dt = date(97, Calendar::MAY, 3, 8, 55);
+    if (fmt == NULL) {
+        dataerrln("Error calling DateFormat::createDateTimeInstance");
+        return;
+    }
+
+    UDate dt = date(97, UCAL_MAY, 3, 8, 55);
     UnicodeString str;
     str = fmt->format(dt, str);
     logln(str);
@@ -208,16 +224,16 @@ void DateFormatRegressionTest::Test4056591(void)
     //try {
         SimpleDateFormat *fmt = new SimpleDateFormat(UnicodeString("yyMMdd"), Locale::getUS(), status);
         failure(status, "new SimpleDateFormat");
-        UDate start = date(1809-1900, Calendar::DECEMBER, 25);
+        UDate start = date(1809-1900, UCAL_DECEMBER, 25);
         fmt->set2DigitYearStart(start, status);
         failure(status, "fmt->setTwoDigitStartDate");
         if( (fmt->get2DigitYearStart(status) != start) || failure(status, "get2DigitStartDate"))
             errln("get2DigitYearStart broken");
         UDate dates [] = {
-            date(1809-1900, Calendar::DECEMBER, 25),
-            date(1909-1900, Calendar::DECEMBER, 24),
-            date(1809-1900, Calendar::DECEMBER, 26),
-            date(1861-1900, Calendar::DECEMBER, 25),
+            date(1809-1900, UCAL_DECEMBER, 25),
+            date(1909-1900, UCAL_DECEMBER, 24),
+            date(1809-1900, UCAL_DECEMBER, 26),
+            date(1861-1900, UCAL_DECEMBER, 25),
         };
 
         UnicodeString strings [] = {
@@ -263,7 +279,7 @@ void DateFormatRegressionTest::Test4059917(void)
     UnicodeString myDate;
 
     fmt = new SimpleDateFormat( UnicodeString("yyyy/MM/dd"), status );
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     myDate = "1997/01/01";
     aux917( fmt, myDate );
     
@@ -271,7 +287,7 @@ void DateFormatRegressionTest::Test4059917(void)
     fmt = NULL;
     
     fmt = new SimpleDateFormat( UnicodeString("yyyyMMdd"), status );
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     myDate = "19970101";
     aux917( fmt, myDate );
               
@@ -319,12 +335,18 @@ void DateFormatRegressionTest::Test4060212(void)
     logln("Using yyyy-DDD.hh:mm:ss");
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *formatter = new SimpleDateFormat(UnicodeString("yyyy-DDD.hh:mm:ss"), status);
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     ParsePosition pos(0);
     UDate myDate = formatter->parse( dateString, pos );
     UnicodeString myString;
     DateFormat *fmt = DateFormat::createDateTimeInstance( DateFormat::FULL,
                                                             DateFormat::LONG);
+    if (fmt == NULL) {
+        dataerrln("Error calling DateFormat::createDateTimeInstance");
+        delete formatter;
+        return;
+    }
+
     myString = fmt->format( myDate, myString);
     logln( myString );
 
@@ -332,23 +354,23 @@ void DateFormatRegressionTest::Test4060212(void)
     failure(status, "new GregorianCalendar");
     cal->setTime(myDate, status);
     failure(status, "cal->setTime");
-    if ((cal->get(Calendar::DAY_OF_YEAR, status) != 40) || failure(status, "cal->get"))
-        errln((UnicodeString) "Fail: Got " + cal->get(Calendar::DAY_OF_YEAR, status) +
+    if ((cal->get(UCAL_DAY_OF_YEAR, status) != 40) || failure(status, "cal->get"))
+        errln((UnicodeString) "Fail: Got " + cal->get(UCAL_DAY_OF_YEAR, status) +
                             " Want 40");
 
     logln("Using yyyy-ddd.hh:mm:ss");
     delete formatter;
     formatter = NULL;
     formatter = new SimpleDateFormat(UnicodeString("yyyy-ddd.hh:mm:ss"), status);
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     pos.setIndex(0);
     myDate = formatter->parse( dateString, pos );
     myString = fmt->format( myDate, myString );
     logln( myString );
     cal->setTime(myDate, status);
     failure(status, "cal->setTime");
-    if ((cal->get(Calendar::DAY_OF_YEAR, status) != 40) || failure(status, "cal->get"))
-        errln((UnicodeString) "Fail: Got " + cal->get(Calendar::DAY_OF_YEAR, status) +
+    if ((cal->get(UCAL_DAY_OF_YEAR, status) != 40) || failure(status, "cal->get"))
+        errln((UnicodeString) "Fail: Got " + cal->get(UCAL_DAY_OF_YEAR, status) +
                             " Want 40");
 
     delete formatter;
@@ -364,6 +386,11 @@ void DateFormatRegressionTest::Test4061287(void)
     UErrorCode status = U_ZERO_ERROR;
     
     SimpleDateFormat *df = new SimpleDateFormat(UnicodeString("dd/MM/yyyy"), status);
+    if(U_FAILURE(status)) {
+      errln("Couldn't create SimpleDateFormat, error: %s", u_errorName(status));
+      delete df;
+      return;
+    }
     failure(status, "new SimpleDateFormat");
     //try {
     logln(UnicodeString("") + df->parse("35/01/1971", status));  
@@ -408,7 +435,16 @@ void DateFormatRegressionTest::Test4065240(void)
         TimeZone::adoptDefault(TimeZone::createTimeZone("EST"));
         curDate = date(98, 0, 1);
         shortdate = DateFormat::createDateInstance(DateFormat::SHORT);
+        if (shortdate == NULL){
+            dataerrln("Error calling DateFormat::createDateInstance");
+            return;
+        }
+
         fulldate = DateFormat::createDateTimeInstance(DateFormat::LONG, DateFormat::LONG);
+        if (fulldate == NULL){
+            dataerrln("Error calling DateFormat::createDateTimeInstance");
+            return;
+        }
         strShortDate = "The current date (short form) is ";
         UnicodeString temp;
         temp = shortdate->format(curDate, temp);
@@ -425,7 +461,7 @@ void DateFormatRegressionTest::Test4065240(void)
 
         // Check to see if the resource is present; if not, we can't test
         ResourceBundle *bundle = new ResourceBundle(
-            u_getDataDirectory(), *curLocale, status);
+            NULL, *curLocale, status);
         failure(status, "new ResourceBundle");
             //(UnicodeString) "java.text.resources.DateFormatZoneData", curLocale);
 
@@ -445,7 +481,6 @@ void DateFormatRegressionTest::Test4065240(void)
     failure(status, "Locale::setDefault");
     TimeZone::setDefault(*saveZone);
     //}
-
     delete shortdate;
     delete fulldate;
     delete saveZone;
@@ -469,12 +504,25 @@ void DateFormatRegressionTest::Test4071441(void)
 {
     DateFormat *fmtA = DateFormat::createInstance();
     DateFormat *fmtB = DateFormat::createInstance();
-    
+
+    if (fmtA == NULL || fmtB == NULL){
+        dataerrln("Error calling DateFormat::createInstance");
+        delete fmtA;
+        delete fmtB;
+        return;
+    }
+
     // {sfb} Is it OK to cast away const here?
     Calendar *calA = (Calendar*) fmtA->getCalendar();
     Calendar *calB = (Calendar*) fmtB->getCalendar();
+    if(!calA || !calB) {
+      errln("Couldn't get proper calendars, exiting");
+      delete fmtA;
+      delete fmtB;
+      return;
+    }
     UDate epoch = date(0, 0, 0);
-    UDate xmas = date(61, Calendar::DECEMBER, 25);
+    UDate xmas = date(61, UCAL_DECEMBER, 25);
 
     UErrorCode status = U_ZERO_ERROR;
     calA->setTime(epoch, status);
@@ -491,6 +539,7 @@ void DateFormatRegressionTest::Test4071441(void)
         errln("Fail: Can't complete test; Calendar instances equal");
     if (*fmtA != *fmtB)
         errln("Fail: DateFormat unequal when Calendars equivalent");
+
     logln("DateFormat.equals ok");
 
     delete fmtA;
@@ -513,7 +562,12 @@ void DateFormatRegressionTest::Test4071441(void)
 void DateFormatRegressionTest::Test4073003(void) 
 {
     //try {
-    DateFormat *fmt = DateFormat::createDateInstance(DateFormat::SHORT, Locale::getUK());
+    UErrorCode ec = U_ZERO_ERROR;
+    SimpleDateFormat fmt("dd/MM/yy", Locale::getUK(), ec);
+    if (U_FAILURE(ec)) {
+        errln("FAIL: SimpleDateFormat constructor");
+        return;
+    }
         UnicodeString tests [] = { 
             (UnicodeString) "12/25/61", 
             (UnicodeString) "12/25/1961", 
@@ -522,26 +576,20 @@ void DateFormatRegressionTest::Test4073003(void)
         };
         UErrorCode status = U_ZERO_ERROR;
         for(int i= 0; i < 4; i+=2) {
-            UDate d = fmt->parse(tests[i], status);
-            failure(status, "fmt->parse");
-            UDate dd = fmt->parse(tests[i+1], status);
-            failure(status, "fmt->parse");
+            UDate d = fmt.parse(tests[i], status);
+            failure(status, "fmt.parse");
+            UDate dd = fmt.parse(tests[i+1], status);
+            failure(status, "fmt.parse");
             UnicodeString s;
-            s = fmt->format(d, s);
+            s = fmt.format(d, s);
             UnicodeString ss;
-            ss = fmt->format(dd, ss);
+            ss = fmt.format(dd, ss);
             if (d != dd)
                 errln((UnicodeString) "Fail: " + d + " != " + dd);
             if (s != ss)
                 errln((UnicodeString)"Fail: " + s + " != " + ss);
             logln("Ok: " + s + " " + d);
         }
-    /*}
-    catch (ParseException e) {
-        errln("Fail: " + e);
-        e.printStackTrace();
-    }*/
-        delete fmt;
 }
 
 /**
@@ -555,6 +603,13 @@ void DateFormatRegressionTest::Test4089106(void)
         TimeZone::setDefault(*z);
         UErrorCode status = U_ZERO_ERROR;
         SimpleDateFormat *f = new SimpleDateFormat(status);
+        if(U_FAILURE(status)) {
+          errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+          delete f;
+          delete def;
+          delete z;
+          return;
+        }
         failure(status, "new SimpleDateFormat");
         if (f->getTimeZone()!= *z)
             errln("Fail: SimpleTimeZone should use TimeZone.getDefault()");
@@ -648,8 +703,8 @@ void DateFormatRegressionTest::Test4101483(void)
 {
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *sdf = new SimpleDateFormat(UnicodeString("z"), Locale::getUS(), status);
-    failure(status, "new SimpleDateFormat");
-    FieldPosition fp(DateFormat::TIMEZONE_FIELD);
+    if(failure(status, "new SimpleDateFormat")) return;
+    FieldPosition fp(UDAT_TIMEZONE_FIELD);
     //Date d = date(9234567890L);
     UDate d = 9234567890.0;
     //StringBuffer buf = new StringBuffer("");
@@ -681,7 +736,7 @@ void DateFormatRegressionTest::Test4103340(void)
     // and some arbitrary time 
     UDate d = date(97, 3, 1, 1, 1, 1); 
     SimpleDateFormat *df = new SimpleDateFormat(UnicodeString("MMMM"), Locale::getUS(), status); 
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
 
     UnicodeString s;
     s = dateToString(d, s);
@@ -710,6 +765,11 @@ void DateFormatRegressionTest::Test4103341(void)
     TimeZone::adoptDefault(TimeZone::createTimeZone("CST"));
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *simple = new SimpleDateFormat(UnicodeString("MM/dd/yyyy HH:mm"), status);
+    if(U_FAILURE(status)) {
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      delete simple;
+      return;
+    }
     failure(status, "new SimpleDateFormat");
     TimeZone *temp = TimeZone::createDefault();
     if(simple->getTimeZone() != *temp)
@@ -730,7 +790,12 @@ void DateFormatRegressionTest::Test4104136(void)
 {
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *sdf = new SimpleDateFormat(status); 
-    failure(status, "new SimpleDateFormat");
+    if(U_FAILURE(status)) {
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      delete sdf;
+      return;
+    }
+    if(failure(status, "new SimpleDateFormat")) return;
     UnicodeString pattern = "'time' hh:mm"; 
     sdf->applyPattern(pattern); 
     logln("pattern: \"" + pattern + "\""); 
@@ -748,7 +813,7 @@ void DateFormatRegressionTest::Test4104136(void)
     };
 
     UDate dates [] = {
-        date(70, Calendar::JANUARY, 1, 10, 30),
+        date(70, UCAL_JANUARY, 1, 10, 30),
         -1,
         -1
     };
@@ -790,6 +855,11 @@ void DateFormatRegressionTest::Test4104522(void)
     UErrorCode status = U_ZERO_ERROR;
     
     SimpleDateFormat *sdf = new SimpleDateFormat(status);
+    if(U_FAILURE(status)) {
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      delete sdf;
+      return;
+    }
     failure(status, "new SimpleDateFormat");
     UnicodeString pattern = "'time' hh:mm";
     sdf->applyPattern(pattern);
@@ -828,6 +898,16 @@ void DateFormatRegressionTest::Test4106807(void)
         new SimpleDateFormat(UnicodeString("yyyyMMddHHmmss'a''a'"), status),
         new SimpleDateFormat(UnicodeString("yyyyMMddHHmmss %"), status)
     };
+    if(U_FAILURE(status)) {
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      delete sdfs[0];
+      delete sdfs[1];
+      delete sdfs[2];
+      delete sdfs[3];
+      delete sdfs[4];
+      return;
+    }
+
     failure(status, "new SimpleDateFormat");
     
     UnicodeString strings [] = {
@@ -870,7 +950,7 @@ void DateFormatRegressionTest::Test4106807(void)
             //logln(df->format(dt)); 
             gc->setTime(dt, status); 
             failure(status, "gc->getTime");
-            logln(UnicodeString("") + gc->get(Calendar::ZONE_OFFSET, status));
+            logln(UnicodeString("") + gc->get(UCAL_ZONE_OFFSET, status));
             failure(status, "gc->get");
             UnicodeString s;
             s = format->format(dt, s, pos);
@@ -922,7 +1002,7 @@ void DateFormatRegressionTest::Test4134203(void)
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString dateFormat = "MM/dd/yy HH:mm:ss zzz";
     SimpleDateFormat *fmt = new SimpleDateFormat(dateFormat, status);
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     ParsePosition p0(0);
     UDate d = fmt->parse("01/22/92 04:52:00 GMT", p0);
     logln(dateToString(d));
@@ -944,10 +1024,10 @@ void DateFormatRegressionTest::Test4151631(void)
     logln("pattern=" + pattern);
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *format = new SimpleDateFormat(pattern, Locale::getUS(), status);
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     UnicodeString result;
     FieldPosition pos(FieldPosition::DONT_CARE);
-    result = format->format(date(1998-1900, Calendar::JUNE, 30, 13, 30, 0), result, pos);
+    result = format->format(date(1998-1900, UCAL_JUNE, 30, 13, 30, 0), result, pos);
     if (result != "TO_DATE('30-06-1998 13:30:00' , 'DD-MM-YYYY HH:MI:SS')") {
         errln("Fail: result=" + result);
     }
@@ -968,7 +1048,7 @@ void DateFormatRegressionTest::Test4151706(void)
     UnicodeString dateString("Thursday, 31-Dec-98 23:00:00 GMT");
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat fmt(UnicodeString("EEEE, dd-MMM-yy HH:mm:ss z"), Locale::getUS(), status);
-    failure(status, "new SimpleDateFormat");
+    if(failure(status, "new SimpleDateFormat")) return;
     //try {
         UDate d = fmt.parse(dateString, status);
         failure(status, "fmt->parse");
@@ -995,8 +1075,10 @@ DateFormatRegressionTest::Test4162071(void)
     UnicodeString format("EEE', 'dd-MMM-yyyy HH:mm:ss z"); // RFC 822/1123
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat df(format, Locale::getUS(), status);
-    if(U_FAILURE(status))
+    if(U_FAILURE(status)) {
         errln("Couldn't create SimpleDateFormat");
+        return;
+    }
     
     //try {
         UDate x = df.parse(dateString, status);
@@ -1042,14 +1124,14 @@ void DateFormatRegressionTest::Test4182066(void) {
     int32_t STRINGS_COUNT = (int32_t)(sizeof(STRINGS) / sizeof(STRINGS[0]));
     UDate FAIL_DATE = (UDate) 0;
     UDate DATES[] = {
-        date(2000-1900, Calendar::FEBRUARY, 29),
-        date(2001-1900, Calendar::JANUARY,  23),
-        date(  -1-1900, Calendar::APRIL,     5),
-        date(  -9-1900, Calendar::JANUARY,  23),
-        date(1314-1900, Calendar::NOVEMBER, 12),
-        date(   1-1900, Calendar::OCTOBER,  31),
+        date(2000-1900, UCAL_FEBRUARY, 29),
+        date(2001-1900, UCAL_JANUARY,  23),
+        date(  -1-1900, UCAL_APRIL,     5),
+        date(  -9-1900, UCAL_JANUARY,  23),
+        date(1314-1900, UCAL_NOVEMBER, 12),
+        date(   1-1900, UCAL_OCTOBER,  31),
         FAIL_DATE, // "+1" isn't recognized by US NumberFormat
-        date(   1-1900, Calendar::SEPTEMBER,12),
+        date(   1-1900, UCAL_SEPTEMBER,12),
     };
 
     UnicodeString out;
@@ -1110,7 +1192,7 @@ DateFormatRegressionTest::Test4210209(void) {
     }
     Calendar* calx = (Calendar*)fmt.getCalendar(); // cast away const!
     calx->setLenient(FALSE);
-    UDate d = date(2000-1900, Calendar::FEBRUARY, 29);
+    UDate d = date(2000-1900, UCAL_FEBRUARY, 29);
     UnicodeString s, ss;
     fmt.format(d, s);
     logln(disp.format(d, ss.remove()) + " f> " + pattern +
@@ -1135,7 +1217,7 @@ DateFormatRegressionTest::Test4210209(void) {
     }
     cal.clear();
     cal.setLenient(FALSE);
-    cal.set(2000, Calendar::FEBRUARY, 29); // This should work!
+    cal.set(2000, UCAL_FEBRUARY, 29); // This should work!
     logln(UnicodeString("Attempt to set Calendar to Feb 29 2000: ") +
                         disp.format(cal.getTime(status), ss.remove()));
     if (U_FAILURE(status)) {
@@ -1150,6 +1232,11 @@ void DateFormatRegressionTest::Test714(void)
     DateFormat *fmt = DateFormat::createDateTimeInstance(DateFormat::NONE,
                                                          DateFormat::MEDIUM,
                                                          Locale::getUS());
+    if (fmt == NULL) {
+        dataerrln("Error calling DateFormat::createDateTimeInstance");
+        return;
+    }
+
     UnicodeString s;
         UnicodeString tests = 
           (UnicodeString) "7:25:43 AM" ;
@@ -1173,4 +1260,171 @@ void DateFormatRegressionTest::Test714(void)
    delete fmt;
 }
 
+class Test1684Data {
+public:
+  int32_t year;
+  int32_t month;
+  int32_t date;
+  int32_t womyear;
+  int32_t wommon;
+  int32_t wom;
+  int32_t dow;
+  UnicodeString data;
+  UnicodeString normalized;
+    
+  Test1684Data(int32_t xyear, int32_t xmonth, int32_t xdate,
+               int32_t xwomyear, int32_t xwommon, int32_t xwom, int32_t xdow,
+               const char *xdata, const char *xnormalized) :
+    year(xyear),
+    month(xmonth-1),
+    date(xdate),
+    womyear(xwomyear),
+    wommon(xwommon-1),
+    wom(xwom),
+    dow(xdow),
+    data(xdata,""),
+    normalized((xnormalized==NULL)?xdata:xnormalized,"")
+  { }
+};
+
+void DateFormatRegressionTest::Test1684(void)
+{
+  //      July 2001            August 2001           January 2002    
+  // Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa
+  //  1  2  3  4  5  6  7            1  2  3  4         1  2  3  4  5
+  //  8  9 10 11 12 13 14   5  6  7  8  9 10 11   6  7  8  9 10 11 12
+  // 15 16 17 18 19 20 21  12 13 14 15 16 17 18  13 14 15 16 17 18 19
+  // 22 23 24 25 26 27 28  19 20 21 22 23 24 25  20 21 22 23 24 25 26
+  // 29 30 31              26 27 28 29 30 31     27 28 29 30 31      
+  Test1684Data *tests[] = {
+    new Test1684Data(2001, 8,  6,  2001,8,2,UCAL_MONDAY,    "2001 08 02 Mon", NULL),
+    new Test1684Data(2001, 8,  7,  2001,8,2,UCAL_TUESDAY,   "2001 08 02 Tue", NULL),
+    new Test1684Data(2001, 8,  5,/*12,*/ 2001,8,2,UCAL_SUNDAY,    "2001 08 02 Sun", NULL),
+    new Test1684Data(2001, 8,6, /*7,  30,*/ 2001,7,6,UCAL_MONDAY,    "2001 07 06 Mon", "2001 08 02 Mon"),
+    new Test1684Data(2001, 8,7, /*7,  31,*/ 2001,7,6,UCAL_TUESDAY,   "2001 07 06 Tue", "2001 08 02 Tue"),
+    new Test1684Data(2001, 8,  5,  2001,7,6,UCAL_SUNDAY,    "2001 07 06 Sun", "2001 08 02 Sun"),
+    new Test1684Data(2001, 7,  30, 2001,8,1,UCAL_MONDAY,    "2001 08 01 Mon", "2001 07 05 Mon"),
+    new Test1684Data(2001, 7,  31, 2001,8,1,UCAL_TUESDAY,   "2001 08 01 Tue", "2001 07 05 Tue"),
+    new Test1684Data(2001, 7,29, /*8,  5,*/  2001,8,1,UCAL_SUNDAY,    "2001 08 01 Sun", "2001 07 05 Sun"),
+    new Test1684Data(2001, 12, 31, 2001,12,6,UCAL_MONDAY,   "2001 12 06 Mon", NULL),
+    new Test1684Data(2002, 1,  1,  2002,1,1,UCAL_TUESDAY,   "2002 01 01 Tue", NULL),
+    new Test1684Data(2002, 1,  2,  2002,1,1,UCAL_WEDNESDAY, "2002 01 01 Wed", NULL),
+    new Test1684Data(2002, 1,  3,  2002,1,1,UCAL_THURSDAY,  "2002 01 01 Thu", NULL),
+    new Test1684Data(2002, 1,  4,  2002,1,1,UCAL_FRIDAY,    "2002 01 01 Fri", NULL),
+    new Test1684Data(2002, 1,  5,  2002,1,1,UCAL_SATURDAY,  "2002 01 01 Sat", NULL),
+    new Test1684Data(2001,12,30, /*2002, 1,  6,*/  2002,1,1,UCAL_SUNDAY,    "2002 01 01 Sun", "2001 12 06 Sun")
+  };
+
+#define kTest1684Count  ((int32_t)(sizeof(tests)/sizeof(tests[0])))
+
+  int32_t pass = 0, error = 0, warning = 0;
+  int32_t i;
+
+  UErrorCode status = U_ZERO_ERROR;
+  UnicodeString pattern("yyyy MM WW EEE","");
+  Calendar *cal = new GregorianCalendar(status);
+  SimpleDateFormat *sdf = new SimpleDateFormat(pattern,status);
+  if (U_FAILURE(status)) {
+    dataerrln("Error constructing SimpleDateFormat");
+    for(i=0;i<kTest1684Count;i++) {
+        delete tests[i];
+    }
+    delete cal;
+    delete sdf;
+    return;
+  }
+  cal->setFirstDayOfWeek(UCAL_SUNDAY);
+  cal->setMinimalDaysInFirstWeek(1);
+
+  sdf->adoptCalendar(cal);
+
+  cal = sdf->getCalendar()->clone(); // sdf may have deleted calendar
+
+  if(!cal || !sdf || U_FAILURE(status)) {
+    errln(UnicodeString("Error setting up test: ") + u_errorName(status));
+  }
+
+  for (i = 0; i < kTest1684Count; ++i) {
+    Test1684Data &test = *(tests[i]);
+    logln(UnicodeString("#") + i + UnicodeString("\n-----\nTesting round trip of ") + test.year +
+        " " + (test.month + 1) +
+        " " + test.date +
+          " (written as) " + test.data);
+    
+    cal->clear();
+    cal->set(test.year, test.month, test.date);
+    UDate ms = cal->getTime(status);
+    
+    cal->clear();
+    cal->set(UCAL_YEAR, test.womyear);
+    cal->set(UCAL_MONTH, test.wommon);
+    cal->set(UCAL_WEEK_OF_MONTH, test.wom);
+    cal->set(UCAL_DAY_OF_WEEK, test.dow);
+    UDate ms2 = cal->getTime(status);
+            
+    if (ms2 != ms) {
+      errln((UnicodeString)"\nError: GregorianUCAL_DOM gave " + ms +
+            "\n       GregorianUCAL_WOM gave " + ms2);
+      error++;
+    } else {
+      pass++;
+    }
+    
+    ms2 = sdf->parse(test.data, status);
+    if(U_FAILURE(status)) {
+      errln("parse exception: " + UnicodeString(u_errorName(status)));
+    }
+    
+    if (ms2!=ms) {
+      errln((UnicodeString)"\nError: GregorianCalendar gave      " + ms +
+            "\n       SimpleDateFormat.parse gave " + ms2);
+      error++;
+    } else {
+      pass++;
+    }
+
+    UnicodeString result;
+    sdf->format(ms, result);
+    if (result != test.normalized) {
+      errln("\nWarning: format of '" + test.data + "' gave" +
+            "\n                   '" + result + "'" +
+            "\n          expected '" + test.normalized + "'");
+      warning++;
+    } else {
+      pass++;
+    }
+            
+    UDate ms3;
+    ms3 = sdf->parse(result, status);
+    if(U_FAILURE(status)) {
+      errln("parse exception 2: " + (UnicodeString)u_errorName(status));
+    }
+    
+    if (ms3!=ms) {
+      error++;
+      errln((UnicodeString)"\nError: Re-parse of '" + result + "' gave time of " +
+          "\n        " + ms3 +
+          "\n    not " + ms);
+    } else {
+      pass++;
+            }
+  }
+
+  UnicodeString info 
+    = UnicodeString("Passed: ") + pass + ", Warnings: " + warning + ", Errors: " + error;
+  if (error > 0) {
+    errln(info);
+  } else {
+    logln(info);
+  }
+
+  for(i=0;i<kTest1684Count;i++) {
+    delete tests[i];
+  }
+  delete cal;
+  delete sdf;
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 //eof
