@@ -1,19 +1,19 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2002, International Business Machines Corporation and
+ * Copyright (c) 1997-2005, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-
 #include "unicode/utypes.h"
 
 #include "cstring.h"
 #include "unicode/unistr.h"
 #include "unicode/resbund.h"
 #include "restsnew.h"
+
+#include <stdlib.h>
+#include <time.h>
+#include <string.h>
 #include <limits.h>
 
 //***************************************************************************************
@@ -34,11 +34,11 @@ enum E_Where
 
 //***************************************************************************************
 
-#define CONFIRM_EQ(actual,expected) if ((expected)==(actual)) { record_pass(); } else { record_fail(); OUT << action << " returned " << (actual) << " instead of " << (expected) << endl; }
-#define CONFIRM_GE(actual,expected) if ((actual)>=(expected)) { record_pass(); } else { record_fail(); OUT << action << " returned " << (actual) << " instead of x >= " << (expected) << endl; }
-#define CONFIRM_NE(actual,expected) if ((expected)!=(actual)) { record_pass(); } else { record_fail(); OUT << action << " returned " << (actual) << " instead of x != " << (expected) << endl; }
+#define CONFIRM_EQ(actual,expected) if ((expected)==(actual)) { record_pass(); } else { record_fail(); errln(action + (UnicodeString)" returned " + (actual) + (UnicodeString)" instead of " + (expected) + (UnicodeString)"\n"); }
+#define CONFIRM_GE(actual,expected) if ((actual)>=(expected)) { record_pass(); } else { record_fail(); errln(action + (UnicodeString)" returned " + (actual) + (UnicodeString)" instead of x >= " + (expected) + (UnicodeString)"\n"); }
+#define CONFIRM_NE(actual,expected) if ((expected)!=(actual)) { record_pass(); } else { record_fail(); errln(action + (UnicodeString)" returned " + (actual) + (UnicodeString)" instead of x != " + (expected) + (UnicodeString)"\n"); }
 
-#define CONFIRM_UErrorCode(actual,expected) if ((expected)==(actual)) { record_pass(); } else { record_fail(); OUT << action << " returned " << u_errorName(actual) << " instead of " << u_errorName(expected) << endl; }
+#define CONFIRM_UErrorCode(actual,expected) if ((expected)==(actual)) { record_pass(); } else { record_fail(); errln(action + (UnicodeString)" returned " + (UnicodeString)u_errorName(actual) + (UnicodeString)" instead of " + (UnicodeString)u_errorName(expected) + (UnicodeString)"\n"); }
 
 //***************************************************************************************
 
@@ -102,9 +102,9 @@ param[] =
     { "root",       0,   U_ZERO_ERROR,             e_Root,      { TRUE, FALSE, FALSE }, { TRUE, FALSE, FALSE } },
     { "te",         0,   U_ZERO_ERROR,             e_te,        { FALSE, TRUE, FALSE }, { TRUE, TRUE, FALSE  } },
     { "te_IN",      0,   U_ZERO_ERROR,             e_te_IN,     { FALSE, FALSE, TRUE }, { TRUE, TRUE, TRUE   } },
-    { "te_NE",      0,   U_USING_FALLBACK_ERROR,   e_te,        { FALSE, TRUE, FALSE }, { TRUE, TRUE, FALSE  } },
-    { "te_IN_NE",   0,   U_USING_FALLBACK_ERROR,   e_te_IN,     { FALSE, FALSE, TRUE }, { TRUE, TRUE, TRUE   } },
-    { "ne",         0,   U_USING_DEFAULT_ERROR,    e_Root,      { TRUE, FALSE, FALSE }, { TRUE, FALSE, FALSE } }
+    { "te_NE",      0,   U_USING_FALLBACK_WARNING, e_te,        { FALSE, TRUE, FALSE }, { TRUE, TRUE, FALSE  } },
+    { "te_IN_NE",   0,   U_USING_FALLBACK_WARNING, e_te_IN,     { FALSE, FALSE, TRUE }, { TRUE, TRUE, TRUE   } },
+    { "ne",         0,   U_USING_DEFAULT_WARNING,  e_Root,      { TRUE, FALSE, FALSE }, { TRUE, FALSE, FALSE } }
 };
 
 static int32_t bundles_count = sizeof(param) / sizeof(param[0]);
@@ -155,8 +155,7 @@ static int32_t randi(int32_t n)
 */
 NewResourceBundleTest::NewResourceBundleTest()
 : pass(0),
-  fail(0),
-  OUT(it_out)
+  fail(0)
 {
     if (param[5].locale == NULL) {
         param[0].locale = new Locale("root");
@@ -187,7 +186,8 @@ void NewResourceBundleTest::runIndexedTest( int32_t index, UBool exec, const cha
     case 1: name = "TestConstruction"; if (exec) TestConstruction(); break;
     case 2: name = "TestIteration"; if (exec) TestIteration(); break;
     case 3: name = "TestOtherAPI";  if(exec) TestOtherAPI(); break;
-
+    case 4: name = "TestNewTypes";  if(exec) TestNewTypes(); break;
+    case 5: name = "TestGetByFallback";  if(exec) TestGetByFallback(); break;
         default: name = ""; break; //needed to end loop
     }
 }
@@ -262,40 +262,8 @@ NewResourceBundleTest::TestConstruction()
         delete[] versionID1;
         delete[] versionID2;
     }
-    {
-        UErrorCode   err = U_ZERO_ERROR;
-        const char* testdatapath;
-        Locale       locale("te", "IN");
-
-        testdatapath=loadTestData(err);
-        if(U_FAILURE(err))
-        {
-            errln("Could not load testdata.dat %s " + UnicodeString(u_errorName(err)));
-            return;
-        }
-
-
-        wchar_t* wideDirectory = new wchar_t[256];
-        mbstowcs(wideDirectory, testdatapath, 256);
-    
-        ResourceBundle  test2(wideDirectory, locale, err);
-
-        UnicodeString   result2;
-
-        result2 = test2.getStringEx("string_in_Root_te_te_IN", err);
-        if (U_FAILURE(err)) {
-            errln("Something threw an error in TestConstruction()");
-            return;
-        }
-
-        logln("for string_in_Root_te_te_IN, te_IN.txt had " + result2);
-
-        if (result2 != "TE_IN")
-            errln("Construction test failed; run verbose for more information");
-
-        delete[] wideDirectory;
-    }
 }
+
 void
 NewResourceBundleTest::TestIteration()
 {
@@ -351,7 +319,7 @@ NewResourceBundleTest::TestIteration()
                 if(U_SUCCESS(err)){
                     expected=element;
                     if(bundle.getSize() > 1){
-                        CONFIRM_EQ(bundle.getType(), RES_ARRAY);
+                        CONFIRM_EQ(bundle.getType(), URES_ARRAY);
                         expected+=itoa(row, buf);
                         ResourceBundle rowbundle=bundle.get(row, err);
                         if(!U_FAILURE(err) && rowbundle.getSize()>1){
@@ -370,7 +338,7 @@ NewResourceBundleTest::TestIteration()
                         }
                     }
                     else{
-                        CONFIRM_EQ(bundle.getType(), (int32_t)RES_STRING);
+                        CONFIRM_EQ(bundle.getType(), (int32_t)URES_STRING);
                     }
                 }
                 CONFIRM_EQ(got, expected);
@@ -409,11 +377,32 @@ NewResourceBundleTest::TestIteration()
     delete locale;
 }
 
+// TODO: add operator== and != to ResourceBundle
+static UBool
+equalRB(ResourceBundle &a, ResourceBundle &b) {
+    UResType type;
+    UErrorCode status;
+
+    type=a.getType();
+    status=U_ZERO_ERROR;
+    return
+        type==b.getType() &&
+        a.getLocale()==b.getLocale() &&
+        0==strcmp(a.getName(), b.getName()) &&
+        type==URES_STRING ?
+            a.getString(status)==b.getString(status) :
+            type==URES_INT ?
+                a.getInt(status)==b.getInt(status) :
+                TRUE;
+}
+
 void
 NewResourceBundleTest::TestOtherAPI(){
     UErrorCode   err = U_ZERO_ERROR;
     const char* testdatapath;
     testdatapath=loadTestData(err);
+    UnicodeString tDataPathUS = UnicodeString(testdatapath, "");
+
     if(U_FAILURE(err))
     {
         errln("Could not load testdata.dat %s " + UnicodeString(u_errorName(err)));
@@ -421,6 +410,12 @@ NewResourceBundleTest::TestOtherAPI(){
     }
     Locale       *locale=new Locale("te_IN");
 
+    ResourceBundle test0(tDataPathUS, *locale, err);
+    if(U_FAILURE(err)){
+        errln("Construction failed");
+        return;
+    }
+
     ResourceBundle  test1(testdatapath, *locale, err);
     if(U_FAILURE(err)){
         errln("Construction failed");
@@ -438,6 +433,7 @@ NewResourceBundleTest::TestOtherAPI(){
     ResourceBundle defaultresource(err);
     if(U_FAILURE(err)){
         errln("Construction of default resourcebundle failed");
+        return;
     }
     if(strcmp(defaultresource.getLocale().getName(), Locale::getDefault().getName()) != 0){
         errln("Construction of default resourcebundle didn't take the defaultlocale\n");
@@ -459,7 +455,19 @@ NewResourceBundleTest::TestOtherAPI(){
         errln("copy construction for subresource failed\n");
     }
 
+    ResourceBundle *p;
 
+    p = defaultresource.clone();
+    if(p == &defaultresource || !equalRB(*p, defaultresource)) {
+        errln("ResourceBundle.clone() failed");
+    }
+    delete p;
+
+    p = defaultSub.clone();
+    if(p == &defaultSub || !equalRB(*p, defaultSub)) {
+        errln("2nd ResourceBundle.clone() failed");
+    }
+    delete p;
 
     UVersionInfo ver;
     copyRes.getVersion(ver);
@@ -516,7 +524,7 @@ NewResourceBundleTest::TestOtherAPI(){
                   if(U_SUCCESS(err)){
                       expected=element;
                       if(ures_getSize(bundle) > 1){
-                          CONFIRM_EQ(ures_getType(bundle), RES_ARRAY);
+                          CONFIRM_EQ(ures_getType(bundle), URES_ARRAY);
                           expected+=itoa(row, buf);
                           rowbundle=ures_getByIndex(bundle, row, rowbundle, &err);
                           if(!U_FAILURE(err) && ures_getSize(rowbundle)>1){
@@ -540,7 +548,7 @@ NewResourceBundleTest::TestOtherAPI(){
                           }
                       }
                       else{
-                          CONFIRM_EQ(ures_getType(bundle), (int32_t)RES_STRING);
+                          CONFIRM_EQ(ures_getType(bundle), (int32_t)URES_STRING);
                       }
                   }
                   CONFIRM_EQ(got, expected);
@@ -631,9 +639,9 @@ NewResourceBundleTest::testTag(const char* frag,
                 if(j == actual_bundle) /* it's in the same bundle OR it's a nonexistent=default bundle (5) */
                   expected_resource_status = U_ZERO_ERROR;
                 else if(j == 0)
-                  expected_resource_status = U_USING_DEFAULT_ERROR;
+                  expected_resource_status = U_USING_DEFAULT_WARNING;
                 else
-                  expected_resource_status = U_USING_FALLBACK_ERROR;
+                  expected_resource_status = U_USING_FALLBACK_WARNING;
                 
                 break;
             }
@@ -699,7 +707,7 @@ NewResourceBundleTest::testTag(const char* frag,
         {
             //confirm the resource type is an array
             UResType bundleType=array.getType();
-            CONFIRM_EQ(bundleType, RES_ARRAY);
+            CONFIRM_EQ(bundleType, URES_ARRAY);
 
             count=array.getSize();
             CONFIRM_GE(count,1);
@@ -774,7 +782,7 @@ NewResourceBundleTest::testTag(const char* frag,
         {
             //confirm the resource type is an 2darray
             UResType bundleType=array2d.getType();
-            CONFIRM_EQ(bundleType, RES_ARRAY);
+            CONFIRM_EQ(bundleType, URES_ARRAY);
 
             row_count=array2d.getSize();
             CONFIRM_GE(row_count,1);
@@ -785,7 +793,7 @@ NewResourceBundleTest::testTag(const char* frag,
                 if(U_SUCCESS(status)){
                     //confirm the resourcetype of each table row is an array
                     UResType rowType=tablerow.getType();
-                    CONFIRM_EQ(rowType, RES_ARRAY);
+                    CONFIRM_EQ(rowType, URES_ARRAY);
 
                     column_count=tablerow.getSize();
                     CONFIRM_GE(column_count,1);
@@ -861,7 +869,7 @@ NewResourceBundleTest::testTag(const char* frag,
 
         if (U_SUCCESS(status)) {
             UResType bundleType=tags.getType();
-            CONFIRM_EQ(bundleType, RES_TABLE);
+            CONFIRM_EQ(bundleType, URES_TABLE);
 
             tag_count=tags.getSize();
             CONFIRM_GE((int32_t)tag_count, (int32_t)0); 
@@ -921,7 +929,7 @@ NewResourceBundleTest::testTag(const char* frag,
                 ResourceBundle tagelement=tags.get(item_tag, status);
                 if(!U_FAILURE(status)){
                     UResType elementType=tagelement.getType();
-                    CONFIRM_EQ(elementType, (int32_t)RES_STRING);
+                    CONFIRM_EQ(elementType, (int32_t)URES_STRING);
                     const char* key=tagelement.getKey();
                     CONFIRM_EQ((UnicodeString)key, (UnicodeString)item_tag);
                     UnicodeString t=tagelement.getString(status);
@@ -960,5 +968,184 @@ NewResourceBundleTest::record_fail()
   err();
   ++fail;
 }
+
+
+void
+NewResourceBundleTest::TestNewTypes() {
+    char action[256];
+    const char* testdatapath;
+    UErrorCode status = U_ZERO_ERROR;
+    uint8_t *binResult = NULL;
+    int32_t len = 0;
+    int32_t i = 0;
+    int32_t intResult = 0;
+    uint32_t uintResult = 0;
+    UChar expected[] = { 'a','b','c','\0','d','e','f' };
+    const char* expect ="tab:\t cr:\r ff:\f newline:\n backslash:\\\\ quote=\\\' doubleQuote=\\\" singlequoutes=''";
+    UChar uExpect[200];
+
+    testdatapath=loadTestData(status);
+
+    if(U_FAILURE(status))
+    {
+        logln("Could not load testdata.dat %s \n",u_errorName(status));
+        return;
+    }
+
+    ResourceBundle theBundle(testdatapath, "testtypes", status);
+    ResourceBundle bundle(testdatapath, Locale("te_IN"),status);
+
+    UnicodeString emptyStr = theBundle.getStringEx("emptystring", status);
+    if(!emptyStr.length()==0) {
+      logln("Empty string returned invalid value\n");
+    }
+
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+
+    /* This test reads the string "abc\u0000def" from the bundle   */
+    /* if everything is working correctly, the size of this string */
+    /* should be 7. Everything else is a wrong answer, esp. 3 and 6*/
+
+    strcpy(action, "getting and testing of string with embeded zero");
+    ResourceBundle res = theBundle.get("zerotest", status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_STRING);
+    UnicodeString zeroString=res.getString(status);
+    len = zeroString.length();
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(len, 7);
+        CONFIRM_NE(len, 3);
+    }
+    for(i=0;i<len;i++){
+        if(zeroString[i]!= expected[i]){
+            logln("Output didnot match Expected: \\u%4X Got: \\u%4X", expected[i], zeroString[i]);
+        }
+    }
+
+    strcpy(action, "getting and testing of binary type");
+    res = theBundle.get("binarytest", status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_BINARY);
+    binResult=(uint8_t*)res.getBinary(len, status);
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(len, 15);
+        for(i = 0; i<15; i++) {
+            CONFIRM_EQ(binResult[i], i);
+        }
+    }
+
+    strcpy(action, "getting and testing of imported binary type");
+    res = theBundle.get("importtest",status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_BINARY);
+    binResult=(uint8_t*)res.getBinary(len, status);
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(len, 15);
+        for(i = 0; i<15; i++) {
+            CONFIRM_EQ(binResult[i], i);
+        }
+    }
+
+    strcpy(action, "getting and testing of integer types");
+    res = theBundle.get("one",  status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_INT);
+    intResult=res.getInt(status);
+    uintResult = res.getUInt(status);
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(uintResult, (uint32_t)intResult);
+        CONFIRM_EQ(intResult, 1);
+    }
+
+    strcpy(action, "getting minusone");
+    res = theBundle.get((const char*)"minusone", status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_INT);
+    intResult=res.getInt(status);
+    uintResult = res.getUInt(status);
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(uintResult, 0x0FFFFFFF); /* a 28 bit integer */
+        CONFIRM_EQ(intResult, -1);
+        CONFIRM_NE(uintResult, (uint32_t)intResult);
+    }
+
+    strcpy(action, "getting plusone");
+    res = theBundle.get("plusone",status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_INT);
+    intResult=res.getInt(status);
+    uintResult = res.getUInt(status);
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(uintResult, (uint32_t)intResult);
+        CONFIRM_EQ(intResult, 1);
+    }
+
+    res = theBundle.get("onehundredtwentythree",status);
+    CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+    CONFIRM_EQ(res.getType(), URES_INT);
+    intResult=res.getInt(status);
+    if(U_SUCCESS(status)){
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        CONFIRM_EQ(intResult, 123);
+    }
+
+    /* this tests if escapes are preserved or not */
+    {
+        UnicodeString str = theBundle.getStringEx("testescape",status);
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        if(U_SUCCESS(status)){
+            u_charsToUChars(expect,uExpect,(int32_t)uprv_strlen(expect)+1);
+            if(str.compare(uExpect)!=0){
+                errln("Did not get the expected string for testescape expected. Expected : " 
+                    +UnicodeString(uExpect )+ " Got: " + str);
+            }
+        }
+    }
+    /* test for jitterbug#1435 */
+    {
+        UnicodeString str = theBundle.getStringEx("test_underscores",status);
+        expect ="test message ....";
+        CONFIRM_UErrorCode(status, U_ZERO_ERROR);
+        u_charsToUChars(expect,uExpect,(int32_t)uprv_strlen(expect)+1);
+        if(str.compare(uExpect)!=0){
+            errln("Did not get the expected string for test_underscores.\n");
+        }
+    }
+
+
+}
+
+void
+NewResourceBundleTest::TestGetByFallback() {
+    UErrorCode status = U_ZERO_ERROR;
+
+    ResourceBundle heRes(NULL, "he", status);
+
+    heRes.getWithFallback("calendar", status).getWithFallback("islamic-civil", status).getWithFallback("DateTime", status);
+    if(U_SUCCESS(status)) {
+        errln("he locale's Islamic-civil DateTime resource exists. How did it get here?\n");
+    }
+    status = U_ZERO_ERROR;
+
+    heRes.getWithFallback("calendar", status).getWithFallback("islamic-civil", status).getWithFallback("eras", status);
+    if(U_FAILURE(status)) {
+        errln("Didn't get Islamic Eras. I know they are there!\n");
+    }
+    status = U_ZERO_ERROR;
+
+    ResourceBundle rootRes(NULL, "root", status);
+    rootRes.getWithFallback("calendar", status).getWithFallback("islamic-civil", status).getWithFallback("DateTime", status);
+    if(U_SUCCESS(status)) {
+        errln("Root's Islamic-civil's DateTime resource exists. How did it get here?\n");
+    }
+    status = U_ZERO_ERROR;
+
+}
 //eof
 
