@@ -1,15 +1,20 @@
 /*
 *****************************************************************************
-* Copyright (C) 2001, International Business Machines orporation and others.  
-* All Rights Reserved.
+* Copyright (C) 2001-2006, International Business Machines orporation  
+* and others. All Rights Reserved.
 ****************************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "srchtest.h"
 #include "../cintltst/usrchdat.c"
 #include "unicode/stsearch.h"
 #include "unicode/ustring.h"
 #include "unicode/schriter.h"
 #include <string.h>
+#include <stdio.h>
 
 // private definitions -----------------------------------------------------
 
@@ -19,13 +24,18 @@
         if (exec) {                   \
             logln(#test "---");       \
             logln((UnicodeString)""); \
-            test();                   \
+            if(areBroken) {           \
+                  errln(__FILE__ " cannot test - failed to create collator.");  \
+            } else {                  \
+                test();               \
+            }                         \
         }                             \
         break;
 
 // public contructors and destructors --------------------------------------
 
-StringSearchTest::StringSearchTest() 
+StringSearchTest::StringSearchTest() :
+    m_en_wordbreaker_(NULL), m_en_characterbreaker_(NULL)
 {
     UErrorCode    status = U_ZERO_ERROR;
     
@@ -33,6 +43,19 @@ StringSearchTest::StringSearchTest()
     m_fr_fr_ = (RuleBasedCollator *)Collator::createInstance("fr_FR", status);
     m_de_    = (RuleBasedCollator *)Collator::createInstance("de_DE", status);
     m_es_    = (RuleBasedCollator *)Collator::createInstance("es_ES", status);
+    if(U_FAILURE(status)) {
+      delete m_en_us_;
+      delete m_fr_fr_;
+      delete m_de_;
+      delete m_es_;
+      m_en_us_ = 0;
+      m_fr_fr_ = 0;
+      m_de_ = 0;
+      m_es_ = 0;
+      errln("Collator creation failed with %s", u_errorName(status));
+      return;
+    }
+
     
     UnicodeString rules;
     rules.setTo(((RuleBasedCollator *)m_de_)->getRules());
@@ -50,10 +73,12 @@ StringSearchTest::StringSearchTest()
 
     m_es_ = new RuleBasedCollator(rules, status); 
         
+#if !UCONFIG_NO_BREAK_ITERATION
     m_en_wordbreaker_      = BreakIterator::createWordInstance(
-                                                    Locale::ENGLISH, status);
+                                                    Locale::getEnglish(), status);
     m_en_characterbreaker_ = BreakIterator::createCharacterInstance(
-                                                    Locale::ENGLISH, status);
+                                                    Locale::getEnglish(), status);
+#endif
 }
 
 StringSearchTest::~StringSearchTest() 
@@ -62,8 +87,10 @@ StringSearchTest::~StringSearchTest()
     delete m_fr_fr_;
     delete m_de_;
     delete m_es_;
+#if !UCONFIG_NO_BREAK_ITERATION
     delete m_en_wordbreaker_;
     delete m_en_characterbreaker_;
+#endif
 }
 
 // public methods ----------------------------------------------------------
@@ -71,12 +98,11 @@ StringSearchTest::~StringSearchTest()
 void StringSearchTest::runIndexedTest(int32_t index, UBool exec, 
                                       const char* &name, char* ) 
 {
+    UBool areBroken = FALSE;
     if (m_en_us_ == NULL && m_fr_fr_ == NULL && m_de_ == NULL &&
         m_es_ == NULL && m_en_wordbreaker_ == NULL &&
         m_en_characterbreaker_ == NULL && exec) {
-      errln(__FILE__ " cannot test - failed to create collator.");
-      name = "";
-      return;
+        areBroken = TRUE;
     }
 
     switch (index) {
@@ -85,7 +111,13 @@ void StringSearchTest::runIndexedTest(int32_t index, UBool exec,
         CASE(2, TestBasic)
         CASE(3, TestNormExact)
         CASE(4, TestStrength)
+#if UCONFIG_NO_BREAK_ITERATION
+    case 5:
+        name = "TestBreakIterator";
+        break;
+#else
         CASE(5, TestBreakIterator)
+#endif
         CASE(6, TestVariable)
         CASE(7, TestOverlap)
         CASE(8, TestCollator)
@@ -103,7 +135,13 @@ void StringSearchTest::runIndexedTest(int32_t index, UBool exec,
         CASE(20, TestCanonical)
         CASE(21, TestNormCanonical)
         CASE(22, TestStrengthCanonical)
+#if UCONFIG_NO_BREAK_ITERATION
+    case 23:
+        name = "TestBreakIteratorCanonical";
+        break;
+#else
         CASE(23, TestBreakIteratorCanonical)
+#endif
         CASE(24, TestVariableCanonical)
         CASE(25, TestOverlapCanonical)
         CASE(26, TestCollatorCanonical)
@@ -113,7 +151,9 @@ void StringSearchTest::runIndexedTest(int32_t index, UBool exec,
         CASE(30, TestGetSetOffsetCanonical)
         CASE(31, TestSupplementaryCanonical)
         CASE(32, TestContractionCanonical)
-        CASE(33, TestSearchIterator)
+        CASE(33, TestUClassID)
+        CASE(34, TestSubclass)
+        CASE(35, TestCoverage)
         default: name = ""; break;
     }
 }
@@ -141,6 +181,9 @@ RuleBasedCollator * StringSearchTest::getCollator(const char *collator)
  
 BreakIterator * StringSearchTest::getBreakIterator(const char *breaker)
 {
+#if UCONFIG_NO_BREAK_ITERATION
+    return NULL;
+#else
     if (breaker == NULL) {
         return NULL;
     }
@@ -150,37 +193,24 @@ BreakIterator * StringSearchTest::getBreakIterator(const char *breaker)
     else {
         return m_en_characterbreaker_;
     }
+#endif
 }
 
 char * StringSearchTest::toCharString(const UnicodeString &text)
 {
-           UChar  unichars[512];
     static char   result[1024];
-           int    count  = 0;
            int    index  = 0;
+           int    count  = 0;
            int    length = text.length();
 
-    text.extract(0, text.length(), unichars, 0);
-
     for (; count < length; count ++) {
-        UChar ch = unichars[count];
+        UChar ch = text[count];
         if (ch >= 0x20 && ch <= 0x7e) {
             result[index ++] = (char)ch;
         }
         else {
-            char digit[5];
-            int  zerosize;
-            result[index ++] = '\\';
-            result[index ++] = 'u';
-            sprintf(digit, "%x", ch);
-            zerosize = 4 - strlen(digit);
-            while (zerosize != 0) {
-                result[index ++] = '0';
-                zerosize --;
-            }
-            result[index] = 0;
-            strcat(result, digit);
-            index += strlen(digit);
+            sprintf(result+index, "\\u%04x", ch);
+            index += 6; /* \uxxxx */
         }
     }
     result[index] = 0;
@@ -305,9 +335,14 @@ UBool StringSearchTest::assertEqual(const SearchData *search)
     
     Collator      *collator = getCollator(search->collator);
     BreakIterator *breaker  = getBreakIterator(search->breaker);
-    StringSearch  *strsrch; 
+    StringSearch  *strsrch, *strsrch2;
     UChar          temp[128];
     
+#if UCONFIG_NO_BREAK_ITERATION
+    if(search->breaker) {
+      return TRUE; /* skip test */
+    }
+#endif
     u_unescape(search->text, temp, 128);
     UnicodeString text;
     text.setTo(temp);
@@ -315,9 +350,11 @@ UBool StringSearchTest::assertEqual(const SearchData *search)
     UnicodeString  pattern;
     pattern.setTo(temp);
 
+#if !UCONFIG_NO_BREAK_ITERATION
     if (breaker != NULL) {
         breaker->setText(text);
     }
+#endif
     collator->setStrength(getECollationStrength(search->strength));
     strsrch = new StringSearch(pattern, text, (RuleBasedCollator *)collator, 
                                breaker, status);
@@ -331,6 +368,20 @@ UBool StringSearchTest::assertEqual(const SearchData *search)
         delete strsrch;
         return FALSE;
     }
+
+
+    strsrch2 = strsrch->clone();
+    if( strsrch2 == strsrch || *strsrch2 != *strsrch ||
+        !assertEqualWithStringSearch(strsrch2, search)
+    ) {
+        errln("failure with StringSearch.clone()");
+        collator->setStrength(getECollationStrength(UCOL_TERTIARY));
+        delete strsrch;
+        delete strsrch2;
+        return FALSE;
+    }
+    delete strsrch2;
+
     collator->setStrength(getECollationStrength(UCOL_TERTIARY));
     delete strsrch;
     return TRUE;
@@ -344,6 +395,12 @@ UBool StringSearchTest::assertCanonicalEqual(const SearchData *search)
     StringSearch  *strsrch; 
     UChar          temp[128];
     
+#if UCONFIG_NO_BREAK_ITERATION
+    if(search->breaker) {
+      return TRUE; /* skip test */
+    }
+#endif
+
     u_unescape(search->text, temp, 128);
     UnicodeString text;
     text.setTo(temp);
@@ -351,9 +408,11 @@ UBool StringSearchTest::assertCanonicalEqual(const SearchData *search)
     UnicodeString  pattern;
     pattern.setTo(temp);
 
+#if !UCONFIG_NO_BREAK_ITERATION
     if (breaker != NULL) {
         breaker->setText(text);
     }
+#endif
     collator->setStrength(getECollationStrength(search->strength));
     strsrch = new StringSearch(pattern, text, (RuleBasedCollator *)collator, 
                                breaker, status);
@@ -383,6 +442,13 @@ UBool StringSearchTest::assertEqualWithAttribute(const SearchData *search,
     StringSearch  *strsrch; 
     UChar          temp[128];
     
+
+#if UCONFIG_NO_BREAK_ITERATION
+    if(search->breaker) {
+      return TRUE; /* skip test */
+    }
+#endif
+
     u_unescape(search->text, temp, 128);
     UnicodeString text;
     text.setTo(temp);
@@ -390,9 +456,11 @@ UBool StringSearchTest::assertEqualWithAttribute(const SearchData *search,
     UnicodeString  pattern;
     pattern.setTo(temp);
 
+#if !UCONFIG_NO_BREAK_ITERATION
     if (breaker != NULL) {
         breaker->setText(text);
     }
+#endif
     collator->setStrength(getECollationStrength(search->strength));
     strsrch = new StringSearch(pattern, text, (RuleBasedCollator *)collator, 
                                breaker, status);
@@ -503,14 +571,14 @@ void StringSearchTest::TestOpenClose()
     delete result;
 
     status = U_ZERO_ERROR;
-    result = new StringSearch(pattern, text, Locale::ENGLISH, NULL, status);
+    result = new StringSearch(pattern, text, Locale::getEnglish(), NULL, status);
     if (U_FAILURE(status) || result == NULL) {
         errln("Error: NULL break iterator is valid for opening search");
     }
     delete result;
 
     status = U_ZERO_ERROR;
-    result = new StringSearch(pattern, chariter, Locale::ENGLISH, NULL, status);
+    result = new StringSearch(pattern, chariter, Locale::getEnglish(), NULL, status);
     if (U_FAILURE(status)) {
         errln("Error: NULL break iterator is valid for opening search");
     }
@@ -538,6 +606,7 @@ void StringSearchTest::TestInitialization()
     UnicodeString  text;
     UnicodeString  temp("a");
     StringSearch  *result;
+    int count;
 
     /* simple test on the pattern ce construction */
     pattern.append(temp);
@@ -577,7 +646,7 @@ void StringSearchTest::TestInitialization()
     delete result;
     
     /* testing if an extremely large pattern will fail the initialization */
-    for (int count = 0; count < 512; count ++) {
+    for (count = 0; count < 512; count ++) {
         pattern.append(temp);
     }
     result = new StringSearch(pattern, text, m_en_us_, NULL, status);
@@ -660,6 +729,8 @@ void StringSearchTest::TestStrength()
     }
 }
  
+#if !UCONFIG_NO_BREAK_ITERATION
+
 void StringSearchTest::TestBreakIterator()
 {
     UChar temp[128];
@@ -751,6 +822,8 @@ void StringSearchTest::TestBreakIterator()
          count ++;
     }
 }
+
+#endif
     
 void StringSearchTest::TestVariable()
 {
@@ -912,6 +985,7 @@ void StringSearchTest::TestPattern()
 {
           
     UChar temp[512];
+    int templength;
     u_unescape(PATTERN[0].text, temp, 512);
     UnicodeString text;
     text.setTo(temp, u_strlen(temp));
@@ -989,7 +1063,7 @@ void StringSearchTest::TestPattern()
         return;
     }
     /* enormous pattern size to see if this crashes */
-    for (int templength = 0; templength != 512; templength ++) {
+    for (templength = 0; templength != 512; templength ++) {
         temp[templength] = 0x61;
     }
     temp[511] = 0;
@@ -1103,6 +1177,9 @@ void StringSearchTest::TestGetSetOffset()
         pattern.setTo(temp, u_strlen(temp));
         strsrch->setText(text, status);
         strsrch->setPattern(pattern, status);
+        strsrch->getCollator()->setStrength(getECollationStrength(
+                                                          search.strength));
+        strsrch->reset();
 
         int count = 0;
         int32_t matchindex  = search.offset[count];
@@ -1144,6 +1221,8 @@ void StringSearchTest::TestGetSetOffset()
             return;
         }
     }
+    strsrch->getCollator()->setStrength(getECollationStrength(
+                                                             UCOL_TERTIARY));
     delete strsrch;
 }
  
@@ -1396,7 +1475,7 @@ void StringSearchTest::TestContraction()
     UnicodeString rules;
     rules.setTo(temp, u_strlen(temp));
     RuleBasedCollator *collator = new RuleBasedCollator(rules, 
-        getECollationStrength(UCOL_TERTIARY), Normalizer::DECOMP, status); 
+        getECollationStrength(UCOL_TERTIARY), UCOL_ON, status); 
     if (U_FAILURE(status)) {
         errln("Error opening collator %s", u_errorName(status));
     }
@@ -1435,7 +1514,7 @@ void StringSearchTest::TestIgnorable()
     int        count  = 0;
     RuleBasedCollator *collator = new RuleBasedCollator(rules,  
                             getECollationStrength(IGNORABLE[count].strength), 
-                            Normalizer::DECOMP, status);  
+                            UCOL_ON, status);  
     if (U_FAILURE(status)) {
         errln("Error opening collator %s", u_errorName(status));
         return;
@@ -1502,6 +1581,8 @@ void StringSearchTest::TestStrengthCanonical()
     }
 }
     
+#if !UCONFIG_NO_BREAK_ITERATION
+
 void StringSearchTest::TestBreakIteratorCanonical()
 {
     UErrorCode status = U_ZERO_ERROR;
@@ -1542,6 +1623,10 @@ void StringSearchTest::TestBreakIteratorCanonical()
         }
         search  = &(BREAKITERATOREXACT[count + 1]);
         breaker = getBreakIterator(search->breaker);
+        if (breaker == NULL) {
+            errln("Error creating BreakIterator");
+            return;
+        }
         breaker->setText(strsrch->getText());
         strsrch->setBreakIterator(breaker, status);
         if (U_FAILURE(status) || strsrch->getBreakIterator() != breaker) {
@@ -1567,6 +1652,8 @@ void StringSearchTest::TestBreakIteratorCanonical()
          count ++;
     }
 }
+
+#endif
     
 void StringSearchTest::TestVariableCanonical()
 {
@@ -1676,7 +1763,7 @@ void StringSearchTest::TestCollatorCanonical()
     rules.setTo(temp, u_strlen(temp));
     RuleBasedCollator *tailored = new RuleBasedCollator(rules, 
         getECollationStrength(COLLATORCANONICAL[1].strength), 
-        Normalizer::DECOMP, status);
+        UCOL_ON, status);
 
     if (U_FAILURE(status)) {
         errln("Error opening rule based collator %s", u_errorName(status));
@@ -1947,7 +2034,7 @@ void StringSearchTest::TestContractionCanonical()
 
     UErrorCode         status   = U_ZERO_ERROR;
     RuleBasedCollator *collator = new RuleBasedCollator(rules, 
-        getECollationStrength(UCOL_TERTIARY), Normalizer::DECOMP, status); 
+        getECollationStrength(UCOL_TERTIARY), UCOL_ON, status); 
     if (U_FAILURE(status)) {
         errln("Error opening collator %s", u_errorName(status));
     }
@@ -1978,105 +2065,248 @@ void StringSearchTest::TestContractionCanonical()
     delete collator;
 }
 
-class TempSearch : public SearchIterator
+void StringSearchTest::TestUClassID()
+{
+    char id = *((char *)StringSearch::getStaticClassID());
+    if (id != 0) {
+        errln("Static class id for StringSearch should be 0");
+    }
+    UErrorCode     status    = U_ZERO_ERROR;
+    UnicodeString  text("text");
+    UnicodeString  pattern("pattern");
+    StringSearch  *strsrch = new StringSearch(pattern, text, m_en_us_, NULL, 
+                                              status);
+    id = *((char *)strsrch->getDynamicClassID());
+    if (id != 0) {
+        errln("Dynamic class id for StringSearch should be 0");
+    }
+    delete strsrch;
+}
+
+class TestSearch : public SearchIterator
 {
 public:
-    TempSearch();
-    TempSearch(TempSearch &search);
-    ~TempSearch();
-    void            setOffset(int32_t position, UErrorCode &status);
+    TestSearch(const TestSearch &obj);
+    TestSearch(const UnicodeString &text, 
+               BreakIterator *breakiter,
+               const UnicodeString &pattern);
+    ~TestSearch();
+
+    void        setOffset(int32_t position, UErrorCode &status);
     int32_t     getOffset() const;
     SearchIterator* safeClone() const;
+
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @draft ICU 2.2
+     */
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @draft ICU 2.2
+     */
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
+
+    UBool operator!=(const TestSearch &that) const;
+
+    UnicodeString m_pattern_;
+
 protected:
-    int32_t     handleNext(int32_t position, UErrorCode &status);
-    int32_t     handlePrev(int32_t position, UErrorCode &status);
+    int32_t      handleNext(int32_t position, UErrorCode &status);
+    int32_t      handlePrev(int32_t position, UErrorCode &status);
+    TestSearch & operator=(const TestSearch &that);
+
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
+    uint32_t m_offset_;
 };
 
-TempSearch::TempSearch() : SearchIterator()
+const char TestSearch::fgClassID=0;
+
+TestSearch::TestSearch(const TestSearch &obj) : SearchIterator(obj)
+{
+    m_offset_ = obj.m_offset_;
+    m_pattern_ = obj.m_pattern_;
+}
+
+TestSearch::TestSearch(const UnicodeString &text, 
+                       BreakIterator *breakiter,
+                       const UnicodeString &pattern) : SearchIterator()
+{
+    m_breakiterator_ = breakiter;
+    m_pattern_ = pattern;
+    m_text_ = text;
+    m_offset_ = 0;
+    m_pattern_ = pattern;
+}
+
+TestSearch::~TestSearch()
 {
 }
 
-TempSearch::TempSearch(TempSearch &search) : SearchIterator(search) 
-{
-}
 
-TempSearch::~TempSearch()
+void TestSearch::setOffset(int32_t position, UErrorCode &status)
 {
-}
-
-void TempSearch::setOffset(int32_t /*position*/, UErrorCode &/*status*/)
-{
-}
-
-int32_t TempSearch::getOffset() const
-{
-    return USEARCH_DONE;
-}
-
-SearchIterator * TempSearch::safeClone() const 
-{
-    return NULL;
-}
-
-int32_t TempSearch::handleNext(int32_t /*position*/, UErrorCode &/*status*/)
-{
-    return USEARCH_DONE;
-}
-
-int32_t TempSearch::handlePrev(int32_t /*position*/, UErrorCode &/*status*/)
-{
-    return USEARCH_DONE;
-}
-
-void StringSearchTest::TestSearchIterator()
-{
-    TempSearch search;
-    if (search.getBreakIterator() != NULL || 
-        search.getAttribute(USEARCH_OVERLAP) != USEARCH_OFF || 
-        search.getAttribute(USEARCH_CANONICAL_MATCH) != USEARCH_OFF ||
-        search.getMatchedStart() != USEARCH_DONE ||
-        search.getMatchedLength() != 0 || search.getText().length() != 0) {
-        errln("Error subclassing SearchIterator, default constructor failed");
-        return;
+    if (position >= 0 && position <= m_text_.length()) {
+        m_offset_ = position;
     }
-    if (search.getAttribute(USEARCH_ATTRIBUTE_COUNT) != USEARCH_DEFAULT) {
-        errln("Error getting illegal attribute failed");
-        return;
+    else {
+        status = U_INDEX_OUTOFBOUNDS_ERROR;
     }
-    UnicodeString           text("abc");
-    StringCharacterIterator striter(text);
-    UErrorCode              status = U_ZERO_ERROR;
+}
+
+int32_t TestSearch::getOffset() const
+{
+    return m_offset_;
+}
+
+SearchIterator * TestSearch::safeClone() const 
+{
+    return new TestSearch(m_text_, m_breakiterator_, m_pattern_);
+}
+
+UBool TestSearch::operator!=(const TestSearch &that) const
+{
+    if (SearchIterator::operator !=(that)) {
+        return FALSE;
+    }
+    return m_offset_ != that.m_offset_ || m_pattern_ != that.m_pattern_;
+}
+
+int32_t TestSearch::handleNext(int32_t start, UErrorCode &status)
+{
+  if(U_SUCCESS(status)) {
+    int match = m_text_.indexOf(m_pattern_, start);
+    if (match < 0) {
+        m_offset_ = m_text_.length();
+        setMatchStart(m_offset_);
+        setMatchLength(0);
+        return USEARCH_DONE;
+    }
+    setMatchStart(match);
+    m_offset_ = match;
+    setMatchLength(m_pattern_.length());
+    return match;
+  } else {
+    return USEARCH_DONE;
+  }
+}
+
+int32_t TestSearch::handlePrev(int32_t start, UErrorCode &status)
+{
+  if(U_SUCCESS(status)) {
+    int match = m_text_.lastIndexOf(m_pattern_, 0, start);
+    if (match < 0) {
+        m_offset_ = 0;
+        setMatchStart(m_offset_);
+        setMatchLength(0);
+        return USEARCH_DONE;
+    }
+    setMatchStart(match);
+    m_offset_ = match;
+    setMatchLength(m_pattern_.length());
+    return match;
+  } else {
+    return USEARCH_DONE;
+  }
+}
+
+TestSearch & TestSearch::operator=(const TestSearch &that)
+{
+    SearchIterator::operator=(that);
+    m_offset_ = that.m_offset_;
+    m_pattern_ = that.m_pattern_;
+    return *this;
+}
+
+void StringSearchTest::TestSubclass()
+{
+    UnicodeString text("abc abcd abc");
+    UnicodeString pattern("abc");
+    TestSearch search(text, NULL, pattern);
+    TestSearch search2(search);
+    int expected[] = {0, 4, 9};
+    UErrorCode status = U_ZERO_ERROR; 
+    int i;
+    StringCharacterIterator chariter(text);
+
     search.setText(text, status);
-    TempSearch search2;
-    search2.setText(striter, status);
-    if (U_FAILURE(status) || search != search2) {
+    if (search.getText() != search2.getText()) {
         errln("Error setting text");
-        return;
     }
-    if (search != search) {
-        errln("Error: search object has to be equals to itself");
-        return;
-    }
-    TempSearch search3(search);
-    if (search != search3) {
-        errln("Error: search object has to be equals to its copy");
-        return;
-    }
-    search.setAttribute(USEARCH_OVERLAP, USEARCH_ON, status);
-    if (U_FAILURE(status) || 
-        search.getAttribute(USEARCH_OVERLAP) != USEARCH_ON) {
-        errln("Error setting overlap attribute");
+
+    search.setText(chariter, status);
+    if (search.getText() != search2.getText()) {
+        errln("Error setting text");
     }
+
     search.reset();
-    if (search.getAttribute(USEARCH_OVERLAP) != USEARCH_OFF) {
-        errln("Error resetting search");
+    // comparing constructors
+ 
+    for (i = 0; i < (int)(sizeof(expected) / sizeof(expected[0])); i ++) {
+        if (search.next(status) != expected[i]) {
+            errln("Error getting next match");
+        }
+        if (search.getMatchedLength() != search.m_pattern_.length()) {
+            errln("Error getting next match length");
+        }
     }
-    search2 = search3;
-    if (search2 != search3) {
-        errln("Error: search object has to be equals to its assignment copy");
-        return;
+    if (search.next(status) != USEARCH_DONE) {
+        errln("Error should have reached the end of the iteration");
+    }
+    for (i = sizeof(expected) / sizeof(expected[0]) - 1; i >= 0; i --) {
+        if (search.previous(status) != expected[i]) {
+            errln("Error getting previous match");
+        }
+        if (search.getMatchedLength() != search.m_pattern_.length()) {
+            errln("Error getting previous match length");
+        }
+    }
+    if (search.previous(status) != USEARCH_DONE) {
+        errln("Error should have reached the start of the iteration");
     }
 }
 
+class StubSearchIterator:public SearchIterator{
+public:
+    StubSearchIterator(){}
+    virtual void setOffset(int32_t , UErrorCode &) {};
+    virtual int32_t getOffset(void) const {return 0;};
+    virtual SearchIterator* safeClone(void) const {return NULL;};
+    virtual int32_t handleNext(int32_t , UErrorCode &){return 0;};
+    virtual int32_t handlePrev(int32_t , UErrorCode &) {return 0;};
+    virtual UClassID getDynamicClassID() const {
+        static char classID = 0;
+        return (UClassID)&classID; 
+    }
+};
 
+void StringSearchTest::TestCoverage(){
+    StubSearchIterator stub1, stub2;
+    UErrorCode status = U_ZERO_ERROR;
 
+    if (stub1 != stub2){
+        errln("new StubSearchIterator should be equal");
+    }
+
+    stub2.setText(UnicodeString("ABC"), status);
+    if (U_FAILURE(status)) {
+        errln("Error: SearchIterator::SetText");
+    }
+
+    stub1 = stub2;
+    if (stub1 != stub2){
+        errln("SearchIterator::operator =  assigned object should be equal");
+    }
+}
+
+#endif /* #if !UCONFIG_NO_COLLATION */
