@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001, International Business Machines
+*   Copyright (C) 1999-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -10,6 +10,10 @@
 #ifndef TRANSTST_H
 #define TRANSTST_H
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/translit.h"
 #include "intltest.h"
 
@@ -21,7 +25,7 @@ class TransliteratorTest : public IntlTest {
 
 public:
     TransliteratorTest();
-    virtual ~TransliteratorTest() {}
+    virtual ~TransliteratorTest();
 
 private:
     void runIndexedTest(int32_t index, UBool exec, const char* &name,
@@ -246,6 +250,8 @@ private:
     /* Test Compound Indic-Latin transliterators*/
     void TestCompoundLatinRT(void);
 
+    /* Test bindi and tippi for Gurmukhi */
+    void TestGurmukhiDevanagari(void);
     /**
      * Test instantiation from a locale.
      */
@@ -322,6 +328,37 @@ private:
 
     void TestInvalidBackRef(void);
 
+    void TestMulticharStringSet(void);
+
+    void TestUserFunction(void);
+
+    void TestAnyX(void);
+
+    void TestSourceTargetSet(void);
+
+    void TestRuleWhitespace(void);
+
+    void TestAllCodepoints(void);
+
+    void TestBoilerplate(void);
+
+    void TestAlternateSyntax(void);
+
+    /**
+     * Tests the multiple-pass syntax
+     */
+    void TestBeginEnd(void);
+
+    /**
+     * Tests that toRules() works right with the multiple-pass syntax
+     */
+    void TestBeginEndToRules(void);
+
+    /**
+     * Tests the registerAlias() function
+     */
+    void TestRegisterAlias(void);
+
     //======================================================================
     // Support methods
     //======================================================================
@@ -335,6 +372,12 @@ private:
                 const UnicodeString& expectedResult,
                 UTransPosition *pos=0);
 
+    void expect(const UnicodeString& id,
+                const UnicodeString& rules,
+                const UnicodeString& source,
+                const UnicodeString& expectedResult,
+                UTransPosition *pos=0);
+
     void expect(const Transliterator& t,
                 const UnicodeString& source,
                 const UnicodeString& expectedResult,
@@ -363,10 +406,14 @@ private:
     void CheckIncrementalAux(const Transliterator* t, 
                              const UnicodeString& input);
 
+    void reportParseError(const UnicodeString& message, const UParseError& parseError, const UErrorCode& status);
+
 
     const UnicodeString DESERET_DEE;
     const UnicodeString DESERET_dee;
 
 };
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 #endif
