@@ -1,7 +1,7 @@
 /*
 *****************************************************************************************
 *
-*   Copyright (C) 2004-2005, International Business Machines
+*   Copyright (C) 2004-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *****************************************************************************************
@@ -13,6 +13,9 @@
 #if !defined(U_STATIC_IMPLEMENTATION) && (defined(U_WINDOWS) || defined(U_CYGWIN))
     #define T_CTEST_EXPORT __declspec(dllexport)
     #define T_CTEST_IMPORT __declspec(dllimport)
+#elif defined(USE_GCC_VISIBILITY_ATTRIBUTE)
+    #define T_CTEST_EXPORT __attribute__((visibility("default")))
+    #define T_CTEST_IMPORT
 #else
     #define T_CTEST_EXPORT
     #define T_CTEST_IMPORT
