@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001-2002, International Business Machines
+*   Copyright (C) 2001-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -23,10 +23,13 @@
 #include <stdlib.h>
 #include "unicode/utypes.h"
 #include "unicode/uchar.h"
+#include "unicode/ustring.h"
 #include "unicode/putil.h"
+#include "unicode/uclean.h"
+#include "unicode/udata.h"
+#include "unicode/uset.h"
 #include "cmemory.h"
 #include "cstring.h"
-#include "unicode/udata.h"
 #include "unewdata.h"
 #include "uoptions.h"
 #include "uparse.h"
@@ -36,22 +39,31 @@ U_CDECL_BEGIN
 #include "gennorm.h"
 U_CDECL_END
 
-#ifdef WIN32
-#   pragma warning(disable: 4100)
-#endif
-
 UBool beVerbose=FALSE, haveCopyright=TRUE;
 
 /* prototypes --------------------------------------------------------------- */
 
 static void
-parseDerivedNormalizationProperties(const char *filename, UErrorCode *pErrorCode);
+parseDerivedNormalizationProperties(const char *filename, UErrorCode *pErrorCode, UBool reportError);
 
 static void
 parseDB(const char *filename, UErrorCode *pErrorCode);
 
 /* -------------------------------------------------------------------------- */
 
+enum {
+    HELP_H,
+    HELP_QUESTION_MARK,
+    VERBOSE,
+    COPYRIGHT,
+    DESTDIR,
+    SOURCEDIR,
+    UNICODE_VERSION,
+    ICUDATADIR,
+    CSOURCE,
+    STORE_FLAGS
+};
+
 static UOption options[]={
     UOPTION_HELP_H,
     UOPTION_HELP_QUESTION_MARK,
@@ -59,12 +71,17 @@ static UOption options[]={
     UOPTION_COPYRIGHT,
     UOPTION_DESTDIR,
     UOPTION_SOURCEDIR,
-    { "unicode", NULL, NULL, NULL, 'u', UOPT_REQUIRES_ARG, 0 }
+    UOPTION_DEF("unicode", 'u', UOPT_REQUIRES_ARG),
+    UOPTION_ICUDATADIR,
+    UOPTION_DEF("csource", 'C', UOPT_NO_ARG),
+    UOPTION_DEF("prune", 'p', UOPT_REQUIRES_ARG)
 };
 
 extern int
 main(int argc, char* argv[]) {
+#if !UCONFIG_NO_NORMALIZATION
     char filename[300];
+#endif
     const char *srcDir=NULL, *destDir=NULL, *suffix=NULL;
     char *basename=NULL;
     UErrorCode errorCode=U_ZERO_ERROR;
@@ -75,6 +92,7 @@ main(int argc, char* argv[]) {
     options[4].value=u_getDataDirectory();
     options[5].value="";
     options[6].value="3.0.0";
+    options[ICUDATADIR].value=u_getDataDirectory();
     argc=u_parseArgs(argc, argv, sizeof(options)/sizeof(options[0]), options);
 
     /* error handling, printing usage message */
@@ -92,7 +110,7 @@ main(int argc, char* argv[]) {
             "Usage: %s [-options] [suffix]\n"
             "\n"
             "Read the UnicodeData.txt file and other Unicode properties files and\n"
-            "create a binary file " DATA_NAME "." DATA_TYPE " with the normalization data\n"
+            "create a binary file " U_ICUDATA_NAME "_" DATA_NAME "." DATA_TYPE " with the normalization data\n"
             "\n",
             argv[0]);
         fprintf(stderr,
@@ -100,13 +118,28 @@ main(int argc, char* argv[]) {
             "\t-h or -? or --help  this usage text\n"
             "\t-v or --verbose     verbose output\n"
             "\t-c or --copyright   include a copyright notice\n"
-            "\t-u or --unicode     Unicode version, followed by the version like 3.0.0\n");
+            "\t-u or --unicode     Unicode version, followed by the version like 3.0.0\n"
+            "\t-C or --csource     generate a .c source file rather than the .icu binary\n");
+        fprintf(stderr,
+            "\t-p or --prune flags Prune for data modularization:\n"
+            "\t                    Determine what data is to be stored.\n"
+            "\t        0 (zero) stores minimal data (only for NFD)\n"
+            "\t        lowercase letters turn off data, uppercase turn on (use with 0)\n");
+        fprintf(stderr,
+            "\t        k: compatibility decompositions (NFKC, NFKD)\n"
+            "\t        c: composition data (NFC, NFKC)\n"
+            "\t        f: FCD data (will be generated at load time)\n"
+            "\t        a: auxiliary data (canonical closure etc.)\n"
+            "\t        x: exclusion sets (Unicode 3.2-level normalization)\n");
         fprintf(stderr,
             "\t-d or --destdir     destination directory, followed by the path\n"
             "\t-s or --sourcedir   source directory, followed by the path\n"
+            "\t-i or --icudatadir  directory for locating any needed intermediate data files,\n"
+            "\t                    followed by path, defaults to <%s>\n"
             "\tsuffix              suffix that is to be appended with a '-'\n"
             "\t                    to the source file basenames before opening;\n"
-            "\t                    'gennorm new' will read UnicodeData-new.txt etc.\n");
+            "\t                    'gennorm new' will read UnicodeData-new.txt etc.\n",
+            u_getDataDirectory());
         return argc<0 ? U_ILLEGAL_ARGUMENT_ERROR : U_ZERO_ERROR;
     }
 
@@ -122,8 +155,91 @@ main(int argc, char* argv[]) {
         suffix=NULL;
     }
 
+#if UCONFIG_NO_NORMALIZATION
+
+    fprintf(stderr,
+        "gennorm writes a dummy " U_ICUDATA_NAME "_" DATA_NAME "." DATA_TYPE
+        " because UCONFIG_NO_NORMALIZATION is set, \n"
+        "see icu/source/common/unicode/uconfig.h\n");
+    generateData(destDir, options[CSOURCE].doesOccur);
+
+#else
+
     setUnicodeVersion(options[6].value);
 
+    if (options[ICUDATADIR].doesOccur) {
+        u_setDataDirectory(options[ICUDATADIR].value);
+    }
+
+    if(options[STORE_FLAGS].doesOccur) {
+        const char *s=options[STORE_FLAGS].value;
+        char c;
+
+        while((c=*s++)!=0) {
+            switch(c) {
+            case '0':
+                gStoreFlags=0;  /* store minimal data (only for NFD) */
+                break;
+
+            /* lowercase letters: omit data */
+            case 'k':
+                gStoreFlags&=~U_MASK(UGENNORM_STORE_COMPAT);
+                break;
+            case 'c':
+                gStoreFlags&=~U_MASK(UGENNORM_STORE_COMPOSITION);
+                break;
+            case 'f':
+                gStoreFlags&=~U_MASK(UGENNORM_STORE_FCD);
+                break;
+            case 'a':
+                gStoreFlags&=~U_MASK(UGENNORM_STORE_AUX);
+                break;
+            case 'x':
+                gStoreFlags&=~U_MASK(UGENNORM_STORE_EXCLUSIONS);
+                break;
+
+            /* uppercase letters: include data (use with 0) */
+            case 'K':
+                gStoreFlags|=U_MASK(UGENNORM_STORE_COMPAT);
+                break;
+            case 'C':
+                gStoreFlags|=U_MASK(UGENNORM_STORE_COMPOSITION);
+                break;
+            case 'F':
+                gStoreFlags|=U_MASK(UGENNORM_STORE_FCD);
+                break;
+            case 'A':
+                gStoreFlags|=U_MASK(UGENNORM_STORE_AUX);
+                break;
+            case 'X':
+                gStoreFlags|=U_MASK(UGENNORM_STORE_EXCLUSIONS);
+                break;
+
+            default:
+                fprintf(stderr, "ignoring undefined prune flag '%c'\n", c);
+                break;
+            }
+        }
+    }
+
+    /*
+     * Verify that we can work with properties
+     * but don't call u_init() because that needs unorm.icu which we are just
+     * going to build here.
+     */
+    {
+        U_STRING_DECL(ideo, "[:Ideographic:]", 15);
+        USet *set;
+
+        U_STRING_INIT(ideo, "[:Ideographic:]", 15);
+        set=uset_openPattern(ideo, -1, &errorCode);
+        if(U_FAILURE(errorCode) || !uset_contains(set, 0xf900)) {
+            fprintf(stderr, "gennorm is unable to work with properties (uprops.icu): %s\n", u_errorName(errorCode));
+            exit(errorCode);
+        }
+        uset_close(set);
+    }
+
     /* prepare the filename beginning with the source dir */
     uprv_strcpy(filename, srcDir);
     basename=filename+uprv_strlen(filename);
@@ -134,16 +250,28 @@ main(int argc, char* argv[]) {
     /* initialize */
     init();
 
-    /* process DerivedNormalizationProperties.txt (quick check flags) */
+    /* process DerivedNormalizationProps.txt (name changed for Unicode 3.2, to <=31 characters) */
     if(suffix==NULL) {
-        uprv_strcpy(basename, "DerivedNormalizationProperties.txt");
+        uprv_strcpy(basename, "DerivedNormalizationProps.txt");
     } else {
-        uprv_strcpy(basename, "DerivedNormalizationProperties");
+        uprv_strcpy(basename, "DerivedNormalizationProps");
         basename[30]='-';
         uprv_strcpy(basename+31, suffix);
         uprv_strcat(basename+31, ".txt");
     }
-    parseDerivedNormalizationProperties(filename, &errorCode);
+    parseDerivedNormalizationProperties(filename, &errorCode, FALSE);
+    if(U_FAILURE(errorCode)) {
+        /* can be only U_FILE_ACCESS_ERROR - try filename from before Unicode 3.2 */
+        if(suffix==NULL) {
+            uprv_strcpy(basename, "DerivedNormalizationProperties.txt");
+        } else {
+            uprv_strcpy(basename, "DerivedNormalizationProperties");
+            basename[30]='-';
+            uprv_strcpy(basename+31, suffix);
+            uprv_strcat(basename+31, ".txt");
+        }
+        parseDerivedNormalizationProperties(filename, &errorCode, TRUE);
+    }
 
     /* process UnicodeData.txt */
     if(suffix==NULL) {
@@ -161,17 +289,21 @@ main(int argc, char* argv[]) {
         processData();
 
         /* write the properties data file */
-        generateData(destDir);
+        generateData(destDir, options[CSOURCE].doesOccur);
 
         cleanUpData();
     }
 
+#endif
+
     return errorCode;
 }
 
+#if !UCONFIG_NO_NORMALIZATION
+
 /* parser for DerivedNormalizationProperties.txt ---------------------------- */
 
-static void
+static void U_CALLCONV
 derivedNormalizationPropertiesLineFn(void *context,
                                      char *fields[][2], int32_t fieldCount,
                                      UErrorCode *pErrorCode) {
@@ -213,24 +345,42 @@ derivedNormalizationPropertiesLineFn(void *context,
             return;
         }
 
-        if(0==uprv_memcmp(s, "NO", 2)) {
+        if(0==uprv_strncmp(s, "NO", 2)) {
             qcFlags&=0xf;
-        } else if(0==uprv_memcmp(s, "MAYBE", 5)) {
+        } else if(0==uprv_strncmp(s, "MAYBE", 5)) {
             qcFlags&=0x30;
+        } else if(0==uprv_strncmp(s, "QC", 2) && *(s=(char *)u_skipWhitespace(s+2))==';') {
+            /*
+             * Unicode 4.0.1:
+             * changes single field "NFD_NO" -> two fields "NFD_QC; N" etc.
+             */
+            /* start of the field */
+            s=(char *)u_skipWhitespace(s+1);
+            if(*s=='N') {
+                qcFlags&=0xf;
+            } else if(*s=='M') {
+                qcFlags&=0x30;
+            } else {
+                return; /* do nothing for "Yes" because it's the default value */
+            }
         } else {
-            return;
+            return; /* do nothing for "Yes" because it's the default value */
         }
 
         /* set this flag for all code points in this range */
         while(start<=end) {
             setQCFlags(start++, qcFlags);
         }
-    } else if(0==uprv_memcmp(s, "Comp_Ex", 7)) {
+    } else if(0==uprv_memcmp(s, "Comp_Ex", 7) || 0==uprv_memcmp(s, "Full_Composition_Exclusion", 26)) {
         /* full composition exclusion */
         while(start<=end) {
             setCompositionExclusion(start++);
         }
-    } else if(0==uprv_memcmp(s, "FNC", 3) && *(s=(char *)u_skipWhitespace(s+3))==';') {
+    } else if(
+        ((0==uprv_memcmp(s, "FNC", 3) && *(s=(char *)u_skipWhitespace(s+3))==';') || 
+        (0==uprv_memcmp(s, "FC_NFKC", 7) && *(s=(char *)u_skipWhitespace(s+7))==';'))
+        
+    ) {
         /* FC_NFKC_Closure, parse field 2 to get the string */
         char *t;
 
@@ -253,7 +403,7 @@ derivedNormalizationPropertiesLineFn(void *context,
 }
 
 static void
-parseDerivedNormalizationProperties(const char *filename, UErrorCode *pErrorCode) {
+parseDerivedNormalizationProperties(const char *filename, UErrorCode *pErrorCode, UBool reportError) {
     char *fields[2][2];
 
     if(pErrorCode==NULL || U_FAILURE(*pErrorCode)) {
@@ -261,7 +411,7 @@ parseDerivedNormalizationProperties(const char *filename, UErrorCode *pErrorCode
     }
 
     u_parseDelimitedFile(filename, ';', fields, 2, derivedNormalizationPropertiesLineFn, NULL, pErrorCode);
-    if(U_FAILURE(*pErrorCode)) {
+    if(U_FAILURE(*pErrorCode) && (reportError || *pErrorCode!=U_FILE_ACCESS_ERROR)) {
         fprintf(stderr, "gennorm error: u_parseDelimitedFile(\"%s\") failed - %s\n", filename, u_errorName(*pErrorCode));
         exit(*pErrorCode);
     }
@@ -269,7 +419,7 @@ parseDerivedNormalizationProperties(const char *filename, UErrorCode *pErrorCode
 
 /* parser for UnicodeData.txt ----------------------------------------------- */
 
-static void
+static void U_CALLCONV
 unicodeDataLineFn(void *context,
                   char *fields[][2], int32_t fieldCount,
                   UErrorCode *pErrorCode) {
@@ -283,7 +433,7 @@ unicodeDataLineFn(void *context,
 
     /* ignore First and Last entries for ranges */
     if( *fields[1][0]=='<' &&
-        (length=(fields[1][1]-fields[1][0]))>=9 &&
+        (length=(int32_t)(fields[1][1]-fields[1][0]))>=9 &&
         (0==uprv_memcmp(", First>", fields[1][1]-8, 8) || 0==uprv_memcmp(", Last>", fields[1][1]-7, 7))
     ) {
         return;
@@ -292,6 +442,12 @@ unicodeDataLineFn(void *context,
     /* reset the properties */
     uprv_memset(&norm, 0, sizeof(Norm));
 
+    /*
+     * The combiningIndex must not be initialized to 0 because 0 is the
+     * combiningIndex of the first forward-combining character.
+     */
+    norm.combiningIndex=0xffff;
+
     /* get the character code, field 0 */
     code=(uint32_t)uprv_strtoul(fields[0][0], &end, 16);
     if(end<=fields[0][0] || end!=fields[0][1]) {
@@ -393,6 +549,8 @@ parseDB(const char *filename, UErrorCode *pErrorCode) {
     }
 }
 
+#endif /* #if !UCONFIG_NO_NORMALIZATION */
+
 /*
  * Hey, Emacs, please set the following:
  *
