@@ -1,6 +1,6 @@
 /**************************************************************************
 *
-*   Copyright (C) 2000, International Business Machines
+*   Copyright (C) 2000-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ***************************************************************************
@@ -18,6 +18,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "unicode/utypes.h"
+#include "unicode/putil.h"
 #include "cmemory.h"
 #include "cstring.h"
 #include "pkgtypes.h"
@@ -31,8 +32,9 @@ const char *pkg_writeCharListWrap(FileStream *s, CharList *l, const char *delim,
     {
         if(l->str)
         {
-            uprv_strcpy(buffer, l->str);
-            
+            uprv_strncpy(buffer, l->str, 1020);
+            buffer[1019]=0;
+
             if(quote < 0) { /* remove quotes */
                 if(buffer[uprv_strlen(buffer)-1] == '"') {
                     buffer[uprv_strlen(buffer)-1] = '\0';
@@ -43,25 +45,24 @@ const char *pkg_writeCharListWrap(FileStream *s, CharList *l, const char *delim,
             } else if(quote > 0) { /* add quotes */
                 if(l->str[0] != '"') {
                     uprv_strcpy(buffer, "\"");
-                    uprv_strcat(buffer, l->str);
+                    uprv_strncat(buffer, l->str,1020);
                 }
                 if(l->str[uprv_strlen(l->str)-1] != '"') {
                     uprv_strcat(buffer, "\"");
                 }
             }
-            T_FileStream_write(s, buffer, uprv_strlen(buffer));
+            T_FileStream_write(s, buffer, (int32_t)uprv_strlen(buffer));
+
+            ln += (int32_t)uprv_strlen(l->str);
         }
 
-        ln  += uprv_strlen(l->str);
-
-
         if(l->next && delim)
         {
             if(ln > 60 && brk) {
                 ln  = 0;
-                T_FileStream_write(s, brk, uprv_strlen(brk));
+                T_FileStream_write(s, brk, (int32_t)uprv_strlen(brk));
             }
-            T_FileStream_write(s, delim, uprv_strlen(delim));
+            T_FileStream_write(s, delim, (int32_t)uprv_strlen(delim));
         }
         l = l->next;
     }
@@ -76,7 +77,14 @@ const char *pkg_writeCharList(FileStream *s, CharList *l, const char *delim, int
     {
         if(l->str)
         {
-            uprv_strcpy(buffer, l->str);
+            uprv_strncpy(buffer, l->str, 1023);
+            buffer[1023]=0;
+            if(uprv_strlen(l->str) >= 1023)
+            {
+                fprintf(stderr, "%s:%d: Internal error, line too long (greater than 1023 chars)\n",
+                        __FILE__, __LINE__);
+                exit(0);
+            }
             if(quote < 0) { /* remove quotes */
                 if(buffer[uprv_strlen(buffer)-1] == '"') {
                     buffer[uprv_strlen(buffer)-1] = '\0';
@@ -93,12 +101,12 @@ const char *pkg_writeCharList(FileStream *s, CharList *l, const char *delim, int
                     uprv_strcat(buffer, "\"");
                 }
             }
-            T_FileStream_write(s, buffer, uprv_strlen(buffer));
+            T_FileStream_write(s, buffer, (int32_t)uprv_strlen(buffer));
         }
         
         if(l->next && delim)
         {
-            T_FileStream_write(s, delim, uprv_strlen(delim));
+            T_FileStream_write(s, delim, (int32_t)uprv_strlen(delim));
         }
         l = l->next;
     }
@@ -128,6 +136,12 @@ CharList *pkg_prependToList(CharList *l, const char *str)
 {
   CharList *newList;
   newList = uprv_malloc(sizeof(CharList));
+
+  /* test for NULL */
+  if(newList == NULL) {
+    return NULL;
+  }
+
   newList->str = str;
   newList->next = l;
   return newList;
@@ -182,6 +196,83 @@ CharList *pkg_appendToList(CharList *l, CharList** end, const char *str)
   return l;
 }
 
+char * convertToNativePathSeparators(char *path) {
+#if defined(U_MAKE_IS_NMAKE)
+    char *itr;
+    while ((itr = uprv_strchr(path, U_FILE_ALT_SEP_CHAR))) {
+        *itr = U_FILE_SEP_CHAR;
+    }
+#endif
+    return path;
+}
+
+CharList *pkg_appendUniqueDirToList(CharList *l, CharList** end, const char *strAlias) {
+    char aBuf[1024];
+    char *rPtr; 
+    rPtr = uprv_strrchr(strAlias, U_FILE_SEP_CHAR);
+#if (U_FILE_SEP_CHAR != U_FILE_ALT_SEP_CHAR)
+    {
+        char *aPtr = uprv_strrchr(strAlias, U_FILE_ALT_SEP_CHAR);
+        if(!rPtr || /* regular char wasn't found or.. */
+            (aPtr && (aPtr > rPtr)))
+        { /* alt ptr exists and is to the right of r ptr */
+            rPtr = aPtr; /* may copy NULL which is OK */
+        }
+    }
+#endif
+    if(!rPtr) {
+        return l; /* no dir path */
+    }
+    if((rPtr-strAlias) > (sizeof(aBuf)/sizeof(aBuf[0]))) {
+        fprintf(stderr, "## ERR: Path too long [%d chars]: %s\n", (int)sizeof(aBuf), strAlias);
+        return l;
+    }
+    strncpy(aBuf, strAlias,(rPtr-strAlias));
+    aBuf[rPtr-strAlias]=0;  /* no trailing slash */
+    convertToNativePathSeparators(aBuf);
+
+    if(!pkg_listContains(l, aBuf)) {
+        return pkg_appendToList(l, end, uprv_strdup(aBuf));
+    } else {
+        return l; /* already found */
+    }
+}
+
+#if 0
+static CharList *
+pkg_appendFromStrings(CharList *l, CharList** end, const char *s, int32_t len)
+{
+  CharList *endptr = NULL;
+  const char *p;
+  char *t;
+  const char *targ;
+  if(end == NULL) {
+    end = &endptr;
+  }
+
+  if(len==-1) {
+    len = uprv_strlen(s);
+  }
+  targ = s+len;
+  
+  while(*s && s<targ) {
+    while(s<targ&&isspace(*s)) s++;
+    for(p=s;s<targ&&!isspace(*p);p++);
+    if(p!=s) {
+      t = uprv_malloc(p-s+1);
+      uprv_strncpy(t,s,p-s);
+      t[p-s]=0;
+      l=pkg_appendToList(l,end,t);
+      fprintf(stderr, " P %s\n", t);
+    }
+    s=p;
+  }
+  
+  return l;
+}
+#endif
+
+
 /*
  * Delete list 
  */
